@@ -5,7 +5,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #  
-#  Copyright (C) 1998-2012 OpenLink Software
+#  Copyright (C) 1998-2024 OpenLink Software
 #  
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -71,7 +71,12 @@ cache_file 'vocab'		''	'XHTML Vocabulary'
 fileprefix='openlinksw-sparql/'
 uriprefix='http://www.openlinksw.com/sparql/'
 publishingdate='2006-09-11 01:25'
-cache_file 'virtrdf-data-formats.ttl'	''	'Quad Map memtadata for generic formats of relational-to-RDF data mapping'
+cache_file 'virtrdf-data-formats.ttl'	''	'Quad Map metadata for generic formats of relational-to-RDF data mapping'
+
+fileprefix='openlinksw-sparql/'
+uriprefix='http://www.openlinksw.com/sparql/'
+publishingdate='2015-07-16 13:24'
+cache_file 'virtrdf-quad-storages.ttl'	''	'Quad Map metadata for predefined RDF quad map storages'
 
 echo "/* This file is automatically generated by cache_gen.sh */"
 echo
