@@ -1,10 +1,8 @@
 --
---  $Id$
---
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -19,26 +17,6 @@
 --  with this program; if not, write to the Free Software Foundation, Inc.,
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 --
---
---#IF VER=5
-ALTER TABLE WS.WS.SYS_DAV_COL ADD COL_AUTO_VERSIONING char(1)
-;
-
-ALTER TABLE WS.WS.SYS_DAV_COL ADD COL_FORK INTEGER DEFAULT 0
-;
-
-ALTER TABLE WS.WS.SYS_DAV_RES ADD RES_STATUS VARCHAR
-;
-
-ALTER TABLE WS.WS.SYS_DAV_RES ADD RES_VCR_ID INTEGER
-;
-
-ALTER TABLE WS.WS.SYS_DAV_RES ADD RES_VCR_CO_VERSION INTEGER
-;
-
-ALTER TABLE WS.WS.SYS_DAV_RES ADD RES_VCR_STATE INTEGER -- 0 CIN, -- 1 COUT
-;
---#ENDIF
 
 CREATE TABLE WS.WS.SYS_DAV_RES_VERSION (
   RV_RES_ID INTEGER NOT NULL, -- This is equal to either existing resource ID or to an attic ID
@@ -57,10 +35,6 @@ alter index SYS_DAV_RES_VERSION on WS.WS.SYS_DAV_RES_VERSION partition (RV_RES_I
 ;
 
 
---#IF VER=5
-ALTER TABLE WS.WS.SYS_DAV_RES_VERSION ADD RV_WHO VARCHAR
-;
---#ENDIF
 
 CREATE TABLE WS.WS.SYS_DAV_RES_DIFF (
   RD_RES_ID       INTEGER NOT NULL, -- referenced by WS.WS.SYS_DAV_RES_VERSION (RV_RES_ID),
@@ -136,13 +110,6 @@ CREATE TABLE WS.WS.SYS_DAV_BASELINE_RES (
 alter index SYS_DAV_BASELINE_RES on WS.WS.SYS_DAV_BASELINE_RES partition (BR_RES_ID int)
 ;
 
---#IF VER=5
-ALTER TABLE WS.WS.SYS_DAV_RES_VERSION ADD RV_SIZE INTEGER NOT NULL
-;
-
-UPDATE WS.WS.SYS_DAV_RES_VERSION SET RV_SIZE = 0 WHERE RV_SIZE IS NULL
-;
---#ENDIF
 
 --| When DAV_DELETE_INT calls DET function, authentication and check for lock are passed.
 CREATE TRIGGER "Versioning_DAV_DELETE" BEFORE DELETE ON WS.WS.SYS_DAV_RES REFERENCING OLD AS O
@@ -312,7 +279,7 @@ CREATE TRIGGER "Versioning_DAV_RES_UPDATE" BEFORE UPDATE ON WS.WS.SYS_DAV_RES OR
     "Versioning_AUTO_VERSION_PROP" ((select COL_AUTO_VERSIONING from WS.WS.SYS_DAV_COL where COL_ID = N.RES_COL and COL_AUTO_VERSIONING is not null)),
     DAV_HIDE_ERROR(DAV_PROP_GET_INT (N.RES_ID, 'R', 'DAV:auto-version', 0)));
 
-  if (DAV_HIDE_ERROR (DAV_PROP_GET_INT (N.RES_ID,'R','DAV:checked-out')) is null and _auto_version_type is not null) {
+  if (DAV_HIDE_ERROR (DAV_PROP_GET_INT (N.RES_ID,'R','DAV:checked-out', 0)) is null and _auto_version_type is not null) {
     if ((length (N.RES_CONTENT) < 10000000) and (length (O.RES_CONTENT) < 10000000)) {
       if (cast (N.RES_CONTENT as varchar) = cast (O.RES_CONTENT as varchar)) {
         if (N.RES_NAME <> O.RES_NAME) { -- move
@@ -1032,74 +999,83 @@ create function "Versioning_DAV_DIR_FILTER" (in detcol_id any, in path_parts any
 ;
 
 --| When DAV_PROP_GET_INT or DAV_DIR_LIST_INT calls DET function, authentication is performed before the call.
-create function "Versioning_DAV_SEARCH_ID" (in detcol_id any, in path_parts any, in what char(1)) returns any
+create function "Versioning_DAV_SEARCH_ID" (
+  in detcol_id any,
+  in path_parts any,
+  in what char(1)) returns any
 {
+  -- dbg_obj_princ ('Versioning_DAV_SEARCH_ID (', detcol_id, path_parts, what, ')');
   declare base_path varchar;
+
   base_path := "Versioning_GET_BASE_PATH" (detcol_id);
   if (base_path is null)
     return -20;
-   -- dbg_obj_princ ('Versioning_DAV_SEARCH_ID (', detcol_id, path_parts, what, ')');
-  if ('C' = what) {
-    if ( (length (path_parts) = 3) )
-      return -1;
-    declare base_id int;
-    base_id := DAV_SEARCH_ID (base_path, 'C');
-    if (base_id > 0) {
-      declare _res_id int;
-      _res_id :=  (select RES_ID from WS.WS.SYS_DAV_RES where RES_COL = base_id
-          and RES_NAME = aref (path_parts, 0) );
-       -- dbg_obj_princ ('Versioning_DAV_SEARCH_ID = ', _res_id);
-      if (_res_id is not null) {
-        if (DAV_HIDE_ERROR (DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-in', 0)) is null
-          and DAV_HIDE_ERROR (DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-out', 0)) is null)
+
+  if (('C' = what) and (length (path_parts) <> 3))
+  {
+    declare base_id integer;
+
+    base_id := DB.DBA.DAV_SEARCH_ID (base_path, 'C');
+    if (base_id > 0)
+    {
+      declare _res_id integer;
+
+      _res_id :=  (select RES_ID from WS.WS.SYS_DAV_RES where RES_COL = base_id and RES_NAME = aref (path_parts, 0));
+      if (_res_id is not null)
+      {
+        if (DB.DBA.DAV_HIDE_ERROR (DB.DBA.DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-in', 0)) is null and
+            DB.DBA.DAV_HIDE_ERROR (DB.DBA.DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-out', 0)) is null)
           return -1;
+
         return vector (UNAME'Versioning', cast (detcol_id as integer), _res_id);
       }
     }
-    return -1;
   }
-  else if ('R' = what) {
-    if (length (path_parts) = 2) {
-      declare _res_id, _ver_id, _id int;
-      _res_id := DAV_SEARCH_ID (base_path || path_parts[0], 'R');
-       -- dbg_obj_princ ('res id == ', _res_id);
-      if (_res_id < 0)
-        return -1;
-      if (DAV_HIDE_ERROR (DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-in', 0)) is null
-            and DAV_HIDE_ERROR (DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-out', 0)) is null)
-        return -1;
-      if (path_parts[1] = 'history.xml')
-        _ver_id := -1;
-      else if (path_parts[1] = 'last')
-        _ver_id := -2;
-      else if (path_parts[1] like '%.diff') {
-        _ver_id := atoi (path_parts[1]);
-        return vector (UNAME'Versioning', detcol_id, _res_id, _ver_id, 'diff');
-      }
-      else {
-        _ver_id := cast (path_parts[1] as integer);
-        if (not exists (select 1 from WS.WS.SYS_DAV_RES_VERSION where RV_RES_ID = _res_id and RV_ID = _ver_id))
-          return -1;
-      }
-      return vector (UNAME'Versioning', detcol_id, _res_id, _ver_id);
+  else if (('R' = what) and (length (path_parts) = 2))
+  {
+    declare _res_id, _ver_id, _id integer;
+
+    _res_id := DB.DBA.DAV_SEARCH_ID (base_path || path_parts[0], 'R');
+    if (_res_id < 0)
+      return -1;
+
+    if (DB.DBA.DAV_HIDE_ERROR (DB.DBA.DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-in', 0)) is null and DB.DBA.DAV_HIDE_ERROR (DB.DBA.DAV_PROP_GET_INT (_res_id, 'R', 'DAV:checked-out', 0)) is null)
+      return -1;
+
+    if (path_parts[1] = 'history.xml')
+      _ver_id := -1;
+    else if (path_parts[1] = 'last')
+      _ver_id := -2;
+    else if (path_parts[1] like '%.diff')
+    {
+      _ver_id := atoi (path_parts[1]);
+      return vector (UNAME'Versioning', detcol_id, _res_id, _ver_id, 'diff');
     }
-    return -1;
+    else
+    {
+      _ver_id := atoi (path_parts[1]);
+      if (not exists (select 1 from WS.WS.SYS_DAV_RES_VERSION where RV_RES_ID = _res_id and RV_ID = _ver_id))
+        return -1;
+    }
+    return vector (UNAME'Versioning', detcol_id, _res_id, _ver_id);
   }
-  else
-    return -1;
+  return -1;
 }
 ;
 
 --| When DAV_SEARCH_PATH_INT calls DET function, authentication is performed before the call.
-create function "Versioning_DAV_SEARCH_PATH" (in id any, in what char(1)) returns any
+create function "Versioning_DAV_SEARCH_PATH" (
+  in id any,
+  in what char(1)) returns any
 {
   -- dbg_obj_princ ('Versioning_DAV_SEARCH_PATH (', id, what, ')');
-  if (what = 'C' and isarray (id) and length (id) = 3) {
+  if (what = 'C' and isarray (id) and length (id) = 3)
+  {
     declare base_coll varchar;
+
     base_coll := DAV_PROP_GET_INT (id[1], 'C','virt:Versioning-Collection', 0);
-    if (isinteger (base_coll))
-      return -1;
-    return base_coll || (select RES_NAME from WS.WS.SYS_DAV_RES where RES_ID = id[2]) || '/';
+    if (not isinteger (base_coll))
+      return base_coll || (select RES_NAME from WS.WS.SYS_DAV_RES where RES_ID = id[2]) || '/';
   }
   return -1;
 }
@@ -1358,24 +1334,29 @@ create procedure "Versioning_REMOVE_V_PROPERTIES" (in _path varchar)
 ;
 
 --| API: remove the collection from version control
-create function DAV_REMOVE_VERSIONING_CONTROL_INT (in _main varchar, in _auth varchar, in _pwd varchar) returns integer
+create function DAV_REMOVE_VERSIONING_CONTROL_INT (
+  in _main varchar,
+  in _auth varchar,
+  in _pwd varchar) returns integer
 {
   declare _vvc any;
-  _vvc := DAV_PROP_GET (_main, 'virt:Versioning-History', 'dav', 'dav');
-  if (DAV_HIDE_ERROR (_vvc) is null)
+  declare _col_id integer;
+
+  _col_id := DB.DBA.DAV_SEARCH_ID (_main, 'C');
+  _vvc := DB.DBA.DAV_PROP_GET_INT (_col_id, 'C', 'virt:Versioning-History', 0);
+  if (DB.DBA.DAV_HIDE_ERROR (_vvc) is null)
     return _vvc;
-  declare _col_id int;
-  _col_id := DAV_SEARCH_ID (_main, 'C');
-  update WS.WS.SYS_DAV_COL set COL_AUTO_VERSIONING=NULL where COL_ID = DAV_SEARCH_ID (_main, 'C');
-  for select RES_FULL_PATH from WS.WS.SYS_DAV_RES where RES_COL = _col_id
-  do {
+
+  update WS.WS.SYS_DAV_COL set COL_AUTO_VERSIONING = null where COL_ID = _col_id;
+  for (select RES_FULL_PATH from WS.WS.SYS_DAV_RES where RES_COL = _col_id) do
+  {
     "Versioning_REMOVE_V_PROPERTIES" (RES_FULL_PATH);
   }
 
   connection_set ('Versioning REM PROP', 1);
-  DAV_PROP_REMOVE (_main, 'virt:Versioning-History', _auth, _pwd);
-  DAV_PROP_REMOVE (_vvc, 'virt:Versioning-Collection', _auth, _pwd);
-  DAV_PROP_REMOVE (_vvc, 'virt:Versioning-Attic', _auth, _pwd);
+  DB.DBA.DAV_PROP_REMOVE (_main, 'virt:Versioning-History', _auth, _pwd);
+  DB.DBA.DAV_PROP_REMOVE (_vvc, 'virt:Versioning-Collection', _auth, _pwd);
+  DB.DBA.DAV_PROP_REMOVE (_vvc, 'virt:Versioning-Attic', _auth, _pwd);
   connection_set ('Versioning REM PROP', NULL);
   return 1;
 }
