@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -52,6 +52,7 @@ create procedure dbp_ldd_set_ns_decl ()
     dbp_domain || '/resource/' || dbp_category || ':',		'category-' || dbp_lang,
     dbp_domain || '/resource/' || dbp_template || ':', 		'template-' || dbp_lang,
 
+    'http://dbpedia.org/datatype/',				'dbd',
     'http://dbpedia.org/ontology/',				'dbo',
     'http://dbpedia.org/property/',				'dbp',
     'http://dbpedia.org/resource/',				'dbr',
@@ -376,6 +377,30 @@ create procedure dbp_ldd_type (in gr varchar, in subj varchar, out url varchar,
 }
 ;
 
+create procedure dbp_ldd_thumbnail (in _S any, in _G varchar)
+{
+  declare img, meta, data any;
+  declare best_q, q float;
+  declare lang, langs varchar;
+  declare retr int;
+
+  if (__tag of IRI_ID = __tag (_S))
+    _S := id_to_iri (_S);
+  if (__tag of IRI_ID = __tag (_G))
+    _G := id_to_iri (_G);
+
+  img := null;
+
+  exec ('sparql select ?o where { graph `iri(??)` { `iri(??)` dbo:thumbnail ?o } } limit 1', null, null, vector (_G, _S), vector ('use_cache', 1, 'max_rows', 0), meta, data);
+  if (length (data))
+    {
+	img := sprintf ('%H', data[0][0]);
+    }
+
+  return img;
+}
+;
+
 create procedure dbp_ldd_uri_local_part (in uri varchar)
 {
   declare delim integer;
@@ -426,6 +451,9 @@ dbp_ldd_trunc_uri (in s varchar, in maxlen int := 80)
   declare _s varchar;
   declare _h int;
 
+  if (not isstring(s))
+    return s;
+
   _s := trim(s);
   _s := charset_recode (_s, 'UTF-8', '_WIDE_');
 
@@ -442,11 +470,17 @@ create procedure dbp_ldd_split_url (in uri varchar, out pref varchar, out res va
   declare delim any;
   declare uriSearch, nsPrefix any;
 
+  if (uri is null)
+    return;
+  if (iswidestring (uri))
+    uri := charset_recode (uri, '_WIDE_', 'UTF-8');
+
   delim := -1;
   uriSearch := uri;
   nsPrefix := null;
   if (not length (label))
     label := null;
+
   while (nsPrefix is null and delim <> 0)
     {
       delim := coalesce (strrchr (uriSearch, '/'), 0);
@@ -587,11 +621,11 @@ create procedure dbp_ldd_http_print_l (in p_text any, inout odd_position int, in
 }
 ;
 
-create procedure dbp_ldd_rel_print (in val any, in rel any, in obj any, in flag int := 0, in lang varchar := null, in nofollow int := 0)
+create procedure dbp_ldd_rel_print (in val any, in rel any, in obj any, in flag int := 0, in lang varchar := null, in dt varchar := null, in nofollow int := 0)
 {
   declare delim, delim1, delim2, delim3 integer;
   declare inx int;
-  declare nss, loc, res, nspref, lang_def varchar;
+  declare nss, loc, res, nspref, lang_def, dt_def varchar;
 
   delim1 := coalesce (strrchr (val, '/'), -1);
   delim2 := coalesce (strrchr (val, '#'), -1);
@@ -603,6 +637,8 @@ create procedure dbp_ldd_rel_print (in val any, in rel any, in obj any, in flag
   nss := subseq (val, 0, delim + 1);
   loc := subseq (val, delim + 1);
   res := '';
+  lang_def := '';
+  dt_def := '';
 
   nspref := __xml_get_ns_prefix (nss, 2);
   if (nspref is null)
@@ -612,38 +648,56 @@ create procedure dbp_ldd_rel_print (in val any, in rel any, in obj any, in flag
       nspref := sprintf ('ns%d', inx);
     }
 
-
-  if (nspref is not null and nspref not in ('dbp', 'dbpprop', 'owl'))
-    nss := sprintf (' xmlns:%s="%s"', nspref, nss);
+  if (nspref is not null and nspref not in ( 'dbo', 'dbp', 'dc', 'dct', 'foaf', 'owl', 'prov', 'rdf', 'rdfa', 'rdfs', 'schema', 'skos'))
+    nss := sprintf ('prefix="%s: %s" ', nspref, nss);
   else
     nss := '';
+
   if (flag)
-    loc := sprintf ('property="%s:%s"', nspref, loc);
+    loc := sprintf ('property="%s:%s" ', nspref, loc);
   else if (rel)
-    loc := sprintf ('rel="%s:%s%s"', nspref, loc, case when nofollow = 1 and loc <> 'sameAs' then ' nofollow' else '' end);
+    loc := sprintf ('rel="%s:%s%s" ', nspref, loc, case when nofollow = 1 and loc <> 'sameAs' then ' nofollow' else '' end);
   else
-    loc := sprintf ('rev="%s:%s"', nspref, loc);
-  --if (obj is not null)
-  --  res := sprintf (' resource="%V"', obj);
-  lang_def := '';
-  if (isstring (lang) and lang <> '')
-    lang_def := sprintf (' xml:lang="%s"', lang);
-  return concat (loc, res, nss, lang_def);
+    loc := sprintf ('rev="%s:%s" ', nspref, loc);
+
+  if (obj is not null)
+    res := sprintf ('resource="%V" ', obj);
+
+  if (length (lang))
+    lang_def := sprintf ('lang="%V" ', lang);
+  if (length (dt))
+    dt_def := sprintf ('datatype="%V" ', dt);
+
+  return concat (loc, res, nss, lang_def, dt_def);
 }
 ;
 
 create procedure dbp_ldd_http_print_r (in _object any, in org int := 0, in label varchar, in pred varchar, in rel int, inout acc any)
 {
-   declare lang, rdfs_type, rdfa, visible any;
+   declare lang, rdfs_type, rdfs_type_short, rdfa, visible, p_dt, s_dt any;
 
    if (__tag of rdf_box = __tag (_object))
      {
        __rdf_box_make_complete (_object);
      }
-   lang := DB.DBA.RDF_LANGUAGE_OF_OBJ (_object);
+
+   lang := DB.DBA.RDF_LANGUAGE_OF_OBJ (_object, null);
    visible := dbp_ldd_str_lang_check (lang, acc);
-   rdfs_type := DB.DBA.RDF_DATATYPE_OF_OBJ (_object);
-   rdfa := dbp_ldd_rel_print (id_to_iri (pred), rel, null, 1, lang, 0);
+   rdfs_type := DB.DBA.RDF_DATATYPE_OF_OBJ (_object, null);
+
+   if (__tag of IRI_ID = __tag (rdfs_type))
+     rdfs_type := id_to_iri (rdfs_type);
+
+   p_dt := '';
+   s_dt := '';
+   dbp_ldd_split_url (rdfs_type, p_dt, s_dt);
+
+   if (length(s_dt))
+     rdfs_type_short := sprintf ('%s:%s', p_dt, s_dt);
+   else
+     rdfs_type_short := '';
+
+   rdfa := dbp_ldd_rel_print (id_to_iri (pred), rel, null, 1, lang, rdfs_type_short, 0);
 
    http (sprintf ('\t<li%s><span class="literal">', case visible when 0 then ' style="display:none;"' else '' end));
 
@@ -658,22 +712,22 @@ again:
    else if (__tag (_object) = 189)
      {
        http (sprintf ('<span %s>%d</span>', rdfa, _object));
-       lang := 'xsd:integer';
+       lang := rdfs_type_short;
      }
    else if (__tag (_object) = 190)
      {
        http (sprintf ('<span %s>%f</span>', rdfa, _object));
-       lang := 'xsd:float';
+       lang := rdfs_type_short;
      }
    else if (__tag (_object) = 191)
      {
        http (sprintf ('<span %s>%f</span>', rdfa, _object));
-       lang := 'xsd:double';
+       lang := rdfs_type_short;
      }
    else if (__tag (_object) = 219)
      {
        http (sprintf ('<span %s>%s</span>', rdfa, cast (_object as varchar)));
-       lang := 'xsd:double';
+       lang := rdfs_type_short;
      }
    else if (__tag (_object) = 182)
      {
@@ -681,12 +735,13 @@ again:
        http (sprintf ('<span %s>', rdfa));
        http_value (charset_recode (_object, 'UTF-8', '_WIDE_'));
        http ('</span>');
-       --lang := '';
+       if ((lang is NULL or lang = '') and rdfs_type <> 'http://www.w3.org/2001/XMLSchema#string')
+         lang := rdfs_type_short;
      }
    else if (__tag (_object) = 211)
      {
        http (sprintf ('<span %s>%s</span>', rdfa, datestring (_object)));
-       lang := 'xsd:date';
+       lang := rdfs_type_short;
      }
    else if (__tag (_object) = 243)
      {
@@ -722,7 +777,7 @@ again:
        if (s_t is null and _url not like registry_get('dbp_domain') || '/%')
 	 nofollow := 1;
 
-       rdfa := dbp_ldd_rel_print (id_to_iri (pred), rel, _url, 0, lang, nofollow);
+       rdfa := dbp_ldd_rel_print (id_to_iri (pred), rel, _url, 0, lang, rdfs_type_short, nofollow);
 
        if (s_t is null)
 	 {
@@ -743,7 +798,7 @@ again:
    else if (__tag (_object) = 238)
      {
        http (sprintf ('<span %s>', rdfa));
-       http (st_astext (_object));
+       http_value (st_astext (_object));
        http ('</span>');
      }
    else
