@@ -1,9 +1,9 @@
 /*
- *  $Id: qbe.js,v 1.36.2.13 2011/03/29 11:03:35 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2009 OpenLink Software
+ *  Copyright (C) 2009-2024 OpenLink Software
  *
  *  See LICENSE file for details.
  *
@@ -2153,7 +2153,7 @@ iSPARQL.QBE = function (def_obj) {
 	} else { OAT.Dom.hide("qbe_return_btn");}
 
     } else {
-	if (self.svgsparql) { self.loadFromString(self.defaults.query); }
+	if (self.svgsparql) { self.loadFromString(self.defaults.query); self.svgsparql.reposition();}
 	$('qbe_graph').value = self.defaults.graph;
 	OAT.Dom.hide("qbe_return_btn");
     }
