@@ -6,7 +6,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -250,7 +250,7 @@ create method R2RML_FILL_TRIPLESMAP_METAS_CACHE () returns integer for DB.DBA.R2
       all_metas := vector (null, null);
       if ("q" is not null)
         {
-	  if (__tag ("q") = 246)
+	  if (__tag ("q") = __tag of rdf_box)
 	    "q" := rdf_box_data ("q");
           while (("q" <> '') and strchr (' \t\r\n', chr ("q" [length ("q") - 1])) is not null)
             "q" := "LEFT" ("q", length ("q") - 1);
@@ -261,7 +261,24 @@ create method R2RML_FILL_TRIPLESMAP_METAS_CACHE () returns integer for DB.DBA.R2
         }
       else
         {
-          all_metas[0] := vector ('TABLE', DB.DBA.R2RML_UNQUOTE_NAME (coalesce ("ts", 'DB')), DB.DBA.R2RML_UNQUOTE_NAME (coalesce ("to", 'DBA')), DB.DBA.R2RML_UNQUOTE_NAME ("tn"));
+          declare qual, owner, tbname, tree varchar;
+          if ("ts" is null and "to" is null)
+            {
+               declare exit handler for sqlstate '*' {
+                 signal ('R2RML', 'Invalid tableName');
+               };
+               tree := sql_parse (sprintf ('%s ()', "tn"));
+               tbname := complete_table_name (tree[1], 1);
+               qual := name_part (tbname, 0); owner := name_part (tbname, 1); tbname := name_part (tbname, 2);
+               all_metas[0] := vector ('TABLE', qual, owner, tbname);
+            }
+          else
+            {
+              all_metas[0] := vector ('TABLE',
+                    DB.DBA.R2RML_UNQUOTE_NAME (coalesce ("ts", 'DB')),
+                    DB.DBA.R2RML_UNQUOTE_NAME (coalesce ("to", 'DBA')),
+                    DB.DBA.R2RML_UNQUOTE_NAME ("tn"));
+            }
           text_to_prepare := sprintf ('select * from "%I"."%I"."%I"', all_metas[0][1], all_metas[0][2], all_metas[0][3]);
         }
       stat := '00000';
@@ -346,7 +363,7 @@ create method R2RML_GET_COL_DESC (in triplesmap_iid IRI_ID, in col_name varchar)
   all_metas := dict_get (self.triplesmap_metas_cache, triplesmap_iid, null);
   if (dict_get (self.triplesmap_metas_cache, triplesmap_iid, NULL) is null)
     signal ('R2RML', 'Undeclared data source <' || id_to_iri (triplesmap_iid) || '>');
-  res := get_keyword (col_name, all_metas[1], null);
+  res := get_keyword_ucase (col_name, all_metas[1], null);
   if (res is null)
     signal ('R2RML', 'Data source <' || id_to_iri (triplesmap_iid) || '> does not produce column "' || col_name || '"');
   res := vector (triplesmap_iid, all_metas[0], res);
@@ -450,8 +467,8 @@ create function DB.DBA.R2RML_XSD_TYPE_OF_DTP (in dtp integer)
   if (__tag of double precision = dtp) return 'http://www.w3.org/2001/XMLSchema#double';
   if (__tag of numeric = dtp) return 'http://www.w3.org/2001/XMLSchema#double';
   if (__tag of real = dtp) return 'http://www.w3.org/2001/XMLSchema#float';
-  if (230) return 'http://www.w3.org/2001/XMLSchema#XMLLiteral';
-  if (238) return 'http://www.openlinksw.com/schemas/virtrdf#Geometry';
+  if (__tag of XML) return 'http://www.w3.org/2001/XMLSchema#XMLLiteral';
+  if (238) return default_geo_type();
   return 'http://www.w3.org/2001/XMLSchema#any';
 }
 ;
@@ -522,7 +539,7 @@ create method R2RML_GEN_CREATE_IOL_CLASS_OR_REF (in fld_idx integer, in mode int
         when (coltype[1] in (__tag of integer, __tag of smallint)) then '%d'
         when (coltype[1] in (__tag of bigint)) then '%ld'
         when (coltype[1] in (__tag of real, __tag of double precision, __tag of numeric)) then '%g'
-        when (coltype[1] in (__tag of varchar, __tag of nvarchar)) then
+        when (coltype[1] in (__tag of varchar, __tag of nvarchar, __tag of long varchar, __tag of long nvarchar)) then
           case (termtype) when 'http://www.w3.org/ns/r2rml#Literal' then '%s' else '%U' end
         else
           signal ('R2RML',
@@ -569,6 +586,8 @@ create_iol_class:
               when __tag of numeric then 'numeric'
               when __tag of varchar then 'varchar'
               when __tag of nvarchar then 'nvarchar'
+              when __tag of long varchar then 'long varchar'
+              when __tag of long nvarchar then 'long nvarchar'
               else 'any' end ||
             case (argtypes[argctr][0]) when 0 then ' not null' else '' end,
             self.codegen_ses );
@@ -627,7 +646,7 @@ create method R2RML_GEN_FLD (in fld_idx integer, in constfld any, in triplesmap_
           declare col_desc any;
           col_desc := self.R2RML_GET_COL_DESC (triplesmap_iid, col);
           http (self.R2RML_REGISTER_USE_OF_TMAP (fld_idx, col_desc[0]), self.codegen_ses);
-          http (sprintf ('."%I"', col), self.codegen_ses);
+          http (sprintf ('."%I"', col_desc[2][0]), self.codegen_ses);
         }
     }
   else if (constfld is not null)
@@ -640,10 +659,8 @@ create method R2RML_GEN_FLD (in fld_idx integer, in constfld any, in triplesmap_
 
 create method R2RML_MAKE_QM_IMPL_IOL_CLASSES () returns any for DB.DBA.R2RML_MAP
 {
-  foreach (varchar dflttt in vector ('http://www.w3.org/ns/r2rml#IRI', 'http://www.w3.org/ns/r2rml#Literal')) do
-    {
       for (sparql define input:storage "" define output:valmode "LONG"
-        select ?triplesmap ?fldmap ?template ?termtype ?dt ?lang
+        select ?triplesmap ?fldmap ?template ?termtype ?col ?dt ?lang
         where { graph `iri(?:self.graph_iid)` {
                 ?triplesmap a rr:TriplesMap .
                   { ?triplesmap rr:subjectMap [ rr:graphMap ?fldmap ] }
@@ -659,13 +676,22 @@ create method R2RML_MAKE_QM_IMPL_IOL_CLASSES () returns any for DB.DBA.R2RML_MAP
                 optional { ?fldmap rr:termType ?termtype . }
                 optional { ?fldmap rr:datatype ?dt . }
                 optional { ?fldmap rr:language ?lang . }
+                optional { ?fldmap rr:column ?col . }
               } }
         order by asc(str(?template)) asc(str(?dt)) asc(str(?lang)) asc(str(?triplesmap)) asc(str(?fldmap))
         ) do
         {
-          self.R2RML_GEN_CREATE_IOL_CLASS_OR_REF (-1, 1, "triplesmap", __ro2sq ("template"), coalesce (__id2i ("termtype"), dflttt), "dt", __ro2sq ("lang"));
+          declare term_type varchar;
+          term_type := __id2i ("termtype");
+          if (term_type is null)
+            {
+              if ("col" is null and "dt" is null and "lang" is null)
+                term_type := 'http://www.w3.org/ns/r2rml#IRI';
+              else
+                term_type := 'http://www.w3.org/ns/r2rml#Literal';
+            }
+          self.R2RML_GEN_CREATE_IOL_CLASS_OR_REF (-1, 1, "triplesmap", __ro2sq ("template"), term_type, "dt", __ro2sq ("lang"));
         }
-    }
   for (sparql define input:storage "" define output:valmode "LONG"
     select ?triplesmap ?fldmap ?col ?termtype
     where { graph `iri(?:self.graph_iid)` {
@@ -681,7 +707,8 @@ create method R2RML_MAKE_QM_IMPL_IOL_CLASSES () returns any for DB.DBA.R2RML_MAP
             optional { ?fldmap rr:termType ?termtype . }
           } } ) do
     {
-      self.R2RML_GEN_CREATE_IOL_CLASS_OR_REF (-1, 1, "triplesmap", '{' || DB.DBA.R2RML_UNQUOTE_NAME (__ro2sq ("col")) || '}', coalesce (__id2i ("termtype"), 'http://www.w3.org/ns/r2rml#IRI'), null, null);
+      self.R2RML_GEN_CREATE_IOL_CLASS_OR_REF (-1, 1, "triplesmap", '{' || DB.DBA.R2RML_UNQUOTE_NAME (__ro2sq ("col")) || '}',
+        coalesce (__id2i ("termtype"), 'http://www.w3.org/ns/r2rml#IRI'), null, null);
     }
   for (sparql define input:storage "" define output:valmode "LONG"
     select ?triplesmap ?fldmap ?col ?termtype ?dt ?lang
@@ -703,7 +730,7 @@ create method R2RML_MAKE_QM_IMPL_REL_PO (in tmap IRI_ID, in tmap2 IRI_ID, in tma
 {
   declare p_md5 varchar;
   declare where_is_opened integer;
-  -- dbg_obj_princ ('R2RML_MAKE_QM: cross from ', "tmap", ' to ', "tmap2" );
+  -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_REL_PO: cross from ', "tmap", ' to ', "tmap2" );
   for (sparql define input:storage "" define output:valmode "LONG"
     select ?constp, ?consto, ?ocol, ?otmpl, ?ott
     where { graph `iri(?:self.graph_iid)` {
@@ -721,7 +748,7 @@ create method R2RML_MAKE_QM_IMPL_REL_PO (in tmap IRI_ID, in tmap2 IRI_ID, in tma
           } }
     order by 1 2 3 4 5) do
     {
-      p_md5 := md5_box (vector (__rdf_strsqlval ("constp"), null, null));
+      p_md5 := md5_box (vector (tmap, __rdf_strsqlval ("constp"), null, null));
       if (self.prev_p_md5 is null or self.prev_p_md5 <> p_md5)
         {
           if (self.prev_p_md5 is not null)
@@ -794,7 +821,7 @@ create method R2RML_MAKE_QM_IMPL_PLAIN_PO (in tmap IRI_ID, in pofld IRI_ID, in p
 {
   declare p_md5 varchar;
   for (sparql define input:storage "" define output:valmode "LONG"
-    select ?constp, ?pcol, ?ptmpl, ?consto, ?ocol, ?otmpl, ?ott, ?odatatype, ?lang
+    select ?constp, ?pcol, ?ptmpl, ?consto, ?ocol, ?otmpl, ?ott, ?odatatype, ?olang
     where { graph `iri(?:self.graph_iid)` {
               { `iri(?:pofld)` rr:predicate ?constp . filter (?constp = iri(?:pconst)) }
             union
@@ -816,7 +843,7 @@ create method R2RML_MAKE_QM_IMPL_PLAIN_PO (in tmap IRI_ID, in pofld IRI_ID, in p
           } }
     order by 1 2 3 4 5 6 7 8 9) do
     {
-      p_md5 := md5_box (vector (__rdf_strsqlval("constp"), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("pcol")), __rdf_strsqlval("ptmpl")));
+      p_md5 := md5_box (vector (tmap, __rdf_strsqlval("constp"), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("pcol")), __rdf_strsqlval("ptmpl")));
       if (self.prev_p_md5 is null or self.prev_p_md5 <> p_md5)
         {
           if (self.prev_p_md5 is not null)
@@ -831,70 +858,184 @@ create method R2RML_MAKE_QM_IMPL_PLAIN_PO (in tmap IRI_ID, in pofld IRI_ID, in p
         }
       else
           http (',\n                            ', self.codegen_ses);
-      self.R2RML_GEN_FLD (3 /* for O */, "consto", tmap, DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("ocol")), __rdf_strsqlval("otmpl"), coalesce (__rdf_strsqlval("ott"), 'http://www.w3.org/ns/r2rml#Literal'), "odatatype", __rdf_strsqlval("lang"));
+      declare term_type varchar;
+      term_type := __rdf_strsqlval ("ott");
+      if (term_type is null)
+        {
+          if ("ocol" is null and "odatatype" is null and "olang" is null)
+            term_type := 'http://www.w3.org/ns/r2rml#IRI';
+          else
+            term_type := 'http://www.w3.org/ns/r2rml#Literal';
+        }
+      self.R2RML_GEN_FLD (3 /* for O */, "consto", tmap, DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("ocol")), __rdf_strsqlval("otmpl"),
+        term_type, "odatatype", __rdf_strsqlval("olang"));
     }
 }
 ;
 
 create method R2RML_MAKE_QM_IMPL_CHILDS (in needs_inner_g_field integer) returns any for DB.DBA.R2RML_MAP
 {
-  declare prev_g_md5, prev_s_md5 any;
+  declare prev_g_md5, prev_s_md5, childs any;
+  -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_CHILDS(', needs_inner_g_field, '), graph_iid is ', self.graph_iid);
   -- For each combination of mapclasses and graph
   prev_g_md5 := prev_s_md5 := null;
   self.prev_p_md5 := null;
+
+-- There was this query here but it fails for unknown reason.
+--  for (sparql define input:storage "" define output:valmode "LONG" define 
+--    select ?constg, ?gcol, ?gtmpl, ?tmap, ?sfld, ?consts, ?scol, ?stmpl, ?stt, ?sclass, ?pofld, ?pconst, ?pfld, ?oconst, ?ofld, ?tmap2, ?tmap2sfld
+--    where { graph `iri(?:self.graph_iid)` {
+--            ?tmap a rr:TriplesMap .
+--              { ?tmap rr:subject ?consts }
+--            union
+--              {
+--                ?tmap rr:subjectMap ?sfld .
+--                  { ?sfld rr:constant ?consts }
+--                union
+--                  { ?sfld rr:column ?scol }
+--                union
+--                  { ?sfld rr:template ?stmpl }
+--                optional { ?sfld rr:termType ?stt }
+--              }
+--              {
+--                ?sfld rr:class ?sclass .
+--              }
+--            union
+--              {
+--                ?tmap rr:predicateObjectMap ?pofld .
+--                  { ?pofld rr:predicate ?pconst }
+--                union
+--                  { ?pofld rr:predicateMap ?pfld }
+--                  { ?pofld rr:object ?oconst }
+--                union
+--                  { ?pofld rr:objectMap ?ofld
+--                    optional {
+--                        ?ofld rr:parentTriplesMap ?tmap2 .
+--                        ?tmap2 a rr:TriplesMap ;
+--                          rr:subjectMap ?tmap2sfld . }
+--                  }
+--              }
+--            optional {
+--                  { ?gcontainer rr:graph ?constg . }
+--                union
+--                  {
+--                    ?gcontainer rr:graphMap ?gfld .
+--                      { ?gfld rr:constant ?constg }
+--                    union
+--                      { ?gfld rr:column ?gcol }
+--                    union
+--                      { ?gfld rr:template ?gtmpl } } }
+--              filter (?gcontainer in (?sfld, ?pofld) || !(bound(?gcontainer)))
+--          } }
+--    order by 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16) do
+--    {
+
+
+  vectorbld_init (childs);
   for (sparql define input:storage "" define output:valmode "LONG"
-    select ?constg, ?gcol, ?gtmpl, ?tmap, ?sfld, ?consts, ?scol, ?stmpl, ?stt, ?sclass, ?pofld, ?pconst, ?pfld, ?oconst, ?ofld, ?tmap2, ?tmap2sfld
+    select ?tmap
     where { graph `iri(?:self.graph_iid)` {
-            ?tmap a rr:TriplesMap .
-              { ?tmap rr:subject ?consts }
-            union
-              {
-                ?tmap rr:subjectMap ?sfld .
-                  { ?sfld rr:constant ?consts }
-                union
-                  { ?sfld rr:column ?scol }
-                union
-                  { ?sfld rr:template ?stmpl }
-                optional { ?sfld rr:termType ?stt }
-              }
-              {
-                ?sfld rr:class ?sclass .
-              }
-            union
-              {
-                ?tmap rr:predicateObjectMap ?pofld .
-                  { ?pofld rr:predicate ?pconst }
-                union
-                  { ?pofld rr:predicateMap ?pfld }
-                  { ?pofld rr:object ?oconst }
-                union
-                  { ?pofld rr:objectMap ?ofld
-                    optional {
-                        ?ofld rr:parentTriplesMap ?tmap2 .
-                        ?tmap2 a rr:TriplesMap ;
-                          rr:subjectMap ?tmap2sfld . }
-                  }
-              }
-            optional {
-                  { ?gcontainer rr:graph ?constg . }
+            ?tmap a rr:TriplesMap . } } ) do
+    {
+      -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_CHILDS: tmap woudl be ', "tmap");
+      for (sparql define input:storage "" define output:valmode "LONG"
+        select ?sfld, ?consts, ?scol, ?stmpl, ?stt
+        where { graph `iri(?:self.graph_iid)` {
+                  { ?:"tmap" rr:subject ?consts }
                 union
                   {
-                    ?gcontainer rr:graphMap ?gfld .
-                      { ?gfld rr:constant ?constg }
+                    ?:"tmap" rr:subjectMap ?sfld .
+                      { ?sfld rr:constant ?consts }
                     union
-                      { ?gfld rr:column ?gcol }
+                      { ?sfld rr:column ?scol }
                     union
-                      { ?gfld rr:template ?gtmpl } }
-                filter (?gcontainer in (?sfld, ?pofld)) }
-          } }
-    order by 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16) do
+                      { ?sfld rr:template ?stmpl }
+                    optional { ?sfld rr:termType ?stt }
+                  } } } ) do
+        {
+          -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_CHILDS: tmap would be ', "tmap", '; s is ', "sfld", "consts", "scol", "stmpl", "stt");
+          for (sparql define input:storage "" define output:valmode "LONG"
+            select ?sclass, ?pofld, ?pconst, ?pfld, ?oconst, ?ofld, ?tmap2, ?tmap2sfld, ?constg, ?gcol, ?gtmpl
+            where { graph `iri(?:self.graph_iid)` {
+                      {
+                        ?:"sfld" rr:class ?sclass .
+                      }
+                    union
+                      {
+                        ?:"tmap" rr:predicateObjectMap ?pofld .
+                          { ?pofld rr:predicate ?pconst }
+                        union
+                          { ?pofld rr:predicateMap ?pfld }
+                          { ?pofld rr:object ?oconst }
+                        union
+                          { ?pofld rr:objectMap ?ofld
+                            optional {
+                                ?ofld rr:parentTriplesMap ?tmap2 .
+                                ?tmap2 a rr:TriplesMap ;
+                                  rr:subjectMap ?tmap2sfld . }
+                          }
+                      }
+                    optional {
+                          { ?gcontainer rr:graph ?constg . }
+                        union
+                          {
+                            ?gcontainer rr:graphMap ?gfld .
+                              { ?gfld rr:constant ?constg }
+                            union
+                              { ?gfld rr:column ?gcol }
+                            union
+                              { ?gfld rr:template ?gtmpl } } }
+                      filter (?gcontainer in (?:"sfld", ?pofld) || !(bound(?gcontainer)))
+                  } } ) do
+            {
+              -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_CHILDS: g would be ', "constg", "gcol", "gtmpl", '; tmap is ', "tmap", '; s is ', "sfld", "consts", "scol", "stmpl", "stt", "sclass", '; po is ', "pconst", "pfld", ' and ', "oconst", "ofld");
+              vectorbld_acc (childs, vector ("constg", "gcol", "gtmpl", "tmap", "sfld", "consts", "scol", "stmpl", "stt", "sclass", "pofld", "pconst", "pfld", "oconst", "ofld", "tmap2", "tmap2sfld"));
+            }
+        }
+    }
+  vectorbld_final (childs);
+  rowvector_obj_sort (childs, 15, 1);
+  rowvector_obj_sort (childs, 14, 1);
+  rowvector_obj_sort (childs, 13, 1);
+  rowvector_obj_sort (childs, 12, 1);
+  rowvector_obj_sort (childs, 11, 1);
+  rowvector_obj_sort (childs, 10, 1);
+  rowvector_obj_sort (childs, 9, 1);
+  rowvector_obj_sort (childs, 8, 1);
+  rowvector_obj_sort (childs, 7, 1);
+  rowvector_obj_sort (childs, 6, 1);
+  rowvector_obj_sort (childs, 5, 1);
+  rowvector_obj_sort (childs, 4, 1);
+  rowvector_obj_sort (childs, 3, 1);
+  rowvector_obj_sort (childs, 2, 1);
+  rowvector_obj_sort (childs, 1, 1);
+  rowvector_obj_sort (childs, 0, 1);
+  foreach (any tmap_case in childs) do
     {
+      declare "constg", "gcol", "gtmpl", "tmap", "sfld", "consts", "scol", "stmpl", "stt", "sclass", "pofld", "pconst", "pfld", "oconst", "ofld", "tmap2", "tmap2sfld" any;
       declare s_md5, p_md5 varchar;
-      -- dbg_obj_princ ('R2RML_MAKE_QM: g is ', "constg", "gcol", "gtmpl", '; tmap is ', "tmap", '; s is ', "sfld", "consts", "scol", "stmpl", "stt", "sclass", '; po is ', "pconst", "pfld", ' and ', "oconst", "ofld");
+      "constg" := tmap_case[0];
+      "gcol" := tmap_case[1];
+      "gtmpl" := tmap_case[2];
+      "tmap" := tmap_case[3];
+      "sfld" := tmap_case[4];
+      "consts" := tmap_case[5];
+      "scol" := tmap_case[6];
+      "stmpl" := tmap_case[7];
+      "stt" := tmap_case[8];
+      "sclass" := tmap_case[9];
+      "pofld" := tmap_case[10];
+      "pconst" := tmap_case[11];
+      "pfld" := tmap_case[12];
+      "oconst" := tmap_case[13];
+      "ofld" := tmap_case[14];
+      "tmap2" := tmap_case[15];
+      "tmap2sfld" := tmap_case[16];
+      -- dbg_obj_princ ('R2RML_MAKE_QM_IMPL_CHILDS: g is ', "constg", "gcol", "gtmpl", '; tmap is ', "tmap", '; s is ', "sfld", "consts", "scol", "stmpl", "stt", "sclass", '; po is ', "pconst", "pfld", ' and ', "oconst", "ofld");
       if (needs_inner_g_field)
         {
           declare g_md5 any;
-          g_md5 := md5_box (vector (coalesce ("constg", self.default_constg), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("gcol")), __rdf_strsqlval("gtmpl")));
+          g_md5 := md5_box (vector ("tmap", coalesce ("constg", self.default_constg), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("gcol")), __rdf_strsqlval("gtmpl")));
           if (prev_g_md5 is null or prev_g_md5 <> g_md5)
             {
               if (prev_g_md5 is not null)
@@ -912,7 +1053,7 @@ create method R2RML_MAKE_QM_IMPL_CHILDS (in needs_inner_g_field integer) returns
               prev_g_md5 := g_md5;
             }
         }
-      s_md5 := md5_box (vector (__rdf_strsqlval("consts"), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("scol")), __rdf_strsqlval("stmpl"), __rdf_strsqlval("stt")));
+      s_md5 := md5_box (vector ("tmap", __rdf_strsqlval("consts"), DB.DBA.R2RML_UNQUOTE_NAME (__rdf_strsqlval("scol")), __rdf_strsqlval("stmpl"), __rdf_strsqlval("stt")));
       if (prev_s_md5 is null or prev_s_md5 <> s_md5)
         {
           if (prev_s_md5 is not null)
@@ -930,7 +1071,7 @@ create method R2RML_MAKE_QM_IMPL_CHILDS (in needs_inner_g_field integer) returns
         }
       if ("sclass" is not null)
         {
-          p_md5 := 'rdf:type';
+          p_md5 := md5_box (vector ("tmap", __bft ('http://www.w3.org/1999/02/22-rdf-syntax-ns#type', 1), null, null));
           if (self.prev_p_md5 is null or self.prev_p_md5 <> p_md5)
             {
               if (self.prev_p_md5 is not null)
@@ -1020,7 +1161,7 @@ create method R2RML_MAKE_QM (in storage_iid IRI_ID := null, in rdfview_iid IRI_I
               { ?gfld rr:column ?c }
             union { ?gfld rr:template ?t }
             filter (?gcontainer in (?smap, ?pomap)) } } );
---  -- dbg_obj_princ ('const_graph_count = ', const_graph_count, ', var_graph_count = ', var_graph_count);
+  -- dbg_obj_princ ('const_graph_count = ', const_graph_count, ', var_graph_count = ', var_graph_count);
   http ('alter quad storage ' || self.R2RML_IRI_ID_AS_QNAME (storage_iid) || '\n', self.codegen_ses);
   iter := self.triplesmap_metas_cache;
   for (dict_iter_rewind (iter); dict_iter_next (iter, iter_tmap, iter_metas); )
@@ -1035,7 +1176,7 @@ create method R2RML_MAKE_QM (in storage_iid IRI_ID := null, in rdfview_iid IRI_I
   http ('  {\n', self.codegen_ses);
   self.used_fld_tmap_aliases := vector (vector(), vector(), vector(), vector(), vector());
   http ('    create ' || self.R2RML_IRI_ID_AS_QNAME (rdfview_iid) || ' as', self.codegen_ses);
-  if (self.default_constg is null) 
+  if (self.default_constg is null)
     self.default_constg := iri_to_id (sprintf ('http://example.com/r2rml?graph=%U', id_to_iri(self.graph_iid)));
   if ((const_graph_count + var_graph_count) <= 1)
     {
@@ -1044,19 +1185,18 @@ create method R2RML_MAKE_QM (in storage_iid IRI_ID := null, in rdfview_iid IRI_I
         {
           declare constg IRI_ID;
           constg := (sparql define input:storage "" define output:valmode "LONG"
-            select ?constg
-            where { graph `iri(?:self.graph_iid)` {
-                    ?tmap a rr:TriplesMap .
-                    optional {
-                        { ?tmap rr:subjectMap ?smap }
-                      union
-                        { ?tmap rr:predicateObjectMap ?pomap } }
-                      { ?gcontainer rr:graph ?constg }
-                    union
-                      {
-                        ?gcontainer rr:graphMap ?gfld .
-                        ?gfld rr:constant ?constg }
-                    filter (?gcontainer in (?smap, ?pomap)) } } );
+          SELECT  ?constg WHERE
+          { GRAPH `iri(?:self.graph_iid)`
+              { ?tmap  a  rr:TriplesMap
+                  { ?tmap  rr:subjectMap [ rr:graph  ?constg ] . }
+                  UNION
+                  { ?tmap  rr:predicateObjectMap [ rr:graph  ?constg ] . }
+                  UNION
+                  { ?tmap  rr:subjectMap [ rr:graphMap  [ rr:constant  ?constg ] ] . }
+                  UNION
+                  { ?tmap  rr:predicateObjectMap [ rr:graphMap  [ rr:constant  ?constg ] ] . }
+              }
+          });
           if (constg is null)
             constg := self.default_constg;
           self.R2RML_GEN_FLD (0 /* for G */, constg, null, null, null, 'http://www.w3.org/ns/r2rml#IRI', null, null);
@@ -1090,12 +1230,18 @@ create method R2RML_MAKE_QM (in storage_iid IRI_ID := null, in rdfview_iid IRI_I
 }
 ;
 
-create function R2RML_MAKE_QM_FROM_G (in g varchar, in tgt_graph varchar := null) returns varchar
+create function R2RML_MAKE_QM_FROM_G (in g varchar, in tgt_graph varchar := null, in qm_uri varchar := null) returns varchar
 {
   declare m R2RML_MAP;
+  declare qm_iid IRI_ID;
+  qm_iid := null;
   m := DB.DBA.R2RML_MAP (iri_to_id (g));
-  m.default_constg := iri_to_id (tgt_graph);
-  m.R2RML_MAKE_QM (null, null);
+  if (tgt_graph is not null)
+    m.default_constg := iri_to_id (tgt_graph);
+  if (qm_uri is null)
+    qm_uri := concat ('urn:qm:', bin2hex(xenc_digest(tgt_graph,'sha1')));
+  qm_iid := iri_to_id(qm_uri);
+  m.R2RML_MAKE_QM (null, qm_iid);
   return string_output_string (m.codegen_ses);
 }
 ;
@@ -1145,7 +1291,7 @@ create procedure R2RML_GENERATE_LINKED_VIEW (in source varchar, in destination_g
     vgraph := sprintf ('http://example.com/r2rml?graph=%U', source);
   if (graph_type = 1)
     {
-      if (exists (sparql define input:storage "" prefix rr: <http://www.w3.org/ns/r2rml#> ask { graph `iri(?:source)` { [] rr:graphMap ?m . ?m rr:template ?g }}))
+      if ((sparql define input:storage "" prefix rr: <http://www.w3.org/ns/r2rml#> ask { graph `iri(?:source)` { [] rr:graphMap ?m . ?m rr:template ?g }}))
 	signal ('42000', 'Can not sync graph template to physical graph');
     }
   vstr := DB.DBA.R2RML_MAKE_QM_FROM_G (source, vgraph);
