@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *  
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *  
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -501,11 +501,12 @@ class ScrollDemo2 extends Frame implements Runnable
     textStatus.reshape(0,62,600,30);
     textStatus.setFont(new Font("Dialog", Font.PLAIN, 10));
     panel4.add(textStatus);
-    setTitle("OpenLink Virutoso JDBC 2.0 Scrollable Cursor Demo");
+    setTitle("OpenLink JDBC 2.0 Scrollable Cursor Demo");
 
     Info.remove(label1);
     Info.remove(textField1);
-    loadDriver("virtuoso.jdbc2.Driver");
+
+    loadDriver("virtuoso.jdbc3.Driver");
   }
 
   public void show()
