@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -63,3 +63,12 @@
 #include <virtext.h>
 /*#define SQL_ENCRYPT_CONNECTION 5004
 #define SQL_SHUTDOWN_ON_CONNECT 5005*/
+
+#ifndef SQL_ATTR_APP_UNICODE_TYPE
+#define SQL_ATTR_APP_UNICODE_TYPE             1064
+#define SQL_ATTR_DRIVER_UNICODE_TYPE          1065
+
+#define SQL_DM_CP_UTF16                 1
+#define SQL_DM_CP_UTF8                  2
+#define SQL_DM_CP_UCS4                  3
+#endif
