@@ -1,10 +1,10 @@
 --
---  $Id$
+--  $Id: nwxmlb.sql,v 1.2.6.1 2013/01/02 16:14:47 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2013 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -107,36 +107,36 @@ xml_view_doc ('ord', 'ord1');
 
 select count (n) from XML_TEXT where xpath_contains (XT_TEXT, '/*/*', n);
 
-ECHO BOTH $IF $EQU $LAST[1] 107 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " top level entities\n";
+echo both $if $equ $last[1] 107 "PASSED" "*** FAILED";
+echo both ": " $last[1] " top level entities\n";
 
 explain ('select count (n) from XML_TEXT where xpath_contains (XT_TEXT, ''/*/*'', n)',  2);
-ECHO BOTH $IF $EQU $STATE OK  "PASSED" "***FAILED";
-ECHO BOTH ": BUG 5796: scrollable cursor over xpath_contains STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
+echo both $if $equ $state OK  "PASSED" "*** FAILED";
+echo both ": BUG 5796: scrollable cursor over xpath_contains STATE=" $state " MESSAGE=" $message "\n";
 
 
 
 select count (n) from XML_TEXT where xpath_contains (XT_TEXT, '/document/category/@description', n);
-ECHO BOTH $IF $EQU $LAST[1] 16 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " top level category entities\n";
+echo both $if $equ $last[1] 16 "PASSED" "*** FAILED";
+echo both ": " $last[1] " top level category entities\n";
 
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '/document/category/product/@ProductName', n);
-ECHO BOTH $IF $EQU $ROWCNT 154 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT  " products under categories \n";
+echo both $if $equ $rowcnt 154 "PASSED" "*** FAILED";
+echo both ": " $rowcnt  " products under categories \n";
 
 select count(n) from XML_TEXT where xpath_contains (XT_TEXT, '//product', n);
-ECHO BOTH $IF $EQU $LAST[1] 2309  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " products under //  \n";
+echo both $if $equ $last[1] 2309  "PASSED" "*** FAILED";
+echo both ": " $last[1] " products under //  \n";
 
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '/document/category [@description like ''%Bread%''] /product/@ProductName', n) and XT_FILE = 'cat1';
-ECHO BOTH $IF $EQU $ROWCNT 7  "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " products under bread category in cat1  \n";
+echo both $if $equ $rowcnt 7  "PASSED" "*** FAILED";
+echo both ": " $rowcnt " products under bread category in cat1  \n";
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '//product[../@description like ''Sea%'']', n);
-ECHO BOTH $IF $EQU $ROWCNT 24 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT  " products under seafood parent  \n";
+echo both $if $equ $rowcnt 24 "PASSED" "*** FAILED";
+echo both ": " $rowcnt  " products under seafood parent  \n";
 
 
 
@@ -145,17 +145,17 @@ select n from XML_TEXT where xpath_contains (XT_TEXT, '/*/category', n) and XT_F
 
 -- compares BLOB colum
 --XPATH [__view 'cat'] //product[../@description like 'Sea%' ];
---ECHO BOTH $IF $EQU $ROWCNT 12 "PASSED" "***FAILED";
---ECHO BOTH ": " $ROWCNT  " products in cat view  under seafood parent  \n";
+--echo both $if $equ $rowcnt 12 "PASSED" "*** FAILED";
+--echo both ": " $rowcnt  " products in cat view  under seafood parent  \n";
 
 
 select count (*) from (XPATH '[__key __view "cat"] //*') n;
-ECHO BOTH $IF $EQU $LAST[1] 85  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in //* in view cat \n";
+echo both $if $equ $last[1] 85  "PASSED" "*** FAILED";
+echo both ": " $last[1] " in //* in view cat \n";
 
 select count (*) from (XPATH '[ __view "ord"] //*') n;
-ECHO BOTH $IF $EQU $LAST[1] 5231  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in //* in view ord \n";
+echo both $if $equ $last[1] 5231  "PASSED" "*** FAILED";
+echo both ": " $last[1] " in //* in view ord \n";
 
 
 
@@ -164,25 +164,25 @@ ECHO BOTH ": " $LAST[1] " in //* in view ord \n";
 XPATH [__view 'cat'] //product[@ProductName like '%' ];
 
 XPATH [__view 'cat'] /category/@description;
-ECHO BOTH $IF $EQU $ROWCNT 8 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT  " categories in cat view\n";
+echo both $if $equ $rowcnt 8 "PASSED" "*** FAILED";
+echo both ": " $rowcnt  " categories in cat view\n";
 
 
 
 --select count (*) from (XPATH '[__* __doc "ord1"] //product') n;
 select count (n) from XML_TEXT where xpath_contains (XT_TEXT, '//product', n) and XT_FILE = 'ord1';
-ECHO BOTH $IF $EQU $LAST[1] 2155  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in //* in view ord \n";
+echo both $if $equ $last[1] 2155  "PASSED" "*** FAILED";
+echo both ": " $last[1] " in //* in view ord \n";
 
 
 select count (*) from (XPATH '[__* __view "ord"] //product') n;
-ECHO BOTH $IF $EQU $LAST[1] 2155  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in //* in view ord \n";
+echo both $if $equ $last[1] 2155  "PASSED" "*** FAILED";
+echo both ": " $last[1] " in //* in view ord \n";
 
 
 XPATH [__key __view 'cat'] //product[@ProductName like '%nla%' ]/@ProductName;
-ECHO BOTH $IF $EQU $LAST[1] "Inlagd Sill"  "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " is the name of %nla% in cat view\n";
+echo both $if $equ $last[1] "Inlagd Sill"  "PASSED" "*** FAILED";
+echo both ": " $last[1] " is the name of %nla% in cat view\n";
 
 
 XPATH [__view 'cat'] /category[@description like 'Sea%'];
@@ -203,8 +203,8 @@ XPATH [__view 'cat'] /category[@* = 1];
 
 -- compares BLOB column
 --XPATH [__view 'cat'] //*[@* = 1];
---ECHO BOTH $IF $EQU $ROWCNT 1 "PASSED" "***FAILED";
---ECHO BOTH ": " $ROWCNT " entities in cat with * = 1 \n";
+--echo both $if $equ $rowcnt 1 "PASSED" "*** FAILED";
+--echo both ": " $rowcnt " entities in cat with * = 1 \n";
 
 
 
@@ -212,13 +212,13 @@ XPATH [__view 'cat'] /category[@* = 1];
 XPATH [__view 'cat'] length (category/@description);
 
 --XPATH [__view 'cat'] /category[@description like 'Sea%']/product | /category[@description like '%Bread%']/product;
---ECHO BOTH $IF $EQU $ROWCNT 19 "PASSED" "***FAILED";
---ECHO BOTH ": " $ROWCNT " rows in seafood union breads \n";
+--echo both $if $equ $rowcnt 19 "PASSED" "*** FAILED";
+--echo both ": " $rowcnt " rows in seafood union breads \n";
 
 
 XPATH [__view 'cat'] /category [.//product [@ProductName like 'L%']];
-ECHO BOTH $IF $EQU $ROWCNT 3 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " categories with a product like L% \n";
+echo both $if $equ $rowcnt 3 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " categories with a product like L% \n";
 
 
 
@@ -266,20 +266,20 @@ select n from XML_TEXT where xpath_contains (XT_TEXT, '//product[@ProductName =
 select n from XML_TEXT where xpath_contains (XT_TEXT, '//product[@ProductName = ''Inlagd Sill'']/following-sibling::*', n);
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '//product[@ProductName = ''Inlagd Sill'']/preceding-sibling::*', n) and XT_FILE = 'cat1';
-ECHO BOTH $IF $EQU $ROWCNT 4 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " siblings before Sill   \n";
+echo both $if $equ $rowcnt 4 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " siblings before Sill   \n";
 
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '//product[@ProductName = ''Inlagd Sill'']/following-sibling::*', n) and XT_FILE = 'cat1';
-ECHO BOTH $IF $EQU $ROWCNT 7 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " siblings after Sill   \n";
+echo both $if $equ $rowcnt 7 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " siblings after Sill   \n";
 
 
 
 
 select n from XML_TEXT where xpath_contains (XT_TEXT, '/document/category[@description like ''**fish'']/descendant-or-self::*', n) and XT_FILE = 'cat1';
-ECHO BOTH $IF $EQU $ROWCNT 13 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in seafood and descendants \n";
+echo both $if $equ $rowcnt 13 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in seafood and descendants \n";
 
 select cat from XML_TEXT where xpath_contains (XT_TEXT, 'document/category', cat);
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, 'document/category', cat);
@@ -292,20 +292,20 @@ select p from XML_TEXT where xpath_contains (XT_TEXT, '/document/category[@descr
 
 
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, '//*', p);
-ECHO BOTH $IF $EQU $LAST[1] 5404 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in xpath_contains '//*'\n";
+echo both $if $equ $last[1] 5404 "PASSED" "*** FAILED";
+echo both ": " $last[1] " in xpath_contains '//*'\n";
 
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, '//*/*', p);
-ECHO BOTH $IF $EQU $LAST[1] 5401 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in xpath_contains '//*/*'\n";
+echo both $if $equ $last[1] 5401 "PASSED" "*** FAILED";
+echo both ": " $last[1] " in xpath_contains '//*/*'\n";
 
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, '//*[@* = ''Inlagd Sill'']', p);
-ECHO BOTH $IF $EQU $LAST[1] 33 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " in xpath_contains //*[@* = 'Inlagd Sill']\n";
+echo both $if $equ $last[1] 33 "PASSED" "*** FAILED";
+echo both ": " $last[1] " in xpath_contains //*[@* = 'Inlagd Sill']\n";
 
 select xpath_eval ('@product_name', p) from XML_TEXT where xpath_contains (XT_TEXT, '//*[@* = ''Inlagd Sill'']', p);
-ECHO BOTH $IF $EQU $ROWCNT 33 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_eval (@product_name) of xpath_contains //*[@* = 'Inlagd Sill']\n";
+echo both $if $equ $rowcnt 33 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_eval (@product_name) of xpath_contains //*[@* = 'Inlagd Sill']\n";
 
 select p from XML_TEXT where xpath_contains (XT_TEXT, '//*[@* = ''Inlagd Sill'' and @unit_price > 100]', p);
 
@@ -410,48 +410,48 @@ DB.DBA.vt_inc_index_DB_DBA_XML_TEXT ();
 
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//title', t);
-ECHO BOTH $IF $EQU $ROWCNT 792 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //title\n";
+echo both $if $equ $rowcnt 792 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //title\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//title [. like ''%ISOLATION%'' ]', t);
-ECHO BOTH $IF $EQU $ROWCNT 4 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //title [. like '%ISOLATION%' ]\n";
-ECHO BOTH $IF $EQU $LAST[1]  "<title>SQL_TXN_ISOLATION</title>" "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " last row in xpath_contains //title [. like '%ISOLATION%' ]\n";
+echo both $if $equ $rowcnt 4 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //title [. like '%ISOLATION%' ]\n";
+echo both $if $equ $last[1]  "<title>SQL_TXN_ISOLATION</title>" "PASSED" "*** FAILED";
+echo both ": " $last[1] " last row in xpath_contains //title [. like '%ISOLATION%' ]\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//title [.=''ISOLATION'' ]', t);
-ECHO BOTH $IF $EQU $ROWCNT 2 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //title [.='ISOLATION' ]\n";
+echo both $if $equ $rowcnt 2 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //title [.='ISOLATION' ]\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//title [. like ''%ISOLATION%'' ]/ancestor::*/title', t);
-ECHO BOTH $IF $EQU $ROWCNT 16 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //title [. like '%ISOLATION%' ]/ancestor::*/title\n";
+echo both $if $equ $rowcnt 16 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //title [. like '%ISOLATION%' ]/ancestor::*/title\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//title [.=''ISOLATION'' ]/ancestor::*/title', t);
-ECHO BOTH $IF $EQU $ROWCNT 7 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //title [.='ISOLATION' ]/ancestor::*/title\n";
--- ECHO BOTH $IF $EQU $LAST[1] "<title>SQL Reference</title>" "PASSED" "***FAILED";
--- ECHO BOTH ": " $LAST[1] " last row in xpath_contains //title [. like '%ISOLATION%' ]/ancestor::*/title\n";
+echo both $if $equ $rowcnt 7 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //title [.='ISOLATION' ]/ancestor::*/title\n";
+-- echo both $if $equ $last[1] "<title>SQL Reference</title>" "PASSED" "*** FAILED";
+-- echo both ": " $last[1] " last row in xpath_contains //title [. like '%ISOLATION%' ]/ancestor::*/title\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//chapter/title', t);
-ECHO BOTH $IF $EQU $ROWCNT 20 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //chapter/title\n";
---ECHO BOTH $IF $EQU $LAST[1] "<title>Virtual Database Concepts</title>" "PASSED" "***FAILED";
---ECHO BOTH ": " $LAST[1] " last row in xpath_contains //chapter/title\n";
+echo both $if $equ $rowcnt 20 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //chapter/title\n";
+--echo both $if $equ $last[1] "<title>Virtual Database Concepts</title>" "PASSED" "*** FAILED";
+--echo both ": " $last[1] " last row in xpath_contains //chapter/title\n";
 
 select t from XML_TEXT where xpath_contains (XT_TEXT, '//chapter/title[position () = 1]', t);
-ECHO BOTH $IF $EQU $ROWCNT 20 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in xpath_contains //chapter/title[position () = 1]\n";
--- ECHO BOTH $IF $EQU $LAST[1] "<title>International character support and compatibility</title>" "PASSED" "***FAILED";
--- ECHO BOTH ": " $LAST[1] " last row in xpath_contains //chapter/title[position () = 1]\n";
+echo both $if $equ $rowcnt 20 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in xpath_contains //chapter/title[position () = 1]\n";
+-- echo both $if $equ $last[1] "<title>International character support and compatibility</title>" "PASSED" "*** FAILED";
+-- echo both ": " $last[1] " last row in xpath_contains //chapter/title[position () = 1]\n";
 
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, '//chapter//para[position () > 10]', t);
-ECHO BOTH $IF $EQU $LAST[1] 6 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " rows in xpath_contains //chapter//para[position () > 10]\n";
+echo both $if $equ $last[1] 6 "PASSED" "*** FAILED";
+echo both ": " $last[1] " rows in xpath_contains //chapter//para[position () > 10]\n";
 
 select count (*) from XML_TEXT where xpath_contains (XT_TEXT, '//chapter/descendant::para[position () > 10]', t);
-ECHO BOTH $IF $EQU $LAST[1] 1630 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " rows in xpath_contains //chapter/descendant::para[position () > 10]\n";
+echo both $if $equ $last[1] 1630 "PASSED" "*** FAILED";
+echo both ": " $last[1] " rows in xpath_contains //chapter/descendant::para[position () > 10]\n";
 
 select c from XML_TEXT where xpath_contains (XT_TEXT, '//customer[.//product/@unit_price > 20]/@name', c);
 
@@ -461,8 +461,8 @@ select p from XML_TEXT where xpath_contains (XT_TEXT, '(document/category/produc
 select p from XML_TEXT where xpath_contains (XT_TEXT, '(document/category/product)[position () > 22 and position() < 33]', p);
 
 select count (*) from XML_TEXT where XT_ID = 10000000000 and xpath_contains (XT_TEXT, '1 > 2 != 2 > 1');
-ECHO BOTH $IF $EQU $LAST[1] 1 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " rows in xpath_contains 1 > 2 != 2 > 1\n";
+echo both $if $equ $last[1] 1 "PASSED" "*** FAILED";
+echo both ": " $last[1] " rows in xpath_contains 1 > 2 != 2 > 1\n";
 
 
 select p from XML_TEXT where xpath_contains (XT_TEXT, '[xmlns:r="http://www.w3.org/TR/RDF/"]//*[@r:id = "Top"]', p);
@@ -470,14 +470,14 @@ select p from XML_TEXT where xpath_contains (XT_TEXT, '[xmlns:r="http://www.w3.o
 
 
 select SCORE, XT_FILE from XML_TEXT where contains (XT_TEXT, '"case sensitive"');
-ECHO BOTH $IF $EQU $ROWCNT 3 "PASSED" "***FAILED";
-ECHO BOTH ": " $ROWCNT " rows in contains case sensitive\n";
-ECHO BOTH $IF $EQU $LAST[1] 120 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " last score in contains case sensitive\n";
+echo both $if $equ $rowcnt 3 "PASSED" "*** FAILED";
+echo both ": " $rowcnt " rows in contains case sensitive\n";
+echo both $if $equ $last[1] 120 "PASSED" "*** FAILED";
+echo both ": " $last[1] " last score in contains case sensitive\n";
 
 explain ('select SCORE, XT_FILE from XML_TEXT where contains (XT_TEXT, ''"case sensitive"'')', 2);
-ECHO BOTH $IF $EQU $STATE OK "PASSED" "***FAILED";
-ECHO BOTH ": BUG 5796: scrollable cursor over contains STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
+echo both $if $equ $state OK "PASSED" "*** FAILED";
+echo both ": BUG 5796: scrollable cursor over contains STATE=" $state " MESSAGE=" $message "\n";
 
 select SCORE, XT_FILE from XML_TEXT where contains (XT_TEXT, '"case sensitive"') order by XT_ID desc;
 
@@ -525,12 +525,12 @@ select xpath_eval ('translate ("1234567 1234567", "123", "abc")', xml_tree_doc (
 select xpath_eval ('round-number (11.2)', xml_tree_doc (xml_tree ('<a><b>11</b><b>33</b></a>')), 1);
 
 select length (xpath_eval ('//@href', xml_tree_doc (xml_tree ('<html><a href="1">11</a><ul><a href = "2"></a></ul><ul><a name="test"></a></ul></html>')), 0));
-ECHO BOTH $IF $EQU $LAST[1] 2 "PASSED" "*** FAILED";
-ECHO BOTH ": " $LAST[1] " rows in xpath_eval //@href\n";
+echo both $IF $EQU $LAST[1] 2 "PASSED" "*** FAILED";
+echo both ": " $LAST[1] " rows in xpath_eval //@href\n";
 
 select length (xpath_eval ('//a/@name', xml_tree_doc (xml_tree ('<html><a href="1">11</a><ul><a href = "2"></a></ul><ul><a name="test"></a></ul></html>')), 0));
-ECHO BOTH $IF $EQU $LAST[1] 1 "PASSED" "*** FAILED";
-ECHO BOTH ": " $LAST[1] " rows in xpath_eval //a/@name\n";
+echo both $IF $EQU $LAST[1] 1 "PASSED" "*** FAILED";
+echo both ": " $LAST[1] " rows in xpath_eval //a/@name\n";
 
 
 --- free text options
