@@ -4,26 +4,26 @@
 --  $Id$
 --
 --  Recovery check test
---  
+--
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
---  
---  Copyright (C) 1998-2013 OpenLink Software
---  
+--
+--  Copyright (C) 1998-2024 OpenLink Software
+--
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
 --  Free Software Foundation; only version 2 of the License, dated June 1991.
---  
+--
 --  This program is distributed in the hope that it will be useful, but
 --  WITHOUT ANY WARRANTY; without even the implied warranty of
 --  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 --  General Public License for more details.
---  
+--
 --  You should have received a copy of the GNU General Public License along
 --  with this program; if not, write to the Free Software Foundation, Inc.,
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
---  
---  
+--
+--
 
 select count(*) from iutest;
 
@@ -42,14 +42,14 @@ select count(*) from iutest;
 --ECHO BOTH $IF $EQU $LAST[3] 500020 "PASSED" "***FAILED";
 --ECHO BOTH ": BLOBS  sum(length (B3))= " $LAST[3] " \n";
 
-ECHO BOTH "recovck1_noreg check trees\n";
+echo both "recovck1_noreg check trees\n";
 
 cl_exec ('backup ''/dev/null''');
 
-select count (*) from T2;
+--select count (*) from T2;
 
-ECHO BOTH $IF $EQU $LAST[1] 13 "PASSED" "***FAILED";
-ECHO BOTH ": " $LAST[1] " rows in T2 after roll forward.\n";
+--ECHO BOTH $IF $EQU $LAST[1] 13 "PASSED" "***FAILED";
+--ECHO BOTH ": " $LAST[1] " rows in T2 after roll forward.\n";
 
 --XXX: VJ
 --reconnect USR1;
@@ -115,12 +115,17 @@ select count (*), sum (length (b.b1)) from tb_stat c, tblob b where c.k = b.k
 select k, length (b1), length (b2), length (b3), length (b4), * from tblob b where not exists (select 1 from tb_stat c where c.k = b.k                                                       and length (b1) = b1_l and length (b2) = b2_l and length (b3) = b3_l                                                       and length (b4) = b4_l and b. e1 = c. e1 and b. e2 = c. e2);
 
 tb_check (1);
---ECHO BOTH $IF $EQU $STATE OK "PASSED" "***FAILED";
---ECHO BOTH ": blobs rollback / roll forward consistency " $STATE "\n";
+echo both $if $equ $state OK "PASSED" "***FAILED";
+echo both ": blobs rollback / roll forward consistency " $state "\n";
+
+check_twide ();
+echo both $if $equ $state OK "PASSED" "***FAILED";
+echo both ": twide consistency " $state "\n";
+
 
 select * from tblob where length (blob_to_string (b4)) <> length (b4);
-ECHO BOTH $IF $EQU $ROWCNT 0 "PASSED" "***FAILED";
-ECHO BOTH ": tblob length check 2\n";
+echo both $if $equ $rowcnt 0 "PASSED" "***FAILED";
+echo both ": tblob length check 2\n";
 
 insert into B2437 values (1);
 ECHO BOTH $IF $EQU $STATE OK "PASSED" "***FAILED";
@@ -191,18 +196,18 @@ ECHO BOTH ":  FKRFWD3 AFK_BAD1 no FK STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
 delete from FK_OK2 where ID = 100;
 insert into FK_OK2 values (100, 100);
-ECHO BOTH $IF $NEQ $STATE OK "PASSED" "***FAILED";
-ECHO BOTH ": FKRFWD4 FK_OK2 stoped STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
+echo both $if $NEQ $STATE OK "PASSED" "***FAILED";
+echo both ": FKRFWD4 FK_OK2 stoped STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
 delete from AFK_OK2 where ID = 100;
 insert into AFK_OK2 values (100, 100);
-ECHO BOTH $IF $NEQ $STATE OK "PASSED" "***FAILED";
-ECHO BOTH ": FKRFWD5 AFK_OK2 stoped STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
+echo both $if $NEQ $STATE OK "PASSED" "***FAILED";
+echo both ": FKRFWD5 AFK_OK2 stoped STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
 delete from AFK_BAD2 where ID = 100;
 insert into AFK_BAD2 values (100, 100);
-ECHO BOTH $IF $EQU $STATE OK "PASSED" "***FAILED";
-ECHO BOTH ": FKRFWD6 FK_BAD2 allowed STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
+echo both $if $EQU $STATE OK "PASSED" "***FAILED";
+echo both ": FKRFWD6 FK_BAD2 allowed STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
 -- rename
 
@@ -251,5 +256,18 @@ ECHO BOTH $IF $EQU $ROWCNT 1 "PASSED" "***FAILED";
 ECHO BOTH ": B6978-10 table with data copied does have data. ROWCNT=" $ROWCNT "\n";
 
 select length (b) from rep_blob;
-ECHO BOTH $IF $EQU $LAST[1] 20000000 "PASSED"  "***FAILED";
-ECHO BOTH ": replicated ins replacing of large blob\n";
+echo both $if $equ $last[1] 20000000 "PASSED"  "***FAILED";
+echo both ": replicated ins replacing of large blob\n";
+
+select * from tainc natural join tainc_ck;
+echo both $if $equ $rowcnt 3  "PASSED" "***FAILED";
+echo both ":  autoinc/ts rfwd\n";
+
+sparql select * from <g1> where { ?s <only1> ?o . };
+echo both $if $equ $rowcnt 0  "PASSED" "***FAILED";
+echo both ": rdf not only1\n";
+
+sparql select * from <g1> where { ?s <only2> ?o . };
+echo both $if $equ $last[2] only2  "PASSED" "***FAILED";
+echo both ": rdf only2\n";
+
