@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -42,7 +42,7 @@ create procedure TDAV_RDF_QUAD_CHECK (in resname varchar, in propuri varchar, in
   dbg_obj_princ ('TDAV_RDF_QUAD_CHECK: ', full_res_uri, propuri, encoded_propval);
   if (crop <> 0)
     return;
-  if (exists (sparql ask where { graph ?:new_dav_graph {
+  if ((sparql define input:storage "" ask where { graph ?:new_dav_graph {
           ?:full_res_uri ?:propuri ?:propval } } ))
     actual := 1;
   else
