@@ -10,7 +10,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -866,7 +866,7 @@ ECHO BOTH ": ins func call casting narrow string into nvarchar column STATE=" $S
 update  WIDEZTEST set data =  cast ('\x5\xfc\x1\xfd\x2\xfe\x3\xff\x4' as nvarchar);
 select bin2hex(DATA) from WIDEZTEST where ID = 3;
 select length (DATA) from WIDEZTEST where ID = 3;
-ECHO BOTH $IF $EQU $LAST[1] 9 "PASSED" "*** SKIPPED";
+ECHO BOTH $IF $EQU $LAST[1] 9 "PASSED" "*** FAILED";
 ECHO BOTH ": update casting narrow string into nvarchar column STATE=" $STATE " MESSAGE=" $MESSAGE " DATA LENGTH=" $LAST[1] ", must be 9\n";
 
 update  WIDEZTEST set data =  ff (cast ('\x5\xfc\x1\xfd\x2\xfe\x3\xff\x4' as nvarchar));
@@ -1537,60 +1537,21 @@ from DB.DBA.RDF_IRI_RANK_C as r
 ',0)
 ;
 
-select xmlelement ("result", xmlattributes ('list' as "type"),
-    xmlagg (xmlelement ("row",
-	xmlelement ("column",
-	xmlattributes (fct_lang ("c1") as "xml:lang",
-	  fct_dtp ("c1") as "datatype",
-	  fct_short_form(__ro2sq("c1")) as "shortform",
-	  fct_sparql_ser ("c1") as "sparql_ser"),
-	__ro2sq ("c1")),
-	xmlelement ("column", fct_label ("c1", 0, 'facets' )))))
-	from (sparql define output:valmode "LONG"  define input:storage virtrdf:IRI_Rank_Storage
-	    select distinct ?s11 as ?c1 ?g where {?s1 ?s1condp ?s2 .
-	    filter (?s2 = """EMBL""") .
-	    ?s1 ?s1condp ?s3 .
-	    filter (?s3 = """GeneID""") .
-	    ?s1 ?s1condp ?s4 .
-	    filter (?s4 = """IPI""") .
-	    ?s1 ?s1condp ?s5 .
-	    filter (?s5 = """InterPro""") .
-	    ?s1 ?s1condp ?s6 .
-	    filter (?s6 = """MGI""") .
-	    ?s1 ?s1condp ?s7 .
-	    filter (?s7 = """UniGene""") .
-	    ?s1 ?s1condp ?s8 .
-	    filter (?s8 = <http://purl.uniprot.org/database/EMBL>) .
-	    quad map virtrdf:DefaultQuadMap { ?s9 <http://rdf.freebase.com/ns/common.webpage.uri> ?s1 .
-	    }?s1 a <http://bio2rdf.org/ncbi_resource:source> .
-	    quad map virtrdf:DefaultQuadMap { ?s10 <http://xmlns.com/foaf/0.1/homepage> ?s1 .
-	    } quad map virtrdf:DefaultQuadMap { ?s11 <http://purl.uniprot.org/core/source> ?s10 .
-	    } filter (?s11 = <http://purl.uniprot.org/proteinmodelportal/Q0R6X1>) .
-	    ?s10 a <http://linkedgeodata.org/ontology/HighwayThing> .
-	    ?s10 a <http://purl.uniprot.org/core/Citation_Statement> .
-	    filter (?s1 = <http://purl.uniprot.org/kegg/xla:100301961>) .
-	    quad map virtrdf:DefaultQuadMap { ?s12 <http://uberblic.org/meta/source_uri> ?s1 .
-	    } quad map virtrdf:DefaultQuadMap { ?s13 <http://vocab.sindice.com/xfn#colleague-hyperlink> ?s1 .
-	    }?s1 a <http://bio2rdf.org/ncbi_resource:Record> .
-	    filter (?s1 = <http://purl.uniprot.org/proteinmodelportal/A7YWZ6>) .
-	    ?s1 a <http://www.w3.org/2006/vcard/ns#Name> .
-	    quad map virtrdf:DefaultQuadMap { ?s14 <http://vocab.sindice.com/xfn#mePage> ?s1 .
-	    }?s14 a <http://purl.uniprot.org/core/Domain_Assignment_Statement> .
-	    filter (?s1 = <http://purl.uniprot.org/refseq/YP_293796.1>) .
-	    ?s1 a <http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq> .
-	    filter (?s1 = <http://purl.uniprot.org/unigene/Dr.82736>) .
-	    ?s1 a <http://www.w3.org/2006/vcard/ns#VCard> .
-	    ?s1 a <http://bio2rdf.org/geneid_resource:Gene-commentary> .
-	    ?s1 a <http://bio2rdf.org/ncbi_resource:date_changed> .
-	    filter (?s1 = <http://purl.uniprot.org/proteinmodelportal/Q97356>) .
-	    quad map virtrdf:DefaultQuadMap { ?s15 <http://dbpedia.org/ontology/wikiPageExternalLink> ?s1 .
-	    }?s15 a <http://bio2rdf.org/geneid_resource:Other-source> .
-	    filter (?s1 = <http://purl.uniprot.org/proteinmodelportal/Q80448>) .
-	    filter (?s1 = <http://purl.uniprot.org/proteinmodelportal/Q4SGH6>) .
-	    filter (?s1 = <http://purl.uniprot.org/proteinmodelportal/A6MYD2>) .
-	    quad map virtrdf:DefaultQuadMap { ?s1 <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?s16 .
-} } order by desc (?srank11)  limit 20  offset 0 ) xx option (quietcast)
-;
+create procedure tlongssl()
+{
+  declare ss any;
+  ss := string_output ();
+  http ('create procedure tlongssl_tmp (\n', ss);
+  for (declare i int, i := 0; i < 70000; i := i + 1)
+    {
+      --http (sprintf ('declare v%d any; v%d := %d; \n', i, i, i), ss);
+      http (sprintf ('in v%d any, \n', i), ss);
+    }
+  http (' in endvar any ) \n { return ; }', ss);
+  exec (string_output_string (ss));
+};
+
+tlongssl();
 ECHO BOTH $IF $NEQ $STATE OK "PASSED" "***FAILED";
 ECHO BOTH ": long sql statement, over 64K ssls STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
