@@ -5,7 +5,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #  
-#  Copyright (C) 1998-2018 OpenLink Software
+#  Copyright (C) 1998-2024 OpenLink Software
 #  
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -71,6 +71,41 @@ VIRTUOSO_VDB=0
 VIRTUOSO_TABLE_SCHEME="row"
 VIRTUOSO_CAPACITY="single"
 
+#
+#  Netstat
+#
+NETSTAT=`which netstat 2>/dev/null`
+if test -z "$NETSTAT"
+then
+    NETSTAT=`which ss 2>/dev/null`
+fi
+if test -z "$NETSTAT"
+then
+    echo ""
+    echo "***"
+    echo "*** ERROR: Please make sure either netstat or ss is installed and in your PATH before running the test suite."
+    echo "***" 
+    exit 1
+fi
+export NETSTAT
+
+
+#
+#  Make sure the following binaries are installed
+#
+for i in curl wget gzip tar unzip
+do
+    B=`which $i 2>/dev/null`
+    if test -z "$B"
+    then
+	echo ""
+	echo "***"
+	echo "*** ERROR: Please make sure \'$i\' is installed and in your PATH before running the test suite."
+	echo "***"
+	exit 1
+    fi
+done
+
 while getopts ":smrcvwV" optname
   do
     case "$optname" in
