@@ -5,7 +5,7 @@
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2018 OpenLink Software
+#  Copyright (C) 1998-2024 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -535,11 +535,11 @@ esac
 CHECK_HTTP_PORT()
 {
   port=$1
-  stat=`netstat -an | grep "[\.\:]$port " | grep LISTEN`
+  stat=`$NETSTAT -an 2>/dev/null | grep "[\.\:]$port " | grep LISTEN`
   while [ "z$stat" = "z" ]
   do
-    sleep 1
-    stat=`netstat -an | grep "[\.\:]$port " | grep LISTEN`
+    sleep 5
+    stat=`$NETSTAT -an 2>/dev/null | grep "[\.\:]$port " | grep LISTEN`
   done
   LOG "PASSED: Virtuoso HTTP/WebDAV Server successfully started on port $port"
 }
