@@ -10,7 +10,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -35,8 +35,8 @@
 
 #include <signal.h>
 #include <sys/types.h>
-#include <Dk.h>
-#include <libutil.h>
+#include "Dk.h"
+#include "libutil.h"
 #include "timeacct.h"
 
 #ifndef WIN32
