@@ -4,7 +4,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -53,9 +53,9 @@ create procedure "VAD"."DBA"."DAV_DELETE_VAD" (
         return rrc;
       }
     }
-    for select COL_ID from WS.WS.SYS_DAV_COL where COL_PARENT = id do
+    for select COL_FULL_PATH from WS.WS.SYS_DAV_COL where COL_PARENT = id do
     {
-      rrc := "VAD"."DBA"."DAV_DELETE_VAD" (WS.WS.COL_PATH(COL_ID), silent, extern);
+      rrc := "VAD"."DBA"."DAV_DELETE_VAD" (COL_FULL_PATH, silent, extern);
       if (rrc <> 1)
       {
         rollback work;
@@ -131,7 +131,7 @@ create procedure "VAD"."DBA"."VAD_DAV_MKCOL" (
   declare ret integer;
   usr := 'dav';
   pwd := pwd_magic_calc('dav', (select U_PWD from WS.WS.SYS_DAV_USER where U_NAME='dav'), 1);
-  ret := "DB"."DBA"."DAV_COL_CREATE" (name, '110100100N', usr, NULL, usr, pwd);
+  ret := "DB"."DBA"."DAV_COL_CREATE" (name, '110100000N', usr, NULL, usr, pwd);
   return ret;
 }
 ;
@@ -1207,17 +1207,18 @@ create procedure "DB"."DBA"."VAD_INSTALL" (
   SQL_MESSAGE := '';
   result_names (SQL_STATE, SQL_MESSAGE);
 
-  if ((is_dav = 0 and 0 = file_stat (fname)) 
-      or 
+  if ((is_dav = 0 and 0 = file_stat (fname))
+      or
       (is_dav <> 0 and (not exists (select 1 from WS.WS.SYS_DAV_RES where RES_FULL_PATH = fname))))
     {
       registry_set ('VAD_wet_run', '0');
       registry_set ('VAD_errcount', '1');
       connection_set ('vad_pkg_fullname', 'unknown');
+      connection_set ('vad_pkg_name', NULL);
       connection_set ('app_endpoint_uri', 'unknown');
       connection_set ('app_usage_uri', 'unknown');
       result ('42VAD', concat ('Could not open ',
-      case when is_dav = 0 then 'filesystem' else 'DAV' end, 
+      case when is_dav = 0 then 'filesystem' else 'DAV' end,
       ' resource ', fname, ' Reason: File not found'));
       goto failure;
     }
@@ -1263,6 +1264,7 @@ create procedure "DB"."DBA"."VAD_INSTALL" (
       declare app_endpoint_uri, app_usage_uri varchar;
       app_endpoint_uri := connection_get ('app_endpoint_uri');
       app_usage_uri := connection_get ('app_usage_uri');
+      registry_set (sprintf ('VAD_%s', connection_get ('vad_pkg_name')), connection_get ('vad_pkg_version'));
 
       if (app_endpoint_uri is not null)
 	    result ('00000', sprintf ('Application is accessible via the endpoint at: %s', app_endpoint_uri));
@@ -1472,6 +1474,8 @@ create procedure "VAD"."DBA"."VAD_READ" (
       "VAD"."DBA"."VAD_CHECK_STICKER_DETAILS" (parr, doc, pkg_name, pkg_vers, pkg_fullname, pkg_date, 0);
 
       connection_set ('vad_pkg_fullname', pkg_fullname);
+      connection_set ('vad_pkg_name', pkg_name);
+      connection_set ('vad_pkg_version', pkg_vers);
 
       items := xpath_eval ('/sticker/caption/name/prop[@name=\'AppEndpointURI\']', doc, 0);
       n := length (items);
@@ -1891,6 +1895,7 @@ create procedure "DB"."DBA"."VAD_UNINSTALL_BY_NAME"(
     "VAD"."DBA"."VAD_FAIL_CHECK"(sprintf('The installation of "%s" does not exist or was corrupted. VAD cannot find its current version.', name));
   DB.DBA.VAD_DEPS_CHECK (parr, name, lval);
   "VAD"."DBA"."VAD_PKG_UNINSTALL"(parr, tid, lval);
+  registry_remove (sprintf ('VAD_%s', name));
   return 'OK';
 }
 ;
@@ -1902,6 +1907,12 @@ create procedure "DB"."DBA"."VAD_CHECK_VERSION"(
   declare parr any;
   declare curdir, ret, tid integer;
   declare res, lname, ltype, lval varchar;
+
+  declare ver any;
+  ver := registry_get (sprintf ('VAD_%s', name));
+  if (isstring(ver))
+    return ver;
+
   parr := null;
   curdir := "VAD"."DBA"."VAD_CHDIR"(parr, 0, '/VAD');
   tid := "VAD"."DBA"."VAD_CHDIR"(parr, curdir, name);
@@ -1912,6 +1923,7 @@ create procedure "DB"."DBA"."VAD_CHECK_VERSION"(
   if (not ret)
     return null;
 --    "VAD"."DBA"."VAD_FAIL_CHECK"(sprintf('The installation of "%s" does not exist or was corrupted. VAD cannot find its current version.', name));
+  registry_set (sprintf ('VAD_%s', name), lval);
   return lval;
 }
 ;
@@ -2053,6 +2065,7 @@ create procedure "DB"."DBA"."VAD_UNINSTALL"(
     "VAD"."DBA"."VAD_FAIL_CHECK"(sprintf('Attempt to remove non-current version of package "%s"', prod));
   DB.DBA.VAD_DEPS_CHECK (parr, prod, lval);
   "VAD"."DBA"."VAD_PKG_UNINSTALL"(parr, tid, version);
+  registry_remove (sprintf ('VAD_%s', prod));
   return 'OK';
 }
 ;
