@@ -8,7 +8,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -60,7 +60,6 @@ create table WS.WS.VFS_QUEUE (
     primary key (VQ_HOST, VQ_URL, VQ_ROOT))
 create index VQ_HOST_ROOT on WS.WS.VFS_QUEUE (VQ_HOST, VQ_ROOT)
 create index VQ_HOST_TIME on WS.WS.VFS_QUEUE (VQ_HOST, VQ_ROOT, VQ_STAT, VQ_TS, VQ_URL)
-create index VQ_TS on WS.WS.VFS_QUEUE (VQ_TS)
 create index VQ_ORIGIN on WS.WS.VFS_QUEUE (VQ_ORIGIN)
 ;
 
@@ -115,59 +114,6 @@ create table WS.WS.VFS_SITE_RDF_MAP (
     primary key (VM_HOST, VM_ROOT, VM_RDF_MAP, VM_SEQ))
 ;
 
---#IF VER=5
-create procedure WS.WS.VFS_TBL_UPGRADE ()
-{
-  declare _err, _state varchar;
-  _state := null;
-  exec ('select VS_OPAGE from WS.WS.VFS_SITE', _state, _err);
-  if (_state is not null)
-    DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_OPAGE varchar');
-  _state := null;
-  exec ('select VS_OTHER from WS.WS.VFS_SITE', _state, _err);
-  if (_state is not null)
-    DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_OTHER char (10)');
-  _state := null;
-  exec ('select VQ_OTHER from WS.WS.VFS_QUEUE', _state, _err);
-  if (_state is not null)
-    DB.DBA.execstr ('alter table WS.WS.VFS_QUEUE add VQ_OTHER varchar');
-  _state := null;
-  exec ('select VQ_ERROR from WS.WS.VFS_QUEUE', _state, _err);
-  if (_state is not null)
-    DB.DBA.execstr ('alter table WS.WS.VFS_QUEUE add VQ_ERROR long varchar');
-  _state := null;
-  exec ('select VU_OTHER from WS.WS.VFS_URL', _state, _err);
-  if (_state is not null)
-    DB.DBA.execstr ('alter table WS.WS.VFS_URL add VU_OTHER varchar');
-  _state := null;
-  exec ('select VS_REDIRECT from WS.WS.VFS_SITE', _state, _err);
-  if (_state is not null)
-    {
-      DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_REDIRECT int default 1');
-      DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_STORE int default 1');
-      DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_UDATA long varbinary');
-    }
-  _state := null;
-  exec ('select VS_EXTRACT_FN from WS.WS.VFS_SITE', _state, _err);
-  if (_state is not null)
-    {
-      DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_EXTRACT_FN varchar');
-      DB.DBA.execstr ('alter table WS.WS.VFS_SITE add VS_STORE_FN varchar');
-    }
-}
-;
-
---!AFTER
-WS.WS.VFS_TBL_UPGRADE ()
-;
-
---!AFTER
-alter table WS.WS.VFS_SITE add VS_DLOAD_META int default 0
-;
-
-alter table WS.WS.VFS_SITE add VS_INST_ID  int
-;
---#ENDIF
 
 alter table WS.WS.VFS_SITE add VS_DEPTH int default null
 ;
