@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -67,9 +67,6 @@ int64 dk_n_max_allocs;
 #undef dk_free
 #endif
 
-extern void dk_box_initialize (void);
-extern void dk_box_finalize (void);
-
 #define NO_MALLOC_CACHE
 #undef CACHE_MALLOC
 
