@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -417,7 +417,7 @@ void
 rbuf_test ()
 {
   rbuf_t rb;
-  ptrlong inx, ctr;
+  int inx, ctr; /* inx is not ptrlong despite use with rbuf_get() etc., because rbuf_delete() want to deal with int * */
   int xx = 0;
   memset (&rb, 0, sizeof (rb));
   for (inx = 0; inx < 1000000; inx++)
@@ -446,7 +446,7 @@ rbuf_test ()
     }
   rbuf_delete_all (&rb);
   for (inx = 1; inx < 100000; inx++)
-    rbuf_add (&rb, (void*)inx);
+    rbuf_add (&rb, (void*)(ptrlong) inx);
   rbuf_rewrite (&rb);
   DO_RBUF (ptrlong, x, rbe, rbe_inx, &rb)
     {
