@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -46,6 +46,98 @@
 
 #define MD5_LOGIN
 
+/*
+ *   Handle Validation
+ */
+static dk_hash_t  *hdl_ht = NULL;
+static dk_mutex_t *hdl_mtx = NULL;
+
+void
+virt_handle_init ()
+{
+  hdl_ht = hash_table_allocate (1001);
+  if (!hdl_ht)
+    return;
+
+  hdl_mtx = mutex_allocate ();
+}
+
+void
+virt_handle_cleanup ()
+{
+  if (!hdl_ht || !hdl_mtx)
+    return;
+
+  hash_table_free (hdl_ht);
+  hdl_ht = NULL;
+
+  mutex_free (hdl_mtx);
+  hdl_mtx = NULL;
+}
+
+void
+virt_handle_register (void *handle, int type)
+{
+  if (!hdl_ht || !hdl_mtx)
+    return;
+
+  mutex_enter (hdl_mtx);
+  sethash (handle, hdl_ht, (void *) (ptrlong) type);
+  mutex_leave (hdl_mtx);
+}
+
+void
+virt_handle_unregister (void *handle)
+{
+  if (!hdl_ht || !hdl_mtx)
+    return;
+
+  mutex_enter (hdl_mtx);
+  remhash (handle, hdl_ht);
+  mutex_leave (hdl_mtx);
+}
+
+int
+virt_handle_check_type (void *handle, int type, int nullable)
+{
+  void *type_p;
+
+  if (!hdl_ht || !hdl_mtx)
+    return 0;
+
+  if (!handle)
+    return nullable ? 1 : 0;
+
+  mutex_enter (hdl_mtx);
+  type_p = gethash (handle, hdl_ht);
+  mutex_leave (hdl_mtx);
+
+  if (type_p != NULL && type_p == (void *) (ptrlong) type)
+    return 1;
+
+  return 0;
+}
+
+void
+virt_handle_debug (void)
+{
+  int *type;
+
+  if (!hdl_ht || !hdl_mtx)
+    return;
+
+  printf (("virt_handle_debug:\n"));
+  DO_HT (void *, k, void *, d, hdl_ht)
+  {
+    printf ("%p -> %p\n", k, d);
+  }
+  END_DO_HT;
+}
+
+
+/*
+ *
+ */
 SQLRETURN SQL_API
 virtodbc__SQLAllocConnect (
 	SQLHENV henv,
@@ -69,6 +161,8 @@ virtodbc__SQLAllocConnect (
   cli->con_defs.cdef_txn_timeout = SO_DEFAULT_TIMEOUT;
   cli->con_defs.cdef_prefetch = SELECT_PREFETCH_QUOTA;
 
+  virt_handle_register (cli, SQL_HANDLE_DBC);
+
   return SQL_SUCCESS;
 }
 
@@ -78,6 +172,8 @@ SQLAllocConnect (
 	SQLHENV henv,
 	SQLHDBC * phdbc)
 {
+  ASSERT_HANDLE_TYPE (henv, SQL_HANDLE_ENV);
+
   return virtodbc__SQLAllocConnect (henv, phdbc);
 }
 
@@ -91,8 +187,9 @@ virtodbc__SQLAllocEnv (
 
   if (firsttime)
     {
-      srand ((unsigned int) time(NULL));
       firsttime = 0;
+      srand ((unsigned int) time(NULL));
+      virt_handle_init();
     }
 
   PrpcInitialize ();
@@ -114,6 +211,8 @@ virtodbc__SQLAllocEnv (
     mts_client_init ();
 #endif
 
+    virt_handle_register (env, SQL_HANDLE_ENV);
+
     return SQL_SUCCESS;
   }
 }
@@ -195,6 +294,8 @@ virtodbc__SQLAllocStmt (
   stmt->stmt_opts->so_timeout = STMT_MSEC_OPTION (con->con_defs.cdef_txn_timeout);
 #endif
 
+  virt_handle_register (stmt, SQL_HANDLE_STMT);
+
   return SQL_SUCCESS;
 }
 
@@ -204,6 +305,8 @@ SQLAllocStmt (
 	SQLHDBC hdbc,
 	SQLHSTMT * phstmt)
 {
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   return virtodbc__SQLAllocStmt (hdbc, phstmt);
 }
 
@@ -218,13 +321,26 @@ SQLBindCol (
 	SQLLEN * pcbValue)
 {
   STMT (stmt, hstmt);
-  col_binding_t *col = stmt_nth_col (stmt, icol);
 
-  col->cb_c_type = fCType;
-  col->cb_place = (caddr_t) rgbValue;
-  col->cb_length = pcbValue;
-  col->cb_max_length = cbValueMax;
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    col_binding_t *col = stmt_nth_col (stmt, icol);
 
+    if (cbValueMax == 0 && icol > 0 && fCType != SQL_C_DEFAULT)
+      {
+	/*
+	 * If fCType == SQL_C_DEFAULT, we need to know the SQL type of the data to
+	 * determine which ODBC C type to use. Depending on when SQLBindCol
+	 * is called, we may not be able to determine the SQL type now
+	 */
+	cbValueMax = sqlc_sizeof (fCType, cbValueMax);
+      }
+
+    col->cb_c_type = fCType;
+    col->cb_place = (caddr_t) rgbValue;
+    col->cb_length = pcbValue;
+    col->cb_max_length = cbValueMax;
+  }
   return SQL_SUCCESS;
 }
 
@@ -253,6 +369,8 @@ SQLRETURN SQL_API
 SQLCancel (
     SQLHSTMT hstmt)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLCancel (hstmt);
 }
 
@@ -269,6 +387,8 @@ SQLColAttributes (
 {
   STMT (stmt, hstmt);
 
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   switch (fDescType)
     {
     case SQL_COLUMN_LABEL:
@@ -804,7 +924,9 @@ internal_sql_connect (
 
   if (con->con_charset)
     {
-      wide_charset_free (con->con_charset);
+      if (con->con_charset != CHARSET_UTF8)
+        wide_charset_free (con->con_charset);
+
       con->con_charset = NULL;
     }
   strncpy (addr_lst, dsn, (sizeof (addr_lst) - 1));
@@ -1047,31 +1169,38 @@ internal_sql_connect (
 
       con_set_defaults (con, login_res);
 
-      if (BOX_ELEMENTS (login_res) > LG_CHARSET)
-	{
-	  caddr_t *cs_info = (caddr_t *) login_res[LG_CHARSET];
-	  if (cs_info && DV_TYPE_OF (cs_info) == DV_ARRAY_OF_POINTER && BOX_ELEMENTS (cs_info) > 1)
-	    con->con_charset =
-		wide_charset_create (cs_info[0], (wchar_t *) cs_info[1], box_length (cs_info[1]) / sizeof (wchar_t) - 1, NULL);
-	}
-
-      if (con->con_charset_name && (!con->con_charset || strcmp (con->con_charset->chrs_name, con->con_charset_name)))
-	{
-	  if (strcmp ("ISO-8859-1", con->con_charset_name))
+      if (con->con_string_is_utf8)
+        {
+          con->con_charset = CHARSET_UTF8;
+        }
+      else
+        {
+          if (BOX_ELEMENTS (login_res) > LG_CHARSET)
 	    {
-	      snprintf (err, sizeof (err),
-		  "Charset %s not available. Server default %s will be used.",
-		  con->con_charset_name, con->con_charset ? con->con_charset->chrs_name : "ISO-8859-1");
-	      set_success_info (&con->con_error, "2C000", "CL035", err, 0);
+	      caddr_t *cs_info = (caddr_t *) login_res[LG_CHARSET];
+	      if (cs_info && DV_TYPE_OF (cs_info) == DV_ARRAY_OF_POINTER && BOX_ELEMENTS (cs_info) > 1)
+	        con->con_charset =
+		    wide_charset_create (cs_info[0], (wchar_t *) cs_info[1], box_length (cs_info[1]) / sizeof (wchar_t) - 1, NULL);
+	    }
+
+          if (con->con_charset_name && (!con->con_charset || strcmp (con->con_charset->chrs_name, con->con_charset_name)))
+	    {
+	      if (strcmp ("ISO-8859-1", con->con_charset_name))
+	        {
+	          snprintf (err, sizeof (err),
+		      "Charset %s not available. Server default %s will be used.",
+		      con->con_charset_name, con->con_charset ? con->con_charset->chrs_name : "ISO-8859-1");
+	          set_success_info (&con->con_error, "2C000", "CL035", err, 0);
+	          rc = SQL_SUCCESS_WITH_INFO;
+	        }
+	    }
+          else if (!con->con_charset_name && con->con_charset && strcmp ("ISO-8859-1", con->con_charset->chrs_name))
+	    {
+	      snprintf (err, sizeof (err), "Switching to the server default charset %s.", con->con_charset->chrs_name);
+	      set_success_info (&con->con_error, "01S02", "CL036", err, 0);
 	      rc = SQL_SUCCESS_WITH_INFO;
 	    }
-	}
-      else if (!con->con_charset_name && con->con_charset && strcmp ("ISO-8859-1", con->con_charset->chrs_name))
-	{
-	  snprintf (err, sizeof (err), "Switching to the server default charset %s.", con->con_charset->chrs_name);
-	  set_success_info (&con->con_error, "01S02", "CL036", err, 0);
-	  rc = SQL_SUCCESS_WITH_INFO;
-	}
+        }
 
       if (con->con_charset_name)
 	dk_free_box (con->con_charset_name);
@@ -1212,14 +1341,17 @@ SQLDescribeCol (
 {
   SQLRETURN rc;
   STMT (stmt, hstmt);
-  NDEFINE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection, SQLSMALLINT);
 
-  NMAKE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection);
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    NDEFINE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection, SQLSMALLINT);
 
-  rc = virtodbc__SQLDescribeCol (hstmt, icol, szColName, _cbColName, _pcbColName, pfSqlType, pcbColDef, pibScale, pfNullable);
+    NMAKE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection);
 
-  NSET_AND_FREE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection);
+    rc = virtodbc__SQLDescribeCol (hstmt, icol, szColName, _cbColName, _pcbColName, pfSqlType, pcbColDef, pibScale, pfNullable);
 
+    NSET_AND_FREE_OUTPUT_CHAR_NARROW (ColName, stmt->stmt_connection);
+  }
   return rc;
 }
 
@@ -1228,6 +1360,9 @@ SQLRETURN SQL_API
 SQLDisconnect (SQLHDBC hdbc)
 {
   CON (con, hdbc);
+
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   if (con->con_session)
     PrpcDisconnect (con->con_session);
 
@@ -1252,6 +1387,10 @@ SQLError (
   SQLCHAR szSqlState[6];
   SQLRETURN rc;
 
+  ASSERT_OPT_HANDLE_TYPE (henv, SQL_HANDLE_ENV);
+  ASSERT_OPT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+  ASSERT_OPT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   if (con || stmt)
     {
       cli_connection_t *conn = con ? con : stmt->stmt_connection;
@@ -1547,13 +1686,17 @@ SQLExecDirect (
   SQLRETURN rc;
   size_t len;
   STMT (stmt, hstmt);
-  NDEFINE_INPUT_NARROW (SqlStr);
 
-  NMAKE_INPUT_ESCAPED_NARROW (SqlStr, stmt->stmt_connection);
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    NDEFINE_INPUT_NARROW (SqlStr);
 
-  rc = virtodbc__SQLExecDirect (hstmt, szSqlStr, cbSqlStr);
+    NMAKE_INPUT_ESCAPED_NARROW (SqlStr, stmt->stmt_connection);
 
-  NFREE_INPUT_NARROW (SqlStr);
+    rc = virtodbc__SQLExecDirect (hstmt, szSqlStr, cbSqlStr);
+
+    NFREE_INPUT_NARROW (SqlStr);
+  }
 
   return rc;
 }
@@ -1563,6 +1706,8 @@ SQLRETURN SQL_API
 SQLExecute (
 	SQLHSTMT hstmt)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLExecDirect (hstmt, NULL, 0);
 }
 
@@ -1648,6 +1793,8 @@ SQLFetch (SQLHSTMT hstmt)
 {
   STMT (stmt, hstmt);
 
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   set_error (&stmt->stmt_error, NULL, NULL, NULL);
 
   if (stmt->stmt_connection->con_environment->env_odbc_version < 3)
@@ -1670,6 +1817,8 @@ SQLFetch (SQLHSTMT hstmt)
 SQLRETURN SQL_API
 SQLFreeConnect (SQLHDBC hdbc)
 {
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   return virtodbc__SQLFreeConnect (hdbc);
 }
 
@@ -1679,6 +1828,8 @@ virtodbc__SQLFreeConnect (SQLHDBC hdbc)
 {
   CON (con, hdbc);
 
+  virt_handle_unregister (con);
+
   set_error (&con->con_error, NULL, NULL, NULL);
 
   if (con->con_session)
@@ -1692,7 +1843,7 @@ virtodbc__SQLFreeConnect (SQLHDBC hdbc)
   if (con->con_bookmarks)
     hash_table_free (con->con_bookmarks);
 
-  if (con->con_charset)
+  if (con->con_charset && con->con_charset != CHARSET_UTF8)
     wide_charset_free (con->con_charset);
 
   if (con->con_user)
@@ -1721,6 +1872,7 @@ virtodbc__SQLFreeConnect (SQLHDBC hdbc)
   dk_set_delete (&con->con_environment->env_connections, (void *) con);
   dk_free ((caddr_t) con, sizeof (cli_connection_t));
 
+
   return SQL_SUCCESS;
 }
 
@@ -1728,6 +1880,8 @@ virtodbc__SQLFreeConnect (SQLHDBC hdbc)
 SQLRETURN SQL_API
 SQLFreeEnv (SQLHENV henv)
 {
+  ASSERT_HANDLE_TYPE (henv, SQL_HANDLE_ENV);
+
   return virtodbc__SQLFreeEnv (henv);
 }
 
@@ -1737,6 +1891,8 @@ virtodbc__SQLFreeEnv (SQLHENV henv)
 {
   ENV (env, henv);
 
+  virt_handle_unregister (env);
+
   set_error (&env->env_error, NULL, NULL, NULL);
 
   mutex_free (env->env_mtx);
@@ -1822,6 +1978,7 @@ virtodbc__SQLFreeStmt (SQLHSTMT hstmt, SQLUSMALLINT fOption)
       }
 
     case SQL_DROP:
+      virt_handle_unregister (hstmt);
       virtodbc__SQLFreeStmt (hstmt, SQL_UNBIND);
       virtodbc__SQLFreeStmt (hstmt, SQL_RESET_PARAMS);
 
@@ -1890,6 +2047,11 @@ virtodbc__SQLFreeStmt (SQLHSTMT hstmt, SQLUSMALLINT fOption)
 	  dk_free ((caddr_t) stmt->stmt_imp_row_descriptor, sizeof (stmt_descriptor_t));
 	  dk_free ((caddr_t) stmt->stmt_app_param_descriptor, sizeof (stmt_descriptor_t));
 	  dk_free ((caddr_t) stmt->stmt_imp_param_descriptor, sizeof (stmt_descriptor_t));
+
+	  stmt->stmt_app_row_descriptor = NULL;
+	  stmt->stmt_imp_row_descriptor = NULL;
+	  stmt->stmt_app_param_descriptor = NULL;
+	  stmt->stmt_imp_param_descriptor = NULL;
 	}
 #endif
       dk_free ((caddr_t) stmt, sizeof (cli_stmt_t));
@@ -1904,6 +2066,8 @@ virtodbc__SQLFreeStmt (SQLHSTMT hstmt, SQLUSMALLINT fOption)
 SQLRETURN SQL_API
 SQLFreeStmt (SQLHSTMT hstmt, SQLUSMALLINT fOption)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLFreeStmt (hstmt, fOption);
 }
 
@@ -1946,14 +2110,17 @@ SQLGetCursorName (
 {
   SQLRETURN rc;
   STMT (stmt, hstmt);
-  NDEFINE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection, SQLSMALLINT);
 
-  NMAKE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection);
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    NDEFINE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection, SQLSMALLINT);
 
-  rc = virtodbc__SQLGetCursorName (hstmt, szCursor, _cbCursor, _pcbCursor);
+    NMAKE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection);
 
-  NSET_AND_FREE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection);
+    rc = virtodbc__SQLGetCursorName (hstmt, szCursor, _cbCursor, _pcbCursor);
 
+    NSET_AND_FREE_OUTPUT_CHAR_NARROW (Cursor, stmt->stmt_connection);
+  }
   return rc;
 }
 
@@ -1961,6 +2128,8 @@ SQLGetCursorName (
 SQLRETURN SQL_API
 SQLNumResultCols (SQLHSTMT hstmt, SQLSMALLINT * pccol)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLNumResultCols (hstmt, pccol);
 }
 
@@ -2040,14 +2209,17 @@ SQLPrepare (
   size_t len;
   SQLRETURN rc;
   STMT (stmt, hstmt);
-  NDEFINE_INPUT_NARROW (SqlStr);
 
-  NMAKE_INPUT_ESCAPED_NARROW (SqlStr, stmt->stmt_connection);
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    NDEFINE_INPUT_NARROW (SqlStr);
 
-  rc = virtodbc__SQLPrepare (hstmt, szSqlStr, SQL_NTS);
+    NMAKE_INPUT_ESCAPED_NARROW (SqlStr, stmt->stmt_connection);
 
-  NFREE_INPUT_NARROW (SqlStr);
+    rc = virtodbc__SQLPrepare (hstmt, szSqlStr, SQL_NTS);
 
+    NFREE_INPUT_NARROW (SqlStr);
+  }
   return rc;
 }
 
@@ -2057,6 +2229,8 @@ SQLRowCount (SQLHSTMT hstmt, SQLLEN * pcrow)
 {
   STMT (stmt, hstmt);
 
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   *pcrow = stmt->stmt_rows_affected;
 
   return SQL_SUCCESS;
@@ -2091,13 +2265,17 @@ SQLSetCursorName (
   SQLRETURN rc;
   size_t len;
   size_t cbCursor = _cbCursor;
-  NDEFINE_INPUT_NARROW (Cursor);
 
-  NMAKE_INPUT_NARROW (Cursor, stmt->stmt_connection);
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+  {
+    NDEFINE_INPUT_NARROW (Cursor);
 
-  rc = virtodbc__SQLSetCursorName (hstmt, szCursor, (SQLSMALLINT) cbCursor);
+    NMAKE_INPUT_NARROW (Cursor, stmt->stmt_connection);
 
-  NFREE_INPUT_NARROW (Cursor);
+    rc = virtodbc__SQLSetCursorName (hstmt, szCursor, (SQLSMALLINT) cbCursor);
+
+    NFREE_INPUT_NARROW (Cursor);
+  }
   return rc;
 }
 
@@ -2141,6 +2319,8 @@ SQLSetParam (
       SQLPOINTER rgbValue,
       SQLLEN * pcbValue)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLSetParam (hstmt, ipar, fCType, fSqlType, cbColDef, ibScale, rgbValue, pcbValue);
 }
 
@@ -2176,9 +2356,6 @@ virtodbc__SQLBindParameter (
   if (fCType == SQL_C_DEFAULT)
     fCType = sql_type_to_sqlc_default (fSqlType);
 
-  if (fCType == SQL_C_WCHAR && cbValueMax % sizeof (wchar_t))
-    cbValueMax = ((SQLLEN) (cbValueMax / sizeof (wchar_t))) * sizeof (wchar_t);
-
   pb->pb_c_type = fCType;
   pb->pb_sql_type = fSqlType;
   pb->pb_place = (caddr_t) rgbValue;
@@ -2204,6 +2381,8 @@ SQLBindParameter (
     SQLLEN cbValueMax,
     SQLLEN * pcbValue)
 {
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return virtodbc__SQLBindParameter (hstmt, ipar, fParamType, fCType, fSqlType, cbColDef, ibScale, rgbValue, cbValueMax, pcbValue);
 }
 
@@ -2214,6 +2393,9 @@ SQLTransact (
       SQLHDBC hdbc,
       SQLUSMALLINT fType)
 {
+  ASSERT_OPT_HANDLE_TYPE (henv, SQL_HANDLE_ENV);
+  ASSERT_OPT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   return virtodbc__SQLTransact (henv, hdbc, fType);
 }
 
@@ -2297,6 +2479,8 @@ SQLSync (SQLHSTMT hstmt)
 {
   STMT (stmt, hstmt);
 
+  ASSERT_HANDLE_TYPE (hstmt, SQL_HANDLE_STMT);
+
   return stmt_process_result (stmt, 1);
 }
 
@@ -2307,6 +2491,8 @@ SQLShutdown (SQLHDBC hdbc, char *new_log)
   CON (dbc, hdbc);
   caddr_t box = new_log ? box_string (new_log) : NULL;
 
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   PrpcFutureFree (PrpcFuture (dbc->con_session, &s_ds_shutdown, box));
 
   return SQL_SUCCESS;
@@ -2319,8 +2505,9 @@ SQLCheckpoint (SQLHDBC hdbc, char *new_log)
   CON (dbc, hdbc);
   caddr_t box = new_log ? box_string (new_log) : NULL;
 
-  VERIFY_INPROCESS_CLIENT (dbc);
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
 
+  VERIFY_INPROCESS_CLIENT (dbc);
   PrpcSync (PrpcFuture (dbc->con_session, &s_ds_makecp, box));
 
   return SQL_SUCCESS;
@@ -2333,6 +2520,8 @@ SQLStatus (SQLHDBC hdbc, char *buffer, int max)
   CON (dbc, hdbc);
   caddr_t *sta;
 
+  ASSERT_HANDLE_TYPE (hdbc, SQL_HANDLE_DBC);
+
   VERIFY_INPROCESS_CLIENT (dbc);
 
   sta = (caddr_t *) PrpcSync (PrpcFutureSetTimeout (PrpcFuture (dbc->con_session, &s_ds_status), 40000L));
