@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -383,6 +383,13 @@ cmp_double (double x1, double x2, double epsilon)
   double delta;
   int exponent;
 
+  if (isnan(x1) && isnan(x2))
+    return DVC_MATCH;
+  if (isnan(x2))
+    return DVC_LESS;
+  if (isnan(x1))
+    return DVC_GREATER;
+
   /*
    *  Get exponent(max(fabs(x1), fabs(x2))) and store it in exponent.
    *
@@ -966,67 +973,71 @@ retry_rdf_boxes: \
     goto int_case; \
   if (dtp1 == DV_DB_NULL || dtp2 == DV_DB_NULL) \
     goto null_result; \
-  if (n_coerce ((caddr_t) & dn1, (caddr_t) & dn2, \
-	  dtp1, dtp2, &res_dtp)) \
+  if (n_coerce ((caddr_t) & dn1, (caddr_t) & dn2, dtp1, dtp2, &res_dtp)) \
     { \
       switch (res_dtp) \
-	{ \
-	case DV_LONG_INT: \
-	int_case: \
-	  if (isdiv && 0 == *(boxint *) &dn2) \
-	    sqlr_new_error ("22012", "SR084", "Division by 0."); \
-	  if (target) \
-	    return (qst_set_long (qst, target, \
-		(*(boxint *) &dn1 op * (boxint *) &dn2)), (caddr_t) 0); \
-	  return (box_num (*(boxint *) &dn1 op * (boxint *) &dn2)); \
-	case DV_SINGLE_FLOAT: \
-	  if (isdiv && 0 == *(float *) &dn2) \
-	    sqlr_new_error ("22012", "SR085", "Division by 0."); \
-	  if (target) \
-	    return (qst_set_float (qst, target, \
-		(*(float *) &dn1 op * (float *) &dn2)), (caddr_t) 0); \
-	  return (box_float (*(float *) &dn1 op * (float *) &dn2)); \
-	case DV_DOUBLE_FLOAT: \
-	  if (isdiv && 0 == *(double*) &dn2) \
-	    sqlr_new_error ("22012", "SR086", "Division by 0."); \
-	  if (target) \
-	    return (qst_set_double (qst, target, (*(double*) &dn1 op *(double*) &dn2)), (caddr_t) 0); \
-	  return (box_double (*(double*) &dn1 op *(double*) &dn2)); \
-	case DV_NUMERIC: \
-	  return (numeric_bin_op (num_op, (numeric_t) &dn1, (numeric_t) &dn2, qst, target)); \
-	} \
-    } \
-  else \
-    { \
-      if (dtp1 == DV_RDF || dtp2 == DV_RDF) \
         { \
-          if (dtp1 == DV_RDF) \
-            box1 = ((rdf_box_t *)(box1))->rb_box; \
-          if (dtp2 == DV_RDF) \
-            box2 = ((rdf_box_t *)(box2))->rb_box; \
-          goto retry_rdf_boxes; \
+        case DV_LONG_INT: \
+        int_case: \
+          if (isdiv && 0 == *(boxint *) &dn2) \
+            sqlr_new_error ("22012", "SR084", "Division by 0."); \
+          if (target) \
+            return (qst_set_long (qst, target, \
+                (*(boxint *) &dn1 op * (boxint *) &dn2)), (caddr_t) 0); \
+          return (box_num (*(boxint *) &dn1 op * (boxint *) &dn2)); \
+        case DV_SINGLE_FLOAT: \
+          if (isdiv && 0 == *(float *) &dn2) \
+            sqlr_new_error ("22012", "SR085", "Division by 0."); \
+          if (target) \
+            return (qst_set_float (qst, target, \
+                (*(float *) &dn1 op * (float *) &dn2)), (caddr_t) 0); \
+          return (box_float (*(float *) &dn1 op * (float *) &dn2)); \
+        case DV_DOUBLE_FLOAT: \
+          if (isdiv && 0 == *(double*) &dn2) \
+            sqlr_new_error ("22012", "SR086", "Division by 0."); \
+          if (target) \
+            return (qst_set_double (qst, target, (*(double*) &dn1 op *(double*) &dn2)), (caddr_t) 0); \
+          return (box_double (*(double*) &dn1 op *(double*) &dn2)); \
+        case DV_NUMERIC: \
+          return (numeric_bin_op (num_op, (numeric_t) &dn1, (numeric_t) &dn2, qst, target)); \
         } \
-      if ((NULL != dt_op) && ((DV_DATETIME == dtp1) || (DV_DATETIME == dtp2))) \
+    } \
+  if (dtp1 == DV_RDF || dtp2 == DV_RDF) \
+    { \
+      if (dtp1 == DV_RDF) \
         { \
-          caddr_t err = NULL; \
-          caddr_t res = ((arithm_dt_operation_t *)(dt_op)) (box1, box2, &err); \
-          if (NULL == err) \
-            { \
-              if (target) \
-                return (qst_set (qst, target, res), (caddr_t)0); \
-              return res; \
-            } \
-          if (((query_instance_t *)qst)->qi_query->qr_no_cast_error) \
-            { \
-              dk_free_tree (err); \
-              goto null_result; \
-            } \
-          sqlr_resignal (err); \
+          if (!((rdf_box_t *)box1)->rb_is_complete) \
+            sqlr_new_error ("22003", "SR674", "Incomplete RDF box as left argument of arithmetic operation '%s'", opsymbol); \
+          box1 = ((rdf_box_t *)(box1))->rb_box; \
+        } \
+      if (dtp2 == DV_RDF) \
+        { \
+          if (!((rdf_box_t *)box2)->rb_is_complete) \
+            sqlr_new_error ("22003", "SR674", "Incomplete RDF box as right argument of arithmetic operation '%s'", opsymbol); \
+          box2 = ((rdf_box_t *)(box2))->rb_box; \
+        } \
+      goto retry_rdf_boxes; \
+    } \
+  if ((NULL != dt_op) && ((DV_DATETIME == dtp1) || (DV_DATETIME == dtp2) || IS_GENERIC_DURATION (box1) || IS_GENERIC_DURATION (box2))) \
+    { \
+      caddr_t err = NULL; \
+      caddr_t res = ((arithm_dt_operation_t *)(dt_op)) (box1, box2, &err); \
+      if (NULL == err) \
+        { \
+          if (target) \
+            return (qst_set (qst, target, res), (caddr_t)0); \
+          return res; \
         } \
       if (((query_instance_t *)qst)->qi_query->qr_no_cast_error) \
-        goto null_result; \
-      sqlr_new_error ("22003", "SR087", "Non numeric argument(s) to arithmetic operation '%s'.", opsymbol); \
+        { \
+          dk_free_tree (err); \
+          goto null_result; \
+        } \
+      sqlr_resignal (err); \
     } \
+  if (((query_instance_t *)qst)->qi_query->qr_no_cast_error) \
+    goto null_result; \
+  sqlr_new_error ("22003", "SR087", "Non numeric argument(s) to arithmetic operation '%s'", opsymbol); \
 null_result: \
   if (target) \
     { \
@@ -1097,10 +1108,10 @@ null_result:
   return (dk_alloc_box (0, DV_DB_NULL));
 }
 
-ARTM_BIN_FUNC (box_add, "+", +, numeric_add		, arithm_dt_add		, 0)
-ARTM_BIN_FUNC (box_sub, "-", -, numeric_subtract	, arithm_dt_subtract	, 0)
-ARTM_BIN_FUNC (box_mpy, "*", *, numeric_multiply	, NULL			, 0)
-ARTM_BIN_FUNC (box_div, "/", /, numeric_divide		, NULL			, 1)
+ARTM_BIN_FUNC (box_add, "+", +, numeric_add		, arithm_dt_add			, 0)
+ARTM_BIN_FUNC (box_sub, "-", -, numeric_subtract		, arithm_dt_subtract		, 0)
+ARTM_BIN_FUNC (box_mpy, "*", *, numeric_multiply	, arithm_duration_multiply	, 0)
+ARTM_BIN_FUNC (box_div, "/", /, numeric_divide		, arithm_duration_divide	, 1)
 
 
 caddr_t
@@ -1153,8 +1164,8 @@ dvc_int_double (int64 i, double d)
       return NUM_COMPARE (i, i2);
     }
   if (d > -1 && d < 1)
-    return NUM_COMPARE (((double) i), d);
-  r = NUM_COMPARE ((double) i, d);
+    return NUM_COMPARE_DBL (((double) i), d);
+  r = NUM_COMPARE_DBL ((double) i, d);
   if (DVC_MATCH == r)
     {
       /* if int and double look like eq and the double is outside the int precise int range, then the int is seen as farther from zero */
@@ -1177,6 +1188,8 @@ dvc_num_double (numeric_t num1, double d2)
       return numeric_compare_dvc ((numeric_t) num1, (numeric_t) num2);
     }
   numeric_to_double (num1, &d1);
+  if (isnan (d2))
+    return DVC_LESS;
   if (d1 == d2)
     {
       if (d2 > MIN_INT_DOUBLE && d2 < MAX_INT_DOUBLE)
@@ -1235,7 +1248,7 @@ dv_num_compare (numeric_t dn1, numeric_t dn2, dtp_t dtp1, dtp_t dtp2)
 	case DV_LONG_INT:
 	  return NUM_COMPARE (*(int64 *) dn1, *(int64 *) dn2);
 	case DV_DOUBLE_FLOAT:
-	  return NUM_COMPARE (*(double *) dn1, *(double *) dn2);
+	  return NUM_COMPARE_DBL (*(double *) dn1, *(double *) dn2);
 	case DV_NUMERIC:
 	  return (numeric_compare_dvc ((numeric_t) dn1, (numeric_t) dn2));
 	default:
@@ -1592,8 +1605,12 @@ name (int64* res, int64 * l, int64* r, int n) \
   int inx; \
   if (is_div) \
     for (inx = 0; inx < n; inx++) \
-      if (0 == ((tp*)r)[inx]) \
-	sqlr_new_error ("22012", "SR084", "Division by 0."); \
+      { \
+        if (0 == ((tp*)r)[inx]) \
+          sqlr_new_error ("22012", "SR084", "Division by 0."); \
+        if (2 == is_div && ((tp*)l)[inx] <= INT64_MIN && -1 == ((tp*)r)[inx]) \
+          sqlr_new_error ("22012", "SR084", "Int64 overflow."); \
+      } \
   for (inx = 0; 0 && inx <= n - 2 * vec_len; inx += 2 * vec_len) \
     { \
       *(vect*)&((tp*)res)[inx] = *(vect*)&((tp*)l)[inx] op  *(vect*)&((tp*)r)[inx]; \
@@ -1616,7 +1633,7 @@ ARTM_VEC (artm_mpy_int, int64, v2di_t, 2, *, 0);
 ARTM_VEC (artm_mpy_float, float, v4sf_t, 4, *, 0);
 ARTM_VEC (artm_mpy_double, double, v2df_t, 2, *, 0);
 
-ARTM_VEC (artm_div_int, int64, v2di_t, 2, /, 1);
+ARTM_VEC (artm_div_int, int64, v2di_t, 2, /, 2);
 ARTM_VEC (artm_div_float, float, v4sf_t, 4, /, 1);
 ARTM_VEC (artm_div_double, double, v2df_t, 2, /, 1);
 
