@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -49,6 +49,8 @@
 #include <openssl/err.h>
 #include <openssl/ssl.h>
 
+#include "util/ssl_compat.h"
+
 #define DO_BOX_ALG_1(A_TYPE,A_NAME,A_PREFIX) \
 void \
 box_##A_NAME##_1 (caddr_t box, A_TYPE##_CTX * ctx) \
@@ -181,21 +183,25 @@ box_hmac (caddr_t box, caddr_t key, int alg)
   unsigned char temp[EVP_MAX_MD_SIZE];
   unsigned int size = 0;
   caddr_t res = NULL;
-  HMAC_CTX ctx;
+  HMAC_CTX *ctx;
   const EVP_MD *md = EVP_sha1 ();
 
   if (alg == 1)
     md = EVP_ripemd160 ();
 
-  HMAC_Init (&ctx, key, box_length (key) - DV_STRINGP (key) ? 1 : 0, md);
-  box_hmac_1 (box, &ctx);
-  HMAC_Final (&ctx, temp, &size);
+  if ((ctx = HMAC_CTX_new()) == NULL)
+    return NULL;
+
+  HMAC_Init_ex (ctx, key, box_length (key) - DV_STRINGP (key) ? 1 : 0, md, NULL);
+  box_hmac_1 (box, ctx);
+  HMAC_Final (ctx, temp, &size);
   if (size)
     {
       res = dk_alloc_box (size + 1, DV_SHORT_STRING);
       memcpy (res, temp, size);
       res[size] = 0;
     }
+  HMAC_CTX_free (ctx);
   return res;
 }
 
@@ -345,16 +351,14 @@ asn1_parse_to_xml (BIO * bp, unsigned char **pp, long length, int offset, int de
 	    }
 	  else if (tag == V_ASN1_BOOLEAN)
 	    {
-	      int ii;
-
 	      opp = op;
-	      ii = d2i_ASN1_BOOLEAN (NULL, (const unsigned char **)&opp, len + hl);
-	      if (ii < 0)
+	      if (len != 1)
 		{
 		  if (BIO_write (bp, "Bad boolean\n", 12))
 		    goto end;
 		}
-	      BIO_printf (bp, "%d", ii);
+	      if (len > 0)
+	        BIO_printf (bp, "%u", p[0]);
 	    }
 	  else if (tag == V_ASN1_BMPSTRING)
 	    {
@@ -415,7 +419,7 @@ asn1_parse_to_xml (BIO * bp, unsigned char **pp, long length, int offset, int de
 		}
 	      if (os != NULL)
 		{
-		  M_ASN1_OCTET_STRING_free (os);
+		  ASN1_STRING_free (os);
 		  os = NULL;
 		}
 	    }
@@ -448,7 +452,7 @@ asn1_parse_to_xml (BIO * bp, unsigned char **pp, long length, int offset, int de
 		  if (BIO_write (bp, "BAD INTEGER", 11) <= 0)
 		    goto end;
 		}
-	      M_ASN1_INTEGER_free (bs);
+	      ASN1_INTEGER_free (bs);
 	    }
 	  else if (tag == V_ASN1_ENUMERATED)
 	    {
@@ -479,7 +483,7 @@ asn1_parse_to_xml (BIO * bp, unsigned char **pp, long length, int offset, int de
 		  if (BIO_write (bp, "BAD ENUMERATED", 11) <= 0)
 		    goto end;
 		}
-	      M_ASN1_ENUMERATED_free (bs);
+	      ASN1_ENUMERATED_free (bs);
 	    }
 	  else if (len > 0 && dump)
 	    {
@@ -515,7 +519,7 @@ end:
   if (o != NULL)
     ASN1_OBJECT_free (o);
   if (os != NULL)
-    M_ASN1_OCTET_STRING_free (os);
+    ASN1_OCTET_STRING_free (os);
   *pp = p;
   return (ret);
 }
@@ -545,7 +549,7 @@ bif_asn1_to_xml (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
       res = NEW_DB_NULL;
       goto err;
     }
-  res = box_dv_short_string (tmpbuf);
+  res = box_dv_short_nchars (tmpbuf, len);
 err:
   if (out != NULL)
     BIO_free_all (out);
@@ -718,12 +722,11 @@ dk_session_t *
 bio_to_strses (BIO * out_bio)
 {
   dk_session_t * ses = strses_allocate ();
-  char buf[4096], *to_free;
+  char buf[4096];
   char *ptr = NULL;
   int len = BIO_get_mem_data (out_bio, &ptr);
   int to_read = len, readed = 0;
 
-  to_free = ((BUF_MEM *) out_bio->ptr)->data;
   BIO_set_flags (out_bio, BIO_FLAGS_MEM_RDONLY);
   CATCH_WRITE_FAIL (ses)
     {
@@ -735,7 +738,7 @@ bio_to_strses (BIO * out_bio)
       } while (to_read > 0);
     }
   END_WRITE_FAIL (ses);
-  ((BUF_MEM *) out_bio->ptr)->data = to_free;
+  BIO_reset (out_bio);
   BIO_clear_flags (out_bio, BIO_FLAGS_MEM_RDONLY);
   return ses;
 }
@@ -746,6 +749,7 @@ bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t msg = bif_arg (qst, args, 0, "smime_verify");
   caddr_t certs = bif_array_arg (qst, args, 1, "smime_verify");
   int flags = 0;
+  int bio_readonly = 0;
   caddr_t ret = NULL;
 
   caddr_t err = NULL;
@@ -770,19 +774,19 @@ bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   if (DV_TYPE_OF (msg) == DV_STRING_SESSION)
     {
       in_bio = strses_to_bio ((dk_session_t *) msg);
-      to_free = ((BUF_MEM *) in_bio->ptr)->data;
       BIO_set_flags (in_bio, BIO_FLAGS_MEM_RDONLY);
+      bio_readonly = 1;
     }
   else
     in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
   if (in_bio)
     {
       p7 = SMIME_read_PKCS7 (in_bio, &data_bio);
-      if (to_free)
-	{
-	  ((BUF_MEM *) in_bio->ptr)->data = to_free;
-	  BIO_clear_flags (in_bio, BIO_FLAGS_MEM_RDONLY);
-	}
+      if (bio_readonly)
+        {
+          BIO_reset (in_bio);
+          BIO_clear_flags (in_bio, BIO_FLAGS_MEM_RDONLY);
+        }
       BIO_free (in_bio);
     }
 
@@ -924,13 +928,16 @@ bif_smime_sign (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
     }
 
   certs = sk_X509_new_null ();
-  if (store && store->objs)
+
+  if (store && X509_STORE_get0_objects(store))
     {
-      for (inx = 0; inx < sk_X509_OBJECT_num (store->objs); inx++)
+      STACK_OF(X509_OBJECT) *store_objs = X509_STORE_get0_objects(store);
+
+      for (inx = 0; inx < sk_X509_OBJECT_num (store_objs); inx++)
 	{
-	  X509_OBJECT *obj = sk_X509_OBJECT_value (store->objs, inx);
-	  if (obj->type == X509_LU_X509)
-	    sk_X509_push (certs, X509_dup (obj->data.x509));
+	  X509_OBJECT *obj = sk_X509_OBJECT_value (store_objs, inx);
+	  if (X509_OBJECT_get_type(obj) == X509_LU_X509)
+	    sk_X509_push (certs, X509_dup (X509_OBJECT_get0_X509(obj)));
 	}
 
     }
@@ -1005,15 +1012,19 @@ bif_smime_encrypt (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
     sqlr_new_error ("42000", "CR006", "No recipient certificates");
 
   certs = sk_X509_new_null ();
-  if (store && store->objs)
+
+  if (store && X509_STORE_get0_objects(store))
     {
-      for (inx = 0; inx < sk_X509_OBJECT_num (store->objs); inx++)
+      STACK_OF(X509_OBJECT) *store_objs = X509_STORE_get0_objects(store);
+
+      for (inx = 0; inx < sk_X509_OBJECT_num (store_objs); inx++)
 	{
-	  X509_OBJECT *obj = sk_X509_OBJECT_value (store->objs, inx);
-	  if (obj->type == X509_LU_X509)
-	    sk_X509_push (certs, X509_dup (obj->data.x509));
+	  X509_OBJECT *obj = sk_X509_OBJECT_value (store_objs, inx);
+	  if (X509_OBJECT_get_type(obj) == X509_LU_X509)
+	    sk_X509_push (certs, X509_dup (X509_OBJECT_get0_X509(obj)));
 	}
     }
+
   if (store)
     X509_STORE_free (store);
   in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
@@ -1176,12 +1187,13 @@ bif_pem_certificates_to_array (caddr_t * qst, caddr_t * err_ret, state_slot_t **
 
 
 static int
-x509_certificate_verify_cb (int ok, X509_STORE_CTX * ctx)
+x509_certificate_verify_cb (int ok, X509_STORE_CTX * x509_store)
 {
-  char *opts = (char *) X509_STORE_CTX_get_app_data (ctx);
+  char *opts = (char *) X509_STORE_CTX_get_ex_data(x509_store, SSL_get_ex_data_X509_STORE_CTX_idx());
+
   if (!ok && opts)
     {
-      switch (ctx->error)
+      switch (X509_STORE_CTX_get_error(x509_store))
 	{
 	case X509_V_ERR_CERT_HAS_EXPIRED:
 	  if (strstr (opts, "expired"))
@@ -1246,11 +1258,7 @@ bif_x509_certificate_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** a
     mem_bio = BIO_new_mem_buf (ca, box_length (ca) - 1);
     if (NULL != strstr (ca, PEM_STRING_X509))
       {
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L
 	cacert = (X509 *)PEM_ASN1_read_bio ((d2i_of_void *)d2i_X509, PEM_STRING_X509, mem_bio, NULL, NULL, NULL);
-#else
-	cacert = (X509 *)PEM_ASN1_read_bio ((char *(*)())d2i_X509, PEM_STRING_X509, mem_bio, NULL, NULL, NULL);
-#endif
       }
     else
       cacert = d2i_X509_bio (mem_bio, NULL);
@@ -1270,24 +1278,20 @@ bif_x509_certificate_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** a
       goto err_ret;
     }
 
-#if (OPENSSL_VERSION_NUMBER < 0x00907000L)
-  X509_STORE_CTX_init (csc, cert_ctx, cert, uchain);
-#else
   if (!X509_STORE_CTX_init (csc, cert_ctx, cert, uchain))
     {
       *err_ret = srv_make_new_error ("22023", "CR018", "Can not initialize X509 verification context");
       goto err_ret;
     }
-#endif
 
-  X509_STORE_CTX_set_app_data (csc, (void *) opts);
+  X509_STORE_CTX_set_ex_data (csc, SSL_get_ex_data_X509_STORE_CTX_idx(), (void *) opts);
 
   i = X509_verify_cert (csc);
 
   if (!i)
     {
       const char *err_str;
-      err_str = X509_verify_cert_error_string (csc->error);
+      err_str = X509_verify_cert_error_string (X509_STORE_CTX_get_error(csc));
       *err_ret = srv_make_new_error ("22023", "CR015", "X509 error: %s", err_str);
     }
 
@@ -1308,23 +1312,102 @@ err_ret:
 #define VIRT_CERT_EXT "2.16.840.1.1113.1"
 
 static caddr_t
-BN_box (BIGNUM * x)
+BN_box (const BIGNUM * x)
 {
   size_t buf_len, n;
   caddr_t buf;
   buf_len = (size_t) BN_num_bytes (x);
   if (buf_len <= BN_BYTES)
-    buf = box_num ((unsigned long) x->d[0]);
+    buf = box_num (BN_get_word (x));
   else
     {
       buf = dk_alloc_box (buf_len, DV_BIN);
-      n = BN_bn2bin (x, (unsigned char *) buf);
-      if (n != buf_len)
-	GPF_T;
+      if (buf)
+	{
+	  n = BN_bn2bin (x, (unsigned char *) buf);
+	  if (n != buf_len)
+	    GPF_T;
+	}
     }
   return buf;
 }
 
+#ifdef _SSL
+caddr_t
+get_client_pem_certificate (caddr_t *qst, caddr_t *err_ret)
+{
+  query_instance_t *qi = (query_instance_t *)qst;
+  ws_connection_t *ws = qi->qi_client->cli_ws;
+  caddr_t pem_cert = NULL;
+  char *ptr;
+  SSL *ssl = NULL;
+  X509 *cert = NULL;
+  BIO *in = NULL;
+  session_t * ses = (ws ? ws->ws_session->dks_session : (qi->qi_client->cli_session ? qi->qi_client->cli_session->dks_session : NULL));
+
+  if (ses)
+    ssl = (SSL *) tcpses_get_ssl (ses);
+
+  if (ssl)
+    cert = SSL_get_peer_certificate (ssl);
+
+  if (cert)
+    {
+      in = BIO_new (BIO_s_mem());
+      if (!in)
+        {
+          char err_buf[512];
+          if (err_ret)
+            *err_ret = srv_make_new_error ("22005", "SR402", "Cannot allocate temp space. SSL error : %s",
+              get_ssl_error_text (err_buf, sizeof (err_buf)));
+          return NULL;
+        }
+      BIO_reset(in);
+      PEM_write_bio_X509 (in, cert);
+      pem_cert = dk_alloc_box (BIO_get_mem_data (in, &ptr) + 1, DV_SHORT_STRING);
+      memcpy (pem_cert, ptr, box_length (pem_cert) - 1);
+      pem_cert[box_length (pem_cert) - 1] = 0;
+      BIO_free (in);
+      return pem_cert;
+    }
+  else if (!cert && NULL != ws && NULL != ws->ws_lines && (NULL != (pem_cert = ws_mime_header_field (ws->ws_lines, "X-SSL-CERT", NULL, 1))))
+    return pem_cert;
+  return NULL;
+}
+
+X509 *
+get_client_certificate (caddr_t *qst, caddr_t *err_ret)
+{
+  query_instance_t *qi = (query_instance_t *)qst;
+  caddr_t pem_cert;
+  ws_connection_t *ws = qi->qi_client->cli_ws;
+  X509 *cert = NULL;
+  SSL *ssl = NULL;
+  BIO *in = NULL;
+  session_t * ses = (ws ? ws->ws_session->dks_session : (qi->qi_client->cli_session ? qi->qi_client->cli_session->dks_session : NULL));
+  if (ses)
+    ssl = (SSL *) tcpses_get_ssl (ses);
+
+  cert = SSL_get_peer_certificate (ssl);
+  if (NULL == cert && NULL != ws && ws->ws_lines && (pem_cert = ws_mime_header_field (ws->ws_lines, "X-SSL-CERT", NULL, 1)))
+    {
+      in = BIO_new_mem_buf (pem_cert, box_length (pem_cert) - 1);
+      if (!in)
+        {
+          char err_buf[512];
+          if (err_ret)
+            *err_ret = srv_make_new_error ("22005", "SR402", 
+                "Cannot allocate temp space. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+          return NULL;
+        }
+      cert = PEM_read_bio_X509 (in, NULL, NULL, NULL);
+      BIO_free (in);
+    }
+  return cert;
+}
+#endif
+
+
 /*
    1 - info type
    2 - certificate
@@ -1385,9 +1468,9 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
     }
   else
     {
-      if (!ssl)
+      cert = get_client_certificate (qst, NULL);
+      if (!cert)
 	sqlr_new_error ("22023", "SR309", "Non-encrypted session");
-      cert = SSL_get_peer_certificate (ssl);
     }
   if (!cert)
     return dk_alloc_box (0, DV_DB_NULL);
@@ -1498,7 +1581,7 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	int i;
 	char tmp[1024];
 	char *ext_oid = (char *) (BOX_ELEMENTS (args) > 4 ? bif_string_arg (qst, args, 4, "get_certificate_info") : VIRT_CERT_EXT);
-	STACK_OF (X509_EXTENSION) * exts = cert->cert_info->extensions;
+	const STACK_OF (X509_EXTENSION) * exts = X509_get0_extensions(cert);
 	for (i = 0; i < sk_X509_EXTENSION_num (exts); i++)
 	  {
 	    X509_EXTENSION *ex = sk_X509_EXTENSION_value (exts, i);
@@ -1510,7 +1593,7 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 		char *data_ptr;
 		BIO *mem = BIO_new (BIO_s_mem ());
 		if (!X509V3_EXT_print (mem, ex, 0, 0))
-		  M_ASN1_OCTET_STRING_print (mem, ex->value);
+		  ASN1_STRING_print (mem, (ASN1_STRING *) X509_EXTENSION_get_data(ex));
 		len = BIO_get_mem_data (mem, &data_ptr);
 		if (len > 0 && data_ptr)
 		  {
@@ -1536,23 +1619,35 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	EVP_PKEY *k = X509_get_pubkey (cert);
 	if (k)
 	  {
+	    switch (EVP_PKEY_id(k))
+	      {
 #ifdef EVP_PKEY_RSA
-	    if (k->type == EVP_PKEY_RSA)
+	      case EVP_PKEY_RSA:
 	      {
-		RSA *x = k->pkey.rsa;
-		ret = list (3, box_dv_short_string ("RSAPublicKey"), BN_box (x->e), BN_box (x->n));
+		  const BIGNUM *n, *e;
+
+		  RSA_get0_key (EVP_PKEY_get0_RSA (k), &n, &e, NULL);
+
+		  ret = list (3, box_dv_short_string ("RSAPublicKey"), BN_box (e), BN_box (n));
+	          break;
 	      }
-	    else
 #endif
+
 #ifdef EVP_PKEY_DSA
-	    if (k->type == EVP_PKEY_DSA)
+	      case EVP_PKEY_DSA:
 	      {
-		DSA *x = k->pkey.dsa;
-		ret = list (2, box_dv_short_string ("DSAPublicKey"), BN_box (x->pub_key));
+		  const BIGNUM *pub_key;
+
+		  DSA_get0_key (EVP_PKEY_get0_DSA (k), &pub_key, NULL);
+
+		  ret = list (2, box_dv_short_string ("DSAPublicKey"), BN_box (pub_key));
+		  break;
 	      }
-	    else
 #endif
+
+	      default:
 	      *err_ret = srv_make_new_error ("42000", "XXXXX", "The certificate's public key not supported");
+	      }
 	    EVP_PKEY_free (k);
 	  }
 	else
@@ -1567,13 +1662,14 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	int n, i, len;
 	char *s, *data_ptr;
 	BIO *mem = BIO_new (BIO_s_mem ());
-	for (i = 0; NULL != subj && i < sk_X509_NAME_ENTRY_num(subj->entries); i++)
+
+	for (i = 0; NULL != subj && i < X509_NAME_entry_count(subj); i++)
 	  {
-	    ne = sk_X509_NAME_ENTRY_value(subj->entries,i);
-	    n = OBJ_obj2nid (ne->object);
+	    ne = X509_NAME_get_entry(subj, i);
+	    n = OBJ_obj2nid (X509_NAME_ENTRY_get_object(ne));
 	    if ((n == NID_undef) || ((s = (char *) OBJ_nid2sn (n)) == NULL))
 	      {
-		i2t_ASN1_OBJECT (buffer, sizeof (buffer), ne->object);
+		i2t_ASN1_OBJECT (buffer, sizeof (buffer), X509_NAME_ENTRY_get_object(ne));
 		s = buffer;
 	      }
 	    if (!strcmp (s, attr))
@@ -1582,9 +1678,10 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 		break;
 	      }
 	  }
+
 	if (ne_ret)
 	  {
-	    ASN1_STRING_print (mem, ne_ret->value);
+	    ASN1_STRING_print (mem, X509_NAME_ENTRY_get_data(ne_ret));
 	    len = BIO_get_mem_data (mem, &data_ptr);
 	    if (len > 0 && data_ptr)
 	      {
@@ -1605,17 +1702,18 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	dk_set_t set = NULL;
 	caddr_t val;
 	BIO *mem = BIO_new (BIO_s_mem ());
-	for (i = 0; NULL != subj && i < sk_X509_NAME_ENTRY_num(subj->entries); i++)
+
+	for (i = 0; NULL != subj && i < X509_NAME_entry_count(subj); i++)
 	  {
 	    val = NULL;
-	    ne = sk_X509_NAME_ENTRY_value(subj->entries,i);
-	    n = OBJ_obj2nid (ne->object);
+	    ne = X509_NAME_get_entry(subj, i);
+	    n = OBJ_obj2nid (X509_NAME_ENTRY_get_object(ne));
 	    if ((n == NID_undef) || ((s = (char *) OBJ_nid2sn (n)) == NULL))
 	      {
-		i2t_ASN1_OBJECT (buffer, sizeof (buffer), ne->object);
+		i2t_ASN1_OBJECT (buffer, sizeof (buffer), X509_NAME_ENTRY_get_object(ne));
 		s = buffer;
 	      }
-	    ASN1_STRING_print (mem, ne->value);
+	    ASN1_STRING_print (mem, X509_NAME_ENTRY_get_data(ne));
 	    len = BIO_get_mem_data (mem, &data_ptr);
 	    if (len > 0 && data_ptr)
 	      {
@@ -1635,11 +1733,13 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
       {
 	const unsigned char *s;
 	int i, n;
-	const ASN1_STRING *sig = cert->signature;
-	X509_ALGOR *sigalg = cert->sig_alg;
+	const ASN1_STRING *sig;
+	const X509_ALGOR *sigalg;
 	char buf[80];
 	caddr_t val;
 
+	X509_get0_signature (&sig, &sigalg, cert);
+
         i2t_ASN1_OBJECT(buf,sizeof (buf), sigalg->algorithm);
 
 	n = sig->length;
@@ -1659,22 +1759,29 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	BIO * b;
 	if (k)
 	  {
+
+	    switch (EVP_PKEY_id (k))
+	      {
 #ifdef EVP_PKEY_RSA
-	    if (k->type == EVP_PKEY_RSA)
+	      case EVP_PKEY_RSA:
 	      {
 		char *data_ptr;
 		int len;
-		RSA *x = k->pkey.rsa;
+		  RSA *x;
+		  x = EVP_PKEY_get0_RSA (k);
 		b = BIO_new (BIO_s_mem());
 		i2d_RSA_PUBKEY_bio (b, x);
 		len = BIO_get_mem_data (b, &data_ptr);
 		ret = dk_alloc_box (len, DV_BIN);
 		memcpy (ret, data_ptr, len);
 		BIO_free (b);
+		  break;
 	      }
-	    else
 #endif
+
+	      default:
 	      *err_ret = srv_make_new_error ("42000", "XXXXX", "The certificate's public key not supported");
+	      }
 	    EVP_PKEY_free (k);
 	  }
 	else
@@ -1697,17 +1804,26 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 static caddr_t
 bif_bin2hex (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  caddr_t bin = bif_bin_arg (qst, args, 0, "bin2hex");
-  caddr_t out = dk_alloc_box (2 * box_length (bin) + 1, DV_SHORT_STRING);
-  uint32 inx;
+  caddr_t bin = bif_arg (qst, args, 0, "bin2hex");
+  caddr_t out;
+  uint32 inx, len;
   char tmp[3];
+
+  if (DV_BIN == DV_TYPE_OF (bin))
+    len = box_length (bin);
+  else if (IS_STRING_DTP (DV_TYPE_OF (bin)))
+    len = box_length (bin) - 1;
+  else
+    sqlr_new_error ("22023", "ENC01", "Function bin2hex expects binary or string as a 1st argument");
+
+  out = dk_alloc_box (2 * len + 1, DV_SHORT_STRING);
   out[0] = 0;
-  for (inx = 0; inx < box_length (bin); inx++)
+  for (inx = 0; inx < len; inx++)
     {
       snprintf (tmp, sizeof (tmp), "%02x", (unsigned char) bin[inx]);
       strcat_box_ck (out, tmp);
     }
-  out[2 * box_length (bin)] = 0;
+  out[2 * len] = 0;
   return out;
 }
 
@@ -1722,7 +1838,7 @@ bif_hex2bin (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   if (!len)
     return NEW_DB_NULL;
   if (len % 2)
-    sqlr_new_error ("22023", "ENC..", "The input string must have a length multiple by two");
+    sqlr_new_error ("22023", "ENC02", "The input string must have a length multiple by two");
   out = dk_alloc_box (len / 2, DV_BIN);
   out[0] = 0;
   for (inx = 0; inx < len; inx += 2)
@@ -1730,7 +1846,7 @@ bif_hex2bin (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
       if (1 != sscanf (str+inx, "%02x", &tmp))
 	{
 	  dk_free_box (out);
-	  sqlr_new_error ("22023", "ENC..", "The input string does not contains hexadecimal string");
+	  sqlr_new_error ("22023", "ENC03", "The input string does not contains hexadecimal string");
 	}
       out [inx/2] = (unsigned char) tmp;
     }
@@ -1816,7 +1932,7 @@ bif_base36enc (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 static caddr_t
 bif_base36dec (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  boxint str = bif_string_arg (qst, args, 0, "base36dec");
+  caddr_t str = bif_string_arg (qst, args, 0, "base36dec");
   boxint ret;
   ret = strtoul(str, NULL, 36);
   return box_num (ret);
@@ -1863,10 +1979,8 @@ bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 void
 bif_crypto_init (void)
 {
-  bif_define_ex ("smime_verify", bif_smime_verify, BMD_RET_TYPE, &bt_varchar, BMD_DONE);
-  bif_define_ex ("smime_sign", bif_smime_verify, BMD_RET_TYPE, &bt_varchar, BMD_DONE);
-  bif_define_ex ("pem_certificates_to_array", bif_get_certificate_info, BMD_RET_TYPE, &bt_any, BMD_DONE);
-  bif_define_ex ("get_certificate_info", bif_get_certificate_info, BMD_RET_TYPE, &bt_any, BMD_DONE);
+  bif_define_ex ("smime_verify", bif_smime_verify, BMD_ALIAS, "smime_sign", BMD_RET_TYPE, &bt_varchar, BMD_DONE);
+  bif_define_ex ("get_certificate_info", bif_get_certificate_info, BMD_ALIAS, "pem_certificates_to_array", BMD_RET_TYPE, &bt_any, BMD_DONE);
 }
 #endif
 
