@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -29,6 +29,6 @@
 
 int cl_trx_inited;
 
-uint32 cl_last_wait_query;
+time_msec_t cl_last_wait_query;
 int cl_wait_query_queued;
 int32 cl_dead_w_interval = 12000;
