@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -57,6 +57,14 @@
 #define CE_DICT_MAX_VALUES ((PAGE_DATA_SZ / 3) * 2)
 #define COL_MAX_BYTES ((PAGE_DATA_SZ - 20) / 2) /* max bytes in non-blob col value before compression on column wise dependent part col */
 
+/* |0|1|2|3|4|5|6|7| 0xNN/0xNNNN 0xMM/0xMMMM */
+/*  bit0   : short flag
+ *  bit1-3 : dtp  int/iri/int64/string/any/nulls
+ *  bit4-7 : ce type
+ *  0xNN/0xNNNN : N values depends of bit0
+ *  0xMM/0xMMMM : if any/string N is bytes, M is n values
+ */
+
 #define CE_TYPE_MASK 0xf
 #define CE_DTP_MASK 0x70
 #define CE_INTLIKE(flags) (!(flags & CE_IS_STRING))
@@ -473,7 +481,6 @@ int  ce_n_values (db_buf_t ce);
 caddr_t mp_box_n_chars (mem_pool_t * mp, caddr_t b, int l);
 int  cr_n_rows (col_data_ref_t * cr);
 int  cr_n_bytes (col_data_ref_t * cr);
-void  itc_range (it_cursor_t * itc, row_no_t lower, row_no_t upper);
 int ce_search (it_cursor_t * itc, db_buf_t ce, row_no_t row_of_ce, int rc, int nth_key);
 
 /* return / rc of ce_search */
@@ -557,7 +564,9 @@ db_buf_t itc_dv_param (it_cursor_t * itc, int nth_key, db_buf_t ctmp);
 #define MAX_FIXED_DV_BYTES 50 /* max bytes in dv representation of fixed len box, e.g. date, decimal */
 int ce_dtp_compare (db_buf_t ce, dtp_t dtp);
 int ce_typed_vec_dtp_compare (db_buf_t ce, dtp_t dtp);
-void bing ();
+#if defined(DEBUG) | defined(MTX_DEBUG)
+void bing (void);
+#endif
 dtp_t ce_dtp_f (db_buf_t ce);
 int  asc_cmp (dtp_t * dv1, dtp_t * dv2);
 int  asc_cmp_delta (dtp_t * dv1, dtp_t * dv2, uint32 * num_ret, int is_int_delta);
