@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -57,6 +57,21 @@
 #define VIRTVARCLASS extern
 #endif
 
+#ifdef MALLOC_DEBUG
+#define DBG_NAME(nm) dbg_##nm
+#define DBG_PARAMS char *file, int line,
+#define DBG_ARGS file, line,
+#define DK_ALLOC(SIZE) dbg_malloc(DBG_ARGS (SIZE))
+#define DK_FREE(BOX,SIZE) dbg_free(DBG_ARGS (BOX))
+#else
+#define DBG_NAME(nm) nm
+#define DBG_PARAMS
+#define DBG_ARGS
+#define DK_ALLOC dk_alloc
+#define DK_FREE dk_free
+#endif
+
+
 #define timer_t opl_timer_t
 
 typedef unsigned char * db_buf_t;
@@ -575,7 +590,7 @@ struct dk_session_s
 
     short		dks_n_threads;
     /*! time of last usage (get_msec_real_time) - use for dropping idle HTTP keep alives */
-    uint32		dks_last_used;
+    time_msec_t	dks_last_used;
     /*! burst mode */
     dks_thread_state_t  dks_thread_state;
     /*! web server thread associated to this if ws computation pending. Used to cancel upon client disconnect */
@@ -618,20 +633,6 @@ thread_t * thread_current (void);
 void * thread_setattr (thread_t *self, void *key, void *value);
 int strnicmp (const char *s1, const char *s2, size_t n);
 
-#ifdef MALLOC_DEBUG
-#define DBG_NAME(nm) dbg_##nm
-#define DBG_PARAMS char *file, int line,
-#define DBG_ARGS file, line,
-#define DK_ALLOC(SIZE) dbg_malloc(DBG_ARGS (SIZE))
-#define DK_FREE(BOX,SIZE) dbg_free(DBG_ARGS (BOX))
-#else
-#define DBG_NAME(nm) nm
-#define DBG_PARAMS
-#define DBG_ARGS
-#define DK_ALLOC dk_alloc
-#define DK_FREE dk_free
-#endif
-
 typedef struct s_node_s s_node_t, *dk_set_t;
 struct s_node_s
 {
