@@ -8,7 +8,7 @@
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -61,41 +61,7 @@ create table MAIL_ATTACHMENT (
    	)
 ;
 
---#IF VER=5
---!AFTER
-alter table MAIL_ATTACHMENT add MA_BLOG_ID varchar
-;
 
-create procedure DB.DBA.UPGRADE_MAIL_MSG ()
-{
-  declare id integer;
-
-  if (not exists (select 1 from SYS_COLS where "COLUMN" = 'MM_BODY_ID' and "TABLE" = 'DB.DBA.MAIL_MESSAGE'))
-    exec ('alter table DB.DBA.MAIL_MESSAGE add MM_BODY_ID integer identity');
-
-  id := sequence_set ('DB.DBA.DB.DBA.MAIL_MESSAGE.MM_BODY_ID',0,2);
-  if (id = 0)
-    sequence_set ('DB.DBA.DB.DBA.MAIL_MESSAGE.MM_BODY_ID',1,1);
-}
-;
-
-
---!AFTER
-DB.DBA.UPGRADE_MAIL_MSG ()
-;
-
---!AFTER
-alter table MAIL_MESSAGE add MM_MOBLOG varchar(50) default NULL
-;
-
---!AFTER
-alter table MAIL_MESSAGE add MM_MSG_ID varchar default NULL
-;
---#ENDIF
-
---#IF VER=5
---!AFTER_AND_BEFORE DB.DBA.MAIL_MESSAGE MM_MSG_ID !
---#ENDIF
 create trigger MAIL_MESSAGE_I after insert on DB.DBA.MAIL_MESSAGE
   {
     if (__proc_exists ('BLOG.DBA.BLOG_MOBLOG_PROCESS_MSG'))
@@ -103,9 +69,6 @@ create trigger MAIL_MESSAGE_I after insert on DB.DBA.MAIL_MESSAGE
   }
 ;
 
---#IF VER=5
---!AFTER_AND_BEFORE DB.DBA.MAIL_MESSAGE MM_MSG_ID !
---#ENDIF
 create trigger MAIL_MESSAGE_U after update on DB.DBA.MAIL_MESSAGE referencing old as O, new as N
   {
     if (__proc_exists ('BLOG.DBA.BLOG_MOBLOG_PROCESS_MSG'))
@@ -225,15 +188,9 @@ MM_FEED_PART (inout vb any, inout mb any, inout body varchar, inout id integer,
 ;
 
 
---#IF VER=5
---!AFTER __PROCEDURE__ DB.DBA.VT_CREATE_TEXT_INDEX !
---#ENDIF
 DB.DBA.vt_create_text_index ('DB.DBA.MAIL_MESSAGE', 'MM_BODY', 'MM_BODY_ID', 2, 0, null, 1, '*ini*', '*ini*')
 ;
 
---#IF VER=5
---!AFTER
---#ENDIF
 DB.DBA.vt_create_ftt ('DB.DBA.MAIL_MESSAGE', null, null, 2)
 ;
 
@@ -291,7 +248,7 @@ create procedure MIME_BODY (in _parts any)
   declare _part any;
 
   _bnd := concat ('-','-','-','-', md5 (cast (now () as varchar)));
-  if (__tag (_parts) = 193)
+  if (__tag (_parts) = __tag of vector)
     _c_type := sprintf ('multipart/mixed; boundary="%s"', _bnd);
   else if (isstring (_parts))
     _c_type := 'text/plain';
@@ -333,9 +290,6 @@ create procedure MIME_BODY (in _parts any)
 }
 ;
 
---#IF VER=5
---!AFTER_AND_BEFORE DB.DBA.MAIL_MESSAGE MM_MSG_ID !
---#ENDIF
 create procedure NEW_MAIL (in _uid varchar, in __msg any)
 {
   declare _id, dummy integer;
