@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -28,16 +28,22 @@
 #include "sqlnode.h"
 #include "sqlbif.h"
 #include "rdf_core.h"
+#include "sqlo.h"
+#include "rdfinf.h"
 #include "aqueue.h"
 #include "sqlbif.h"
 #include "security.h"
+#include "date.h"
 
 
 query_t *rl_queries[10];
 query_t * rl_del_qrs[10];
-query_t *rl_all_keys_qr;
+query_t * rl_all_keys_qr; /* do we need this, obsolete? */
 query_t *rl_graph_words_qr;
+query_t * rl_labels_qr;
+query_t * rl_labels_qr_inx;
 state_slot_t ssl_set_no_dummy;
+#define ssl_int_dummy ssl_set_no_dummy /* infact it is int */
 state_slot_t ssl_iri_dummy;
 state_slot_t ssl_any_dummy;
 extern resource_t *clib_rc;
@@ -88,7 +94,7 @@ cu_local_dispatch (cucurbit_t * cu, value_state_t * vs, cu_func_t * cf, caddr_t
       clo->clo_set_no = mp_data_col (clrg->clrg_pool, &ssl_set_no_dummy, init_sz);
       clo->_.call.func = cf->cf_proc;
       clo->_.call.params = (caddr_t *) mp_box_copy (clrg->clrg_pool, val);
-      DO_BOX (caddr_t, arg, inx, args)
+      DO_BOX_0 (caddr_t, arg, inx, args)
       {
 	ssl_dummy.ssl_sqt = cf->cf_arg_sqt[inx];
 	ssl_dummy.ssl_dc_dtp = sqt_dc_dtp (&ssl_dummy.ssl_sqt);
@@ -117,6 +123,8 @@ cu_rl_local_exec (cucurbit_t * cu)
   cl_req_group_t *clrg = cu->cu_clrg;
   QNCAST (query_instance_t, qi, cu->cu_qst);
   cll_in_box_t *clib;
+  if (!rdf_rpid64_mode)
+    sqlr_new_error ("42000", "CL...", "Can not use dpipe IRI operations before upgrading the RDF_IRI table to 64-bit prefix IDs");
   if (!clrg->clrg_clibs)
     return;			/* this is possible if all rows in dpipe are without lits with id  and all iris came from cache, so iriu resolutions or literals with id */
   clib = (cll_in_box_t *) clrg->clrg_clibs->data;
@@ -160,56 +168,72 @@ cu_rl_local_exec (cucurbit_t * cu)
       {
 	ptrlong nth = dc_any_value (sets, inx);
 	int64 id = dc_any_value (res, inx);
-	value_state_t *vs = gethash ((void *) nth, cu->cu_seq_no_to_vs);
+	value_state_t *vs = (value_state_t *) gethash ((void *) nth, cu->cu_seq_no_to_vs);
 	if (is_ro)
 	  {
-	    if (DV_RDF == DV_TYPE_OF (vs->vs_org_value))
+	    switch (DV_TYPE_OF (vs->vs_org_value))
 	      {
-		rdf_box_t *rb = (rdf_box_t *) vs->vs_org_value;
-		rb->rb_ro_id = id;
-		if (rb->rb_is_text_index && cu->cu_rdf_load_mode != RDF_LD_MULTIGRAPH)
-		  {
-		    client_connection_t *cli = qi->qi_client;
-		    char *g_dict_name = "g_dict";
-		    id_hash_iterator_t **dict_place =
-			(id_hash_iterator_t **) id_hash_get (cli->cli_globals, (caddr_t) & g_dict_name);
-		    id_hash_t *ht;
-		    if (dict_place && (ht = dict_ht (*dict_place)))
-		      {
-			caddr_t one = (caddr_t) 1, id_box;
-			if (ht->ht_mp)
-			  id_box = mp_box_num ((mem_pool_t *) (ht->ht_mp), id);
-			else
-			  id_box = box_num (id);
-			id_hash_set (ht, (caddr_t) & id_box, (caddr_t) & one);
-		      }
-		  }
-		cu_set_value (cu, vs, box_copy_tree ((caddr_t) rb));
-	      }
-	    else if (DV_STRING == DV_TYPE_OF (vs->vs_org_value) || IS_WIDE_STRING_DTP (DV_TYPE_OF (vs->vs_org_value)))
-	      {
-		rdf_box_t *rb = rb_allocate ();
-		rb->rb_ro_id = id;
-		rb->rb_type = RDF_BOX_DEFAULT_TYPE;
-		rb->rb_lang = RDF_BOX_DEFAULT_LANG;
-		if (cu->cu_rdf_load_mode != RDF_LD_MULTIGRAPH)
-		  {
-		    client_connection_t *cli = qi->qi_client;
-		    char *g_dict_name = "g_dict";
-		    id_hash_iterator_t **dict_place =
-			(id_hash_iterator_t **) id_hash_get (cli->cli_globals, (caddr_t) & g_dict_name);
-		    id_hash_t *ht;
-		    if (dict_place && (ht = dict_ht (*dict_place)))
-		      {
-			caddr_t one = (caddr_t) 1, id_box;
-			if (ht->ht_mp)
-			  id_box = mp_box_num ((mem_pool_t *) (ht->ht_mp), id);
-			else
-			  id_box = box_num (id);
-			id_hash_set (ht, (caddr_t) & id_box, (caddr_t) & one);
-		      }
-		  }
-		cu_set_value (cu, vs, (caddr_t) rb);
+	      case DV_RDF:
+		{
+		  rdf_box_t *rb = (rdf_box_t *) vs->vs_org_value;
+		  rb->rb_ro_id = id;
+		  if (rb->rb_is_text_index && cu->cu_rdf_load_mode != RDF_LD_MULTIGRAPH)
+		    {
+		      client_connection_t *cli = qi->qi_client;
+		      const char *g_dict_name = "g_dict";
+		      id_hash_iterator_t **dict_place =
+			  (id_hash_iterator_t **) id_hash_get (cli->cli_globals, (caddr_t) & g_dict_name);
+		      id_hash_t *ht;
+		      if (dict_place && (ht = dict_ht (*dict_place)))
+			{
+			  caddr_t one = (caddr_t) 1, id_box;
+			  if (ht->ht_mp)
+			    id_box = mp_box_num ((mem_pool_t *) (ht->ht_mp), id);
+			  else
+			    id_box = box_num (id);
+			  id_hash_set (ht, (caddr_t) & id_box, (caddr_t) & one);
+			}
+		    }
+		  cu_set_value (cu, vs, box_copy_tree ((caddr_t) rb));
+		  break;
+		}
+	      case DV_STRING:
+	      case DV_WIDE:
+	      case DV_BLOB_HANDLE:
+		{
+		  rdf_box_t *rb = rb_allocate ();
+		  rb->rb_ro_id = id;
+		  rb->rb_type = RDF_BOX_DEFAULT_TYPE;
+		  rb->rb_lang = RDF_BOX_DEFAULT_LANG;
+		  if (cu->cu_rdf_load_mode != RDF_LD_MULTIGRAPH)
+		    {
+		      client_connection_t *cli = qi->qi_client;
+		      const char *g_dict_name = "g_dict";
+		      id_hash_iterator_t **dict_place =
+			  (id_hash_iterator_t **) id_hash_get (cli->cli_globals, (caddr_t) & g_dict_name);
+		      id_hash_t *ht;
+		      if (dict_place && (ht = dict_ht (*dict_place)))
+			{
+			  caddr_t one = (caddr_t) 1, id_box;
+			  if (ht->ht_mp)
+			    id_box = mp_box_num ((mem_pool_t *) (ht->ht_mp), id);
+			  else
+			    id_box = box_num (id);
+			  id_hash_set (ht, (caddr_t) & id_box, (caddr_t) & one);
+			}
+		    }
+		  cu_set_value (cu, vs, (caddr_t) rb);
+		  break;
+		}
+	      case DV_GEO:
+		{
+		  rdf_box_t *rb = rb_allocate ();
+		  rb->rb_ro_id = id;
+		  rb->rb_type = RDF_BOX_GEO_TYPE;
+		  rb->rb_lang = RDF_BOX_DEFAULT_LANG;
+		  cu_set_value (cu, vs, (caddr_t) rb);
+		  break;
+		}
 	      }
 	  }
 	else
@@ -254,6 +278,88 @@ aq_rl_del_key_func (caddr_t av, caddr_t * err_ret)
   return aq_rl_key_func_1 (av, err_ret, 0);
 }
 
+caddr_t cu_rdf_ins_label_normalize (mem_pool_t * mp, caddr_t lbl);
+
+void
+rl_rdf_ins_label (cucurbit_t * cu, caddr_t * quad, caddr_t ** ret_dc_array)
+{
+  static rdf_inf_ctx_t * ctx;
+  static caddr_t err;		/* if init fails do not try every time, supposed to do init once */
+  QNCAST (query_instance_t, qi, cu->cu_qst);
+  dbe_table_t * tbl = sch_name_to_table (wi_inst.wi_schema, "DB.DBA.RDF_LABEL");
+  caddr_t oval;
+  cl_req_group_t * clrg = cu->cu_clrg;
+  /* RL_O any, RL_RO_ID bigint, RL_TEXT varchar, RL_LANG int */
+  data_col_t *rl_o_dc = NULL, *rl_ro_id_dc = NULL, *rl_text_dc = NULL, *rl_lang_dc = NULL;
+
+  if (!tbl || !virtuoso_server_initialized)
+    return;
+
+  if (!ctx && !err)
+    {
+      caddr_t ctx_name = box_string (rdf_label_inf_name);
+      cl_rdf_inf_init (CU_CLI(cu), &err);
+      ctx = rdf_inf_ctx (ctx_name);
+      dk_free_box (ctx_name);
+    }
+
+  if (!ctx)
+    return;
+
+  oval = quad[3];
+
+  if (DV_RDF != DV_TYPE_OF (oval) || !ric_iri_to_sub (ctx, quad[2], RI_SUBPROPERTY, 0))
+    return;
+
+  /* alloc all in MP */
+  if (!*ret_dc_array)
+    {
+      rl_o_dc = mp_data_col (clrg->clrg_pool, &ssl_any_dummy, dc_batch_sz);
+      rl_ro_id_dc = mp_data_col (clrg->clrg_pool, &ssl_int_dummy, dc_batch_sz);
+      rl_text_dc = mp_data_col (clrg->clrg_pool, &ssl_any_dummy, dc_batch_sz);
+      rl_lang_dc = mp_data_col (clrg->clrg_pool, &ssl_int_dummy, dc_batch_sz);
+      /* rdf labels */
+      (*ret_dc_array) = (caddr_t*)mp_alloc_box (clrg->clrg_pool, sizeof (caddr_t) * 4, DV_BIN);
+      (*ret_dc_array)[0] = (caddr_t) rl_o_dc;
+      (*ret_dc_array)[1] = (caddr_t) rl_ro_id_dc;
+      (*ret_dc_array)[2] = (caddr_t) rl_text_dc;
+      (*ret_dc_array)[3] = (caddr_t) rl_lang_dc;
+    }
+  else
+    {
+      rl_o_dc = (data_col_t *)(*ret_dc_array)[0];
+      rl_ro_id_dc = (data_col_t *)(*ret_dc_array)[1];
+      rl_text_dc = (data_col_t *)(*ret_dc_array)[2];
+      rl_lang_dc = (data_col_t *)(*ret_dc_array)[3];
+    }
+
+  if (DV_RDF == DV_TYPE_OF (oval) && ric_iri_to_sub (ctx, quad[2], RI_SUBPROPERTY, 0))
+    {
+      rdf_box_t * rb = (rdf_box_t *) oval;
+      if (!rb->rb_is_complete)
+        rb_complete (rb, qi->qi_trx, qi); /*GPF_T1 ("The rb_box is supposed to be complete in cu_rdf_ins_cb");*/
+      if (!DV_STRINGP (rb->rb_box)) /* labels are supposed to be strings */
+	return;
+      dc_append_box (rl_o_dc, oval);
+      dc_append_box (rl_ro_id_dc, mp_box_num (clrg->clrg_pool, rb->rb_ro_id));
+      dc_append_box (rl_text_dc, cu_rdf_ins_label_normalize (clrg->clrg_pool, rb->rb_box));
+      dc_append_box (rl_lang_dc, mp_box_num (clrg->clrg_pool, rb->rb_lang));
+    }
+}
+
+caddr_t
+aq_rl_lbl_func (caddr_t av, caddr_t * err_ret)
+{
+  caddr_t *params = (caddr_t *) av;
+  client_connection_t *cli = GET_IMMEDIATE_CLIENT_OR_NULL;
+  int is_pk = BOX_ELEMENTS_0 (params) == 4 ? 1 : 0;
+  if (0 == ((data_col_t **)params)[0]->dc_n_values)
+    *err_ret = NULL;
+  else
+    *err_ret = qr_exec (cli, is_pk ? rl_labels_qr : rl_labels_qr_inx, CALLER_LOCAL, "", NULL, NULL, params, NULL, 0);
+  return NULL;
+}
+
 int rl_query_inited;
 int32 enable_rdf_trig = 0;
 
@@ -316,6 +422,14 @@ rl_query_init (dbe_table_t * quad_tb)
     }
   END_DO_SET();
   rl_all_keys_qr = sql_compile ("insert soft DB.DBA.RDF_QUAD option (vectored) (G, S, P, O) values (?, ?, ?, ?)", bootstrap_cli, &err, SQLC_DEFAULT);
+  if (err)
+    sqlr_resignal (err);
+  rl_labels_qr = sql_compile ("insert soft DB.DBA.RDF_LABEL index RDF_LABEL option (vectored) (RL_O, RL_RO_ID, RL_TEXT, RL_LANG) values (?,?,?,?)", 
+      bootstrap_cli, &err, SQLC_DEFAULT);
+  if (err)
+    sqlr_resignal (err);
+  rl_labels_qr_inx = sql_compile ("insert soft DB.DBA.RDF_LABEL index RDF_LABEL_TEXT option (vectored) (RL_TEXT, RL_O) values (?,?)", 
+      bootstrap_cli, &err, SQLC_DEFAULT);
   if (err)
     sqlr_resignal (err);
   rl_graph_words_qr = sql_compile ("DB.DBA.RDF_OBJ_ADD_KEYWORD_FOR_GRAPH  (?, ?)", bootstrap_cli, &err, SQLC_DEFAULT);
@@ -340,7 +454,7 @@ cu_rl_graph_words (cucurbit_t * cu, caddr_t g_iid)
     return;
   hit = *dict_place;
   pars = (caddr_t *) list (2, box_copy_tree (g_iid), box_copy_tree ((caddr_t) hit));
-  save_repl = box_copy_tree (qi->qi_trx->lt_replicate);
+  save_repl = (caddr_t *) box_copy_tree ((caddr_t) (qi->qi_trx->lt_replicate));
   save_ac = qi->qi_client->cli_row_autocommit;
   err = qr_exec (cli, rl_graph_words_qr, CALLER_LOCAL, "", NULL, NULL, pars, NULL, 0);
   qi->qi_trx->lt_replicate = save_repl;
@@ -355,7 +469,6 @@ cu_rl_graph_words (cucurbit_t * cu, caddr_t g_iid)
   dk_free_box ((caddr_t) pars);
 }
 
-void rdf_repl_gs_batch (query_instance_t * qi, caddr_t * batch, int ins);
 
 void
 cu_rl_cols (cucurbit_t * cu, caddr_t g_iid)
@@ -364,7 +477,7 @@ cu_rl_cols (cucurbit_t * cu, caddr_t g_iid)
   caddr_t *inst = cu->cu_qst;
   async_queue_t *aq;
   dbe_table_t *quad_tb = sch_name_to_table (wi_inst.wi_schema, "DB.DBA.RDF_QUAD");
-  caddr_t err1 = NULL, err2 = NULL;
+  caddr_t err1 = NULL, err2 = NULL, *perr2 = &err2;
   int nth_key = 0, inx, n_keys;
   cl_req_group_t *clrg = cu->cu_clrg;
   data_col_t *g_dc = mp_data_col (clrg->clrg_pool, &ssl_iri_dummy, dc_batch_sz);
@@ -373,74 +486,67 @@ cu_rl_cols (cucurbit_t * cu, caddr_t g_iid)
   data_col_t *o_dc = mp_data_col (clrg->clrg_pool, &ssl_any_dummy, dc_batch_sz);
   int is_gs = BOX_ELEMENTS (cu->cu_input_funcs) == 5;
   int is_del = cu->cu_rdf_load_mode == RDF_LD_DEL_GS || cu->cu_rdf_load_mode == RDF_LD_DELETE, allg;
-  dk_set_t set = NULL;
-  caddr_t tmp[5], * quad;
+  caddr_t tmp[5], * quad, * lbl_box = NULL;
   BOX_AUTO_TYPED (caddr_t *, quad, tmp, 4 * sizeof (caddr_t), DV_ARRAY_OF_POINTER);
 
   rl_query_init (quad_tb);
   for (inx = 0; inx < cu->cu_fill; inx++)
     {
       caddr_t *row = (caddr_t *) cu->cu_rows[inx];
-      caddr_t gx;
+      caddr_t x = row[5];
+      QNCAST (rdf_box_t, rb, x);
+      int is_rb = DV_RDF == DV_TYPE_OF (x) && rb->rb_is_complete && rb->rb_ro_id;
       /* dpipe from replication cb  */
       if (cu->cu_rdf_load_mode == RDF_LD_INS_GS || cu->cu_rdf_load_mode == RDF_LD_DEL_GS)
 	{
-	  caddr_t x = row[5];
-	  dc_append_box (g_dc, row[2]); /* G */
-	  dc_append_box (s_dc, row[3]); /* S */
-	  dc_append_box (p_dc, row[4]); /* P */
+	  quad[0] = row[2];	/* G */
+	  quad[1] = row[3];	/* S */
+	  quad[2] = row[4];	/* P */
 	  if (DV_RDF == DV_TYPE_OF (x)) /* O */
-	    {
-	      QNCAST (rdf_box_t, rb, x);
-	      int is_rb = rb->rb_is_complete && rb->rb_ro_id;
-	      if (is_rb) rb->rb_is_complete = 0;
-	      dc_append_box (o_dc, x);
-	      if (is_rb) rb->rb_is_complete = 1;
-	    }
+	    quad[3] = x;
 	  else
-	    dc_append_box (o_dc, row[5]);
-	  continue;
+	    quad[3] = row[5];
 	}
-      /* ttlpv loader */
-      if (is_gs)
-	gx = row[6];
-      else
-	gx = g_iid;
-      dc_append_box (g_dc, gx);
-      dc_append_box (s_dc, row[2]);
-      dc_append_box (p_dc, row[3]);
-      quad[0] = gx;
-      quad[1] = row[2];
-      quad[2] = row[3];
-      if (DV_DB_NULL == DV_TYPE_OF (row[4]))
+      else			/* ttlpv loader dpipes */
+	{
+	  quad[0] = (is_gs ? row[6] : g_iid);	/* G */
+	  quad[1] = row[2];			/* S */
+	  quad[2] = row[3];			/* P */
+	  if (DV_DB_NULL == DV_TYPE_OF (row[4]))	/* inx 4  is for IRI, if null then we expect rdf_box, number or date at inx 5  */
 	{
-	  caddr_t x = row[5];
-	  QNCAST (rdf_box_t, rb, x);
-	  int is_rb = DV_RDF == DV_TYPE_OF (x) && rb->rb_is_complete && rb->rb_ro_id;
 	  dtp_t dtp = DV_TYPE_OF (x); 
-	  if (DV_DB_NULL == dtp || DV_STRING == dtp || IS_WIDE_STRING_DTP (dtp))
+              if (DV_DB_NULL == dtp || DV_STRING == dtp || IS_WIDE_STRING_DTP (dtp) || IS_NONLEAF_DTP (dtp))
 	    sqlr_new_error ("42000",  "CL...",  "%s not allowed for O column value S=" BOXINT_FMT 
-		" P=" BOXINT_FMT , (DV_DB_NULL == dtp ? "NULL" : "string"), unbox_iri_id (quad[1]), unbox_iri_id (quad[2]));
-	  if (is_rb)
-	    rb->rb_is_complete = 0;
-	  dc_append_box (o_dc, x);
-	  if (is_rb)
-	    rb->rb_is_complete = 1;
+                    " P=" BOXINT_FMT , dv_type_title (dtp), unbox_iri_id (quad[1]), unbox_iri_id (quad[2]));
 	  quad[3] = x;
+
+	      if (rdf_label_inf_name)	/* rdf labels fill if enabled */
+	    rl_rdf_ins_label (cu, quad, &lbl_box);
 	}
       else
-	{
-	dc_append_box (o_dc, row[4]);
 	  quad[3] = row[4];
+
+#define IS_DUMMY_QUAD(quad) (is_del && ( !unbox_iri_id (quad[0]) || !unbox_iri_id (quad[1]) || !unbox_iri_id (quad[2]) ))
+
 	}
-      if (rdf_graph_is_in_enabled_repl ((caddr_t *)qi, unbox_iri_id (quad[0]), &allg))
-	dk_set_push (&set, box_copy_tree (quad));
+
+      /* we do not try to delete zero IRI ID */
+      if (IS_DUMMY_QUAD (quad))
+	continue;
+
+      dc_append_box (g_dc, quad[0]);
+      dc_append_box (s_dc, quad[1]);
+      dc_append_box (p_dc, quad[2]);
+      if (is_rb)
+	rb->rb_is_complete = 0;	/* this is important, in insert o_dc we don't want complete boxes */
+      dc_append_box (o_dc, quad[3]);
+      if (is_rb)
+	rb->rb_is_complete = 1;
     }
   BOX_DONE (quad, tmp);
-  if (set)
-    rdf_repl_gs_batch (qi, (caddr_t *) list_to_array (dk_set_nreverse (set)), 1);
 
   cu->cu_cd = (caddr_t *) mp_alloc_box (clrg->clrg_pool, sizeof (caddr_t) * dk_set_length (quad_tb->tb_keys), DV_BIN);
+
   DO_SET (dbe_key_t *, key, &quad_tb->tb_keys)
   {
     int n_parts = dk_set_length (key->key_parts);
@@ -466,6 +572,7 @@ cu_rl_cols (cucurbit_t * cu, caddr_t g_iid)
     nth_key++;
   }
   END_DO_SET ();
+
   n_keys = dk_set_length (quad_tb->tb_keys);
   if (qi->qi_trx->lt_replicate != REPL_NO_LOG)
     n_keys = 0;
@@ -481,11 +588,20 @@ cu_rl_cols (cucurbit_t * cu, caddr_t g_iid)
   }
   END_DO_SET ();
   /*cu_rl_graph_words (cu, g_iid);*/
+  /* rdf labels */
+  if (NULL != lbl_box)
+    {
+      caddr_t * lbl_box_inx = (caddr_t*)mp_alloc_box (clrg->clrg_pool, sizeof (caddr_t) * 2, DV_BIN);
+      lbl_box_inx[0] = lbl_box[2];
+      lbl_box_inx[1] = lbl_box[0];
+      aq_request  (aq, aq_rl_lbl_func, (caddr_t)lbl_box);
+      aq_request  (aq, aq_rl_lbl_func, (caddr_t)lbl_box_inx);
+    }
   IO_SECT (inst);
   aq->aq_wait_qi = qi;
   aq_wait_all (aq, &err1);
   aq->aq_wait_qi = NULL;
-  END_IO_SECT (&err2);
+  END_IO_SECT (perr2);
   if (err1)
     {
       dk_free_tree (err2);
@@ -510,111 +626,146 @@ l_make_ro_disp (cucurbit_t * cu, caddr_t * args, value_state_t * vs)
   dtp_t dtp = DV_TYPE_OF (box);
   int len = 0, is_text = 0;
   static caddr_t l_null;
-  static cu_func_t *cf;
+  static cu_func_t *cf_0, *cf_ne;
+  cu_func_t *cf;
+  int is_del = cu->cu_rdf_load_mode == RDF_LD_DEL_GS || cu->cu_rdf_load_mode == RDF_LD_DELETE;
   AUTO_POOL (12);
-  if (!cf)
+  if (!cf_0)
     {
       l_null = dk_alloc_box (0, DV_DB_NULL);
-      cf = cu_func ("L_O_LOOK", 1);
+      cf_0 = cu_func ("L_O_LOOK", 1);
+      cf_ne = cu_func ("L_O_LOOK_NE", 1);
     }
-  if (IS_WIDE_STRING_DTP (dtp))
+  cf = is_del ? cf_ne : cf_0;
+  switch (dtp)
     {
-      mem_pool_t *pool = cu->cu_clrg->clrg_pool;
-      box = mp_box_wide_as_utf8_char (pool, box, box_length (box) / sizeof (wchar_t) - 1, DV_SHORT_STRING);
-      dtp = DV_TYPE_OF (box);
-    }
-  if (DV_RDF == dtp)
-    {
-      rdf_box_t *rb = (rdf_box_t *) box;
-      caddr_t content = rb->rb_box;
-      dtp_t cdtp = DV_TYPE_OF (content);
-      rdf_obj_ft_rule_iid_hkey_t iid_hkey = { 0, 0 };
-      if (rb->rb_ro_id)
-	{
-	  cu_set_value (cu, vs, box_copy_tree (box));
-	  return NULL;
-	}
-      dt_lang = rb->rb_type << 16 | rb->rb_lang;
-      if (DV_XML_ENTITY == cdtp && rb->rb_chksum_tail)
-	{
-	  QNCAST (rdf_bigbox_t, rbb, rb);
-	  if (!rb->rb_ro_id)
-	    cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, rbb->rbb_chksum, ap_box_num (&ap, dt_lang), content, (caddr_t)(ptrlong)rb->rb_is_text_index, ap_box_num (&ap, 0)));
-	  else
+    case DV_RDF:
+      {
+	rdf_box_t *rb = (rdf_box_t *) box;
+	caddr_t content = rb->rb_box;
+	dtp_t cdtp = DV_TYPE_OF (content);
+	rdf_obj_ft_rule_iid_hkey_t iid_hkey = { 0, 0 };
+	if (rb->rb_ro_id)
+	  {
 	    cu_set_value (cu, vs, box_copy_tree (box));
-	  return NULL;
-	}
-      if (DV_GEO != cdtp
-	  && (DV_STRING != cdtp
-	      || (!rdf_no_string_inline && (box_length (content) - 1 < RB_MAX_INLINED_CHARS && !rb->rb_is_text_index))))
+	    return NULL;
+	  }
+	dt_lang = rb->rb_type << 16 | rb->rb_lang;
+	if (DV_XML_ENTITY == cdtp && rb->rb_chksum_tail)
+	  {
+	    QNCAST (rdf_bigbox_t, rbb, rb);
+	    if (!rb->rb_ro_id)
+	      cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, rbb->rbb_chksum, ap_box_num (&ap, dt_lang), content, (caddr_t) (ptrlong) rb->rb_is_text_index, ap_box_num (&ap, 0)));
+	    else
+	      cu_set_value (cu, vs, box_copy_tree (box));
+	    return NULL;
+	  }
+	if (DV_GEO != cdtp && !IS_GENERIC_DURATION(content)
+	    && (DV_STRING != cdtp
+		|| (!rdf_no_string_inline && (box_length (content) - 1 < RB_MAX_INLINED_CHARS && !rb->rb_is_text_index))))
+	  {
+	    rb->rb_is_text_index = 0;	/* not a string and not xml */
+	    cu_set_value (cu, vs, box_copy_tree (box));
+	    return NULL;
+	  }
+	if ((DV_STRING == cdtp || DV_GEO == cdtp) && rb->rb_ro_id)
+	  {
+	    cu_set_value (cu, vs, box_copy_tree (box));
+	    return NULL;
+	  }
+	rb->rb_is_outlined = 1;
+	if (DV_GEO == cdtp)
+	  {
+	    caddr_t err = NULL;
+	    content = box_to_any_1 (content, &err, NULL, DKS_TO_DC);
+	    allocd_content = content;
+	    is_text = 2;
+	  }
+        else if (IS_GENERIC_DURATION(content))
+          {
+            caddr_t err = NULL;
+            content = box_to_any_1 (content, &err, NULL, DKS_TO_DC);
+            allocd_content = content;
+            /*dt_lang = RDF_BOX_INTERVAL << 16 | RDF_BOX_DEFAULT_LANG;*/
+          }
+	else
+	  is_text = rb->rb_is_text_index;
+	if (!is_text)		/* check if all graphs are enabled */
+	  {
+	    mutex_enter (rdf_obj_ft_rules_mtx);
+	    if (NULL != id_hash_get (rdf_obj_ft_rules_by_iids, (caddr_t) (&iid_hkey)))
+	      is_text = 1;
+	    mutex_leave (rdf_obj_ft_rules_mtx);
+	  }
+	len = box_length (content) - 1;
+	if (len > RB_BOX_HASH_MIN_LEN)
+	  {
+	    caddr_t trid = mdigest5 (content);
+	    cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, trid, ap_box_num (&ap, dt_lang), content, (caddr_t) (ptrlong) is_text, NULL));
+	    dk_free_box (trid);
+	    dk_free_box (allocd_content);
+	    return NULL;
+	  }
+	cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, content, ap_box_num (&ap, dt_lang), l_null, (caddr_t) (ptrlong) is_text, NULL));
+	dk_free_box (allocd_content);
+	return NULL;
+      }
+    case DV_GEO:
+      {
+	/* A trick instead of sqlr_new_error ("22023", "CLGEO", "A geometry without rdf box is not allowed as object of quad"); */
+	caddr_t err = NULL;
+	caddr_t content = box_to_any_1 (box, &err, NULL, DKS_TO_DC);
+	dt_lang = RDF_BOX_GEO << 16 | RDF_BOX_DEFAULT_LANG;
+	allocd_content = content;
+	len = box_length (content) - 1;
+	is_text = 2;
+	if (len > RB_BOX_HASH_MIN_LEN)
+	  {
+	    caddr_t trid = mdigest5 (content);
+	    cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, trid, ap_box_num (&ap, dt_lang), content, (caddr_t) (ptrlong) is_text, NULL));
+	    dk_free_box (trid);
+	  }
+	else
+	  cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, content, ap_box_num (&ap, dt_lang), l_null, (caddr_t) (ptrlong) is_text, NULL));
+	dk_free_box (allocd_content);
+	return NULL;
+      }
+    case DV_STRING:
+      len = box_length (box) - 1;
+      break;
+    case DV_WIDE:
+      allocd_content = box = box_wide_as_utf8_char (box, box_length (box) / sizeof (wchar_t) - 1, DV_SHORT_STRING);
+      len = box_length (box) - 1;
+      dtp = DV_STRING;
+      break;
+    case DV_BLOB_HANDLE:
+      {
+	caddr_t wide = blob_to_string (((query_instance_t *) (cu->cu_qst))->qi_trx, box);
+	QR_RESET_CTX
 	{
-	  rb->rb_is_text_index = 0;	/* not a string and not xml */
-	  cu_set_value (cu, vs, box_copy_tree (box));
-	  return NULL;
+	  allocd_content = box = box_wide_as_utf8_char (box, box_length (box) / sizeof (wchar_t) - 1, DV_SHORT_STRING);
 	}
-      if ((DV_STRING == cdtp || DV_GEO == cdtp) && rb->rb_ro_id)
+	QR_RESET_CODE
 	{
-	  cu_set_value (cu, vs, box_copy_tree (box));
-	  return NULL;
+	  dk_free_box (wide);
+	  POP_QR_RESET;
 	}
-      rb->rb_is_outlined = 1;
-      if (DV_GEO == cdtp)
-	{
-	  caddr_t err = NULL;
-	  content = box_to_any_1 (content, &err, NULL, DKS_TO_DC);
-	  allocd_content = content;
-	  is_text = 2;
-	}
-      else
-	is_text = rb->rb_is_text_index;
-      if (!is_text) /* check if all graphs are enabled */
-	{
-	  mutex_enter (rdf_obj_ft_rules_mtx);
-	  if (NULL != id_hash_get (rdf_obj_ft_rules_by_iids, (caddr_t)(&iid_hkey)))
-	    is_text = 1;
-	  mutex_leave (rdf_obj_ft_rules_mtx);
-	}
-      len = box_length (content) - 1;
-      if (len > RB_BOX_HASH_MIN_LEN)
-	{
-	  caddr_t trid = mdigest5 (content);
-	  cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, trid, ap_box_num (&ap, dt_lang), content, (caddr_t)(ptrlong)is_text, NULL));
-	  dk_free_box (trid);
-	  dk_free_box (allocd_content);
-	  return NULL;
-	}
-      cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, content, ap_box_num (&ap, dt_lang), l_null, (caddr_t)(ptrlong)is_text, NULL));
-      dk_free_box (allocd_content);
-      return NULL;
+	END_QR_RESET dk_free_box (wide);
+	dtp = DV_STRING;
+	len = box_length (box) - 1;
+	break;
+      }
     }
-  if (DV_GEO == dtp)
+
+  if ((DV_STRING != dtp && DV_UNAME != dtp) || (!rdf_no_string_inline && len < RB_MAX_INLINED_CHARS))
     {
-      /* A trick instead of sqlr_new_error ("22023", "CLGEO", "A geometry without rdf box is not allowed as object of quad"); */
-      caddr_t err = NULL;
-      caddr_t content = box_to_any_1 (box, &err, NULL, DKS_TO_DC);
-      dt_lang = RDF_BOX_GEO << 16 | RDF_BOX_DEFAULT_LANG;
-      allocd_content = content;
-      len = box_length (content) - 1;
-      is_text = 2;
-      if (len > RB_BOX_HASH_MIN_LEN)
-        {
-          caddr_t trid = mdigest5 (content);
-          cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, trid, ap_box_num (&ap, dt_lang), content, (caddr_t)(ptrlong)is_text, NULL));
-          dk_free_box (trid);
-        }
+      if (allocd_content == box)
+	cu_set_value (cu, vs, box);
       else
-        cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, content, ap_box_num (&ap, dt_lang), l_null, (caddr_t)(ptrlong)is_text, NULL));
-      dk_free_box (allocd_content);
+	cu_set_value (cu, vs, box_copy_tree (box));
       return NULL;
     }
-  else if (DV_STRING == dtp)
-    len = box_length (box) - 1;
-  if (DV_STRING != dtp || (!rdf_no_string_inline && len < RB_MAX_INLINED_CHARS))
-    {
-      cu_set_value (cu, vs, box_copy_tree (box));
-      return NULL;
-    }
-  if (BF_IRI == box_flags (box))
+  if (BF_IRI == box_flags (box) || DV_UNAME == dtp)
     return l_iri_id_disp (cu, box, vs);
 
   dt_lang = (RDF_BOX_DEFAULT_TYPE << 16) | RDF_BOX_DEFAULT_LANG;
@@ -623,9 +774,11 @@ l_make_ro_disp (cucurbit_t * cu, caddr_t * args, value_state_t * vs)
       caddr_t trid = mdigest5 (box);
       cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, trid, ap_box_num (&ap, dt_lang), vs->vs_org_value, (caddr_t) 1, NULL));
       dk_free_box (trid);
+      dk_free_box (allocd_content);
       return NULL;
     }
   cu_local_dispatch (cu, vs, cf, (caddr_t) ap_list (&ap, 5, box, ap_box_num (&ap, dt_lang), l_null, (caddr_t) 1, NULL));
+  dk_free_box (allocd_content);
   return NULL;
 }
 
@@ -635,14 +788,20 @@ l_iri_id_disp (cucurbit_t * cu, caddr_t name, value_state_t * vs)
 {
   static cu_func_t *cf;
   static cu_func_t *cf_np;
+  static cu_func_t *cf_ne;
+  static cu_func_t *cf_npe;
   boxint pref_id_no, iri_id_no;
   caddr_t prefix, local;
   dtp_t dtp = DV_TYPE_OF (name);
   caddr_t box_to_delete = NULL, name_to_delete = NULL;
+  int is_del = cu->cu_rdf_load_mode == RDF_LD_DEL_GS || cu->cu_rdf_load_mode == RDF_LD_DELETE;
+
   if (!cf)
     {
       cf = cu_func ("L_I_LOOK", 1);
       cf_np = cu_func ("L_I_LOOK_NP", 1);
+      cf_ne = cu_func ("L_I_LOOK_NE", 1);
+      cf_npe = cu_func ("L_I_LOOK_NPE", 1);
     }
 
   switch (dtp)
@@ -659,33 +818,19 @@ l_iri_id_disp (cucurbit_t * cu, caddr_t name, value_state_t * vs)
     case DV_UNAME:
       break;
     default:
-      sqlr_new_error ("42000", "CL...", "expected string for arg of disp fn");
+      sqlr_new_error ("42000", "CL...",
+	  "IRI ID lookup dispatching function expects IRI_ID or string as an argument, not a value with tag %d (%s)", dtp,
+	  dv_type_title (dtp));
     }
   if (!strncmp (name, "nodeID://", 9))
     {
-      unsigned char *tail = (unsigned char *) (name + 9);
-      int64 acc = 0;
-      int b_first = 0;
-      if ('b' == tail[0])
-	{
-	  b_first = 1;
-	  tail++;
-	}
-      while (isdigit (tail[0]))
-	acc = acc * 10 + ((tail++)[0] - '0');
-      if ('\0' != tail[0])
+      const char *error_fmt = NULL;
+      int64 acc = iri_nodeid_to_iid ((unsigned char *) (name + 9), &error_fmt);
+      if (NULL != error_fmt)
 	{
 	  if (NULL != box_to_delete)
 	    dk_free_box (box_to_delete);
-	  sqlr_new_error ("42000", "CL...", "Invalid tail of nodeID:// IRI");
-	}
-      if (b_first)
-	acc += MIN_64BIT_BNODE_IRI_ID;
-      if ((acc > (2 * min_bnode_iri_id ())) || (acc < min_bnode_iri_id ()))
-	{
-	  if (NULL != box_to_delete)
-	    dk_free_box (box_to_delete);
-	  sqlr_new_error ("42000", "CL...", "Invalid numeric value in the tail of nodeID:// IRI");
+	  sqlr_new_error ("42000", "CL...", error_fmt, name);
 	}
       cu_set_value (cu, vs, box_iri_id (acc));
       return NULL;
@@ -713,13 +858,20 @@ l_iri_id_disp (cucurbit_t * cu, caddr_t name, value_state_t * vs)
   if (!pref_id_no)
     {
       AUTO_POOL (8);
-      cu_local_dispatch (cu, vs, cf_np, (caddr_t) ap_list (&ap, 3, prefix, local, ap_box_iri_id (&ap, 0)));
+      cu_local_dispatch (cu, vs, (is_del ? cf_npe : cf_np), (caddr_t) ap_list (&ap, 3, prefix, local, ap_box_iri_id (&ap, 0)));
       dk_free_box (local);
       dk_free_box (prefix);
       return NULL;
     }
   dk_free_box (prefix);
-  LONG_SET_NA (local, pref_id_no);
+  RPID_SET_NA (local, pref_id_no);
+  if (is_del)
+    {
+      AUTO_POOL (8);
+      cu_local_dispatch (cu, vs, cf_ne, (caddr_t) ap_list (&ap, 2, local, ap_box_iri_id (&ap, 0)));
+      dk_free_box (local);
+      return NULL;
+    }
   iri_id_no = 0;		/*nic_name_id (iri_name_cache, local); */
   if (iri_id_no)
     {
@@ -760,20 +912,16 @@ bif_rl_set_pref_id (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 	{
 	  long l, hl;
 	  db_buf_length ((db_buf_t) str, &hl, &l);
-	  if (l < 4)
-	    sqlr_new_error ("42000", ".....", "Must have a string of at least 4 chars to set prefix");
-	  str[hl] = id >> 24;
-	  str[hl + 1] = id >> 16;
-	  str[hl + 2] = id >> 8;
-	  str[hl + 3] = id;
+	  if (l < RPID_SZ)
+	    sqlr_new_error ("42000", ".....", "Must have a string of at least %d chars to set prefix", RPID_SZ);
+	  RPID_SET_NA (((unsigned char *) (&str[hl])), id);
 	}
       else
-	sqlr_new_error ("42000", ".....", "__rl_set_pref_id expects a any type column");
+	sqlr_new_error ("42000", ".....", "__rl_set_pref_id() expects a column of type \"any\"");
     }
   return NULL;
 }
 
-
 caddr_t
 bif_rl_dp_ids (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
@@ -783,10 +931,14 @@ bif_rl_dp_ids (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   void * save;
   if (!cu)
     sqlr_new_error ("42000", "CL...", "Not a dpipe daq");
+  if (!rdf_rpid64_mode)
+    sqlr_new_error ("42000", "CL...", "Can not use dpipe IRI operations before upgrading the RDF_IRI table to 64-bit prefix IDs");
   cu->cu_qst = qst;
   save = cu->cu_ready_cb;
   cu->cu_ready_cb = NULL; /* local exec, CBs are for clustered operation */
   cu_rl_local_exec (cu);
+  if (cu->cu_fill > dc_max_batch_sz)
+    sqlr_new_error ("42000", "RLD01",  "Too many row in vectored batch (%d), exceeds max vector length %d", cu->cu_fill, dc_max_batch_sz);
   cu_rl_cols (cu, g_iid);
   cu->cu_ready_cb = save;
   return NULL;
@@ -808,16 +960,36 @@ bif_rld_init ()
   bif_define ("rl_dp_ids", bif_rl_dp_ids);
   bif_define ("__rl_set_pref_id", bif_rl_set_pref_id);
   bif_set_vectored (bif_rl_set_pref_id, (bif_vec_t) bif_rl_set_pref_id);
+
+  /* local iri_to_id dpipe */
   dpipe_define ("L_IRI_TO_ID", NULL, "", (cu_op_func_t) l_iri_id_disp, CF_1_ARG);
+
+  /* called from l_iri_id_disp in case when IRI prefix is known cache iri */
   dpipe_define ("L_I_LOOK", NULL, "DB.DBA.RL_I2ID", (cu_op_func_t) NULL, 0);
   dpipe_signature ("L_I_LOOK", 2, DV_STRING, DV_IRI_ID_8);
+
+  /* called from l_iri_id_disp in case when IRI prefix is NOT known cache prefix */
   dpipe_define ("L_I_LOOK_NP", NULL, "DB.DBA.RL_I2ID_NP", (cu_op_func_t) NULL, 0);
   dpipe_signature ("L_I_LOOK_NP", 3, DV_STRING, DV_STRING, DV_IRI_ID_8);
 
+  /* called from l_iri_id_disp in case when we check if IRI to delete exists */
+  dpipe_define ("L_I_LOOK_NE", NULL, "DB.DBA.RL_I2ID_NE", (cu_op_func_t) NULL, 0);
+  dpipe_signature ("L_I_LOOK_NE", 2, DV_STRING, DV_IRI_ID_8);
 
+  /* called from l_iri_id_disp in case when IRI prefix is NOT known cache prefix, to delete if IRI exists */
+  dpipe_define ("L_I_LOOK_NPE", NULL, "DB.DBA.RL_I2ID_NPE", (cu_op_func_t) NULL, 0);
+  dpipe_signature ("L_I_LOOK_NPE", 3, DV_STRING, DV_STRING, DV_IRI_ID_8);
+
+  /* local rdf obj from sql value dpipe */
   dpipe_define ("L_MAKE_RO", NULL, "", (cu_op_func_t) l_make_ro_disp, CF_1_ARG);
+
+  /* called from l_make_ro_disp, fetch ro_id if not make new rdf_obj */
   dpipe_define ("L_O_LOOK", NULL, "DB.DBA.L_O_LOOK", (cu_op_func_t) NULL, 0);
   dpipe_signature ("L_O_LOOK", 5, DV_STRING, DV_LONG_INT, DV_STRING, DV_LONG_INT, DV_LONG_INT);
+
+  dpipe_define ("L_O_LOOK_NE", NULL, "DB.DBA.L_O_LOOK_NE", (cu_op_func_t) NULL, 0);
+  dpipe_signature ("L_O_LOOK_NE", 5, DV_STRING, DV_LONG_INT, DV_STRING, DV_LONG_INT, DV_LONG_INT);
+
   ssl_set_no_dummy.ssl_dc_dtp = ssl_set_no_dummy.ssl_sqt.sqt_dtp = DV_LONG_INT;
   ssl_iri_dummy.ssl_dc_dtp = ssl_iri_dummy.ssl_sqt.sqt_dtp = DV_IRI_ID;
   ssl_any_dummy.ssl_dc_dtp = ssl_any_dummy.ssl_sqt.sqt_dtp = DV_ANY;
