@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -463,7 +463,7 @@ spar_qm_collist_crc (SPART **cols, const char *prefix, int ignore_order)
         total_crc += agg * (5 + ctr);
     }
   END_DO_BOX_FAST;
-  return t_box_sprintf (50, "%.20s%Lx", prefix, (long long)total_crc);
+  return t_box_sprintf (50, "%.20s%llx", prefix, (long long)total_crc);
 }
 
 void
