@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2013 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -60,6 +60,11 @@ sql_compile_st (ST ** ptree, client_connection_t * cli,
   query_t *volatile qr;
   client_connection_t *old_cli = sqlc_client ();
   DK_ALLOC_QUERY (qr);
+#ifdef QUERY_DEBUG
+  qr->qr_text_is_constant = 1;
+  qr->qr_text = "(no source, only a parsed tree)";
+  log_query_event (qr, 1, "ALLOC by sql_compile_st");
+#endif
   memset (&sc, 0, sizeof (sc));
 
   CC_INIT (cc, cli);
@@ -86,6 +91,8 @@ sql_compile_st (ST ** ptree, client_connection_t * cli,
     SET_THR_ATTR (THREAD_CURRENT_THREAD, TA_SQLC_ERROR, NULL);
     sql_stmt_comp (&sc, ptree);
     qr_set_local_code_and_funref_flag (sc.sc_cc->cc_query);
+    if (sc.sc_cc->cc_query->qr_proc_vectored || sc.sc_cc->cc_has_vec_subq)
+      sqlg_vector (&sc, sc.sc_cc->cc_query);
     qr_resolve_aliases (qr);
     qr_set_freeable (&cc, qr);
     qr->qr_instance_length = cc.cc_instance_fill * sizeof (caddr_t);
@@ -208,7 +215,7 @@ qc_make_cols (sql_comp_t * sc, query_cursor_t * qc, ST * tree)
 	DO_SET (dbe_column_t *, col, &order_key->key_parts)
 	{
 	  ST *ref = sqlc_ct_col_ref (ct, col->col_name);
-	  ST *spec = (ST *) t_list (3, ORDER_BY, ref, (ptrlong) ORDER_ASC);
+	  ST *spec = (ST *) t_list (4, ORDER_BY, ref, (ptrlong) ORDER_ASC, NULL);
 	  t_NCONCF1 (new_sel, ref);
 	  NCONCF1 (new_order_by, spec);
 	  t_NCONCF1 (order_pos, (ptrlong) col_pos);
@@ -226,7 +233,7 @@ qc_make_cols (sql_comp_t * sc, query_cursor_t * qc, ST * tree)
 	{
 	  ST *ref = crr->crr_col_ref;
 	  ptrlong ord = ct->ct_order;
-	  ST *spec = (ST *) t_list (3, ORDER_BY, ref, ord);
+	  ST *spec = (ST *) t_list (4, ORDER_BY, ref, ord, NULL);
 	  t_NCONCF1 (new_sel, ref);
 	  NCONCF1 (new_order_by, spec);
 	  nth++;
@@ -596,7 +603,7 @@ sqlc_top_select_wrap_dt (sql_comp_t * sc, ST * tree)
   top = SEL_TOP (tree);
   if (top)
     {
-      ST * out_names = (ST *) sqlc_selection_names (tree, 0);
+      ST * out_names = (ST *) sqlc_selection_names (tree);
       ST ** oby = tree->_.select_stmt.table_exp->_.table_exp.order_by;
       if (oby)
 	{
@@ -625,7 +632,7 @@ sqlc_cursor (sql_comp_t * sc, ST ** ptree, int cr_type)
   if (IS_UNION_ST (tree))
     {
       tree = sqlp_view_def (NULL, tree, 1);
-      tree = sqlc_union_dt_wrap (tree);
+      tree = sqlc_union_dt_wrap (sc, tree);
       *ptree = tree;
     }
   sqlc_top_select_wrap_dt (sc, tree);
