@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -741,8 +741,10 @@ scalar_exp_generate (sql_comp_t * sc, ST * tree, dk_set_t * code)
   if (sc->sc_so)
     {
       dfe = sqlo_df (sc->sc_so, tree);
+#if 0
       if (st_is_call (tree, "__ro2lo", 1))
-  bing ();
+        bing ();
+#endif
 
       if (dfe->dfe_ssl)
 	return (dfe->dfe_ssl);
@@ -1001,7 +1003,7 @@ cv_artm_set_type (instruction_t * ins)
 	  switch (ins->ins_type)
             {
             case IN_ARTM_PLUS:
-              if ((DV_DATETIME == ins->_.artm.left->ssl_dtp) || (DV_DATETIME == ins->_.artm.right->ssl_dtp))
+              if (IS_DATE_DTP(ins->_.artm.left->ssl_dtp) || IS_DATE_DTP (ins->_.artm.right->ssl_dtp))
                 {
                   ins->_.artm.result->ssl_dtp = DV_DATETIME;
                   ins->_.artm.result->ssl_sqt.sqt_non_null = 0;
@@ -1009,10 +1011,10 @@ cv_artm_set_type (instruction_t * ins)
                 }
               break;
             case IN_ARTM_MINUS:
-              if (DV_DATETIME == ins->_.artm.left->ssl_dtp)
+              if (IS_DATE_DTP (ins->_.artm.left->ssl_dtp))
                 {
                   ins->_.artm.result->ssl_sqt.sqt_non_null = 0;
-                  if (DV_DATETIME == ins->_.artm.right->ssl_dtp)
+                  if (IS_DATE_DTP (ins->_.artm.right->ssl_dtp))
                     {
                       ins->_.artm.result->ssl_dtp = DV_NUMERIC;
                       goto result_dtp_is_set;
@@ -1034,6 +1036,9 @@ cv_artm_set_type (instruction_t * ins)
 result_dtp_is_set:
 	  if (DV_NUMERIC == ins->_.artm.result->ssl_dtp)
 	    {
+              if (ins->_.artm.left->ssl_is_callret || ins->_.artm.left->ssl_vary ||
+                  ins->_.artm.right->ssl_is_callret || ins->_.artm.right->ssl_vary)
+                ins->_.artm.result->ssl_vary = 1; /* numeric ssl dc is boxes, can have box of int, thus cast on ins is needed */
 	      ins->_.artm.result->ssl_sqt.sqt_precision = NUMERIC_MAX_PRECISION;
 	      ins->_.artm.result->ssl_sqt.sqt_scale = NUMERIC_MAX_SCALE;
 	    }
@@ -1357,8 +1362,11 @@ cv_subq_ret (sql_comp_t * sc, instruction_t * ins)
   qr->qr_select_node->sel_out_slots[0]->ssl_sqt.sqt_non_null = 0;
   if (qr->qr_proc_vectored)
     {
-      ins->_.subq.scalar_ret = sqlc_new_temp (sc, "scalar", sel->sel_out_slots[0]->ssl_sqt.sqt_dtp);
-      ins->_.subq.scalar_ret->ssl_sqt = sel->sel_out_slots[0]->ssl_sqt;
+      state_slot_t * out = sel->sel_out_slots[0];
+      ins->_.subq.scalar_ret = sqlc_new_temp (sc, "scalar", out->ssl_sqt.sqt_dtp);
+      ins->_.subq.scalar_ret->ssl_sqt = out->ssl_sqt;
+      ins->_.subq.scalar_ret->ssl_vary = SSL_REF != out->ssl_type ?
+          out->ssl_is_callret : ((state_slot_ref_t *)out)->sslr_ssl->ssl_is_callret;
 
       if (sqlg_is_vector)
 	sel->sel_scalar_ret = ins->_.subq.scalar_ret;
@@ -2879,7 +2887,7 @@ select_ref_generate (sql_comp_t * sc, ST * tree, dk_set_t * code,
 	    jmp_label_t setenv_end = sqlc_new_label (sc);
 
 	    state_slot_t *flag = ssl_new_inst_variable (sc->sc_cc, "user_aggr_notfirst", DV_SHORT_INT);
-	    state_slot_t *env = ssl_new_inst_variable (sc->sc_cc, "user_aggr_env", DV_UNKNOWN);
+	    state_slot_t *env = ssl_new_inst_variable (sc->sc_cc, "user_aggr_env", DV_ARRAY_OF_POINTER);
 	    state_slot_t *ret = ssl_new_inst_variable (sc->sc_cc, "user_aggr_ret", DV_UNKNOWN);
 	    caddr_t deflt_env;
 	    state_slot_t ** acc_args = (state_slot_t **) dk_alloc_box (sizeof (state_slot_t *) * (1 + BOX_ELEMENTS(tree->_.fn_ref.fn_arglist)), DV_ARRAY_OF_POINTER);
