@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -47,9 +47,9 @@ dfe_left_col (df_elt_t * tb_dfe, df_elt_t * pred)
   in_list = sqlo_in_list (pred, tb_dfe, NULL);
   if (in_list && dfe_tables (pred) && !pred->dfe_tables->next)
     return in_list[0];
-  if (DFE_COLUMN == pred->_.bin.left->dfe_type && tb_dfe->_.table.ot == (op_table_t *) pred->_.bin.left->dfe_tables->data)
+  if (!DFE_SHORTCUT(pred->_.bin.left) && DFE_COLUMN == pred->_.bin.left->dfe_type && (op_table_t *)pred->_.bin.left->dfe_tables && tb_dfe->_.table.ot == (op_table_t *)pred->_.bin.left->dfe_tables->data)
     return pred->_.bin.left;
-  if (DFE_COLUMN == pred->_.bin.right->dfe_type && tb_dfe->_.table.ot == (op_table_t *) pred->_.bin.right->dfe_tables->data)
+  if (!DFE_SHORTCUT(pred->_.bin.right) && DFE_COLUMN == pred->_.bin.right->dfe_type && (op_table_t *)pred->_.bin.right->dfe_tables && tb_dfe->_.table.ot == (op_table_t *)pred->_.bin.right->dfe_tables->data)
     return pred->_.bin.right;
   return NULL;
 }
@@ -67,9 +67,9 @@ dfe_right (df_elt_t * tb_dfe, df_elt_t * pred)
 	return in_list[1];
       return NULL;
     }
-  if (!dk_set_member (pred->_.bin.left->dfe_tables, (void *) tb_dfe->_.table.ot))
+  if (!DFE_SHORTCUT(pred->_.bin.left) && !dk_set_member (pred->_.bin.left->dfe_tables, (void *) tb_dfe->_.table.ot))
     return pred->_.bin.left;
-  if (!dk_set_member (pred->_.bin.right->dfe_tables, (void *) tb_dfe->_.table.ot))
+  if (!DFE_SHORTCUT(pred->_.bin.right) && !dk_set_member (pred->_.bin.right->dfe_tables, (void *) tb_dfe->_.table.ot))
     return pred->_.bin.right;
   return NULL;
 }
@@ -582,8 +582,10 @@ dfe_jp_fill (sqlo_t * so, op_table_t * ot, df_elt_t * tb_dfe, join_plan_t * jp,
   }
   END_DO_SET ();
   jp->jp_fanout = jp_fanout (jp);
+#if 0
   if (jp->jp_fanout < 0 && jp->jp_fanout != -1)
     bing ();
+#endif
 }
 
 
