@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -130,7 +130,7 @@ sqlo_ot_effective_prefix (op_table_t * ot)
 {
   if (ot->ot_prefix)
     return (ot->ot_prefix);
-  return (ot->ot_table->tb_name);
+  return (ot->ot_table ? ot->ot_table->tb_name : NULL);
 }
 
 
@@ -510,12 +510,10 @@ sqlo_col_scope_1 (sqlo_t * so, ST * col_ref, int generate)
     return 1;
   if (!generate)
     return 0;
-    {
-      char cn[MAX_NAME_LEN * 5 + 10];
-      if (col_ref->_.col_ref.prefix)
-	snprintf (cn, sizeof (cn), "%s.%s", col_ref->_.col_ref.prefix, col_ref->_.col_ref.name);
-      sqlc_error (so->so_sc->sc_cc, "S0022", "No column %s.", col_ref->_.col_ref.prefix ? cn : col_ref->_.col_ref.name);
-    }
+  if (col_ref->_.col_ref.prefix)
+    sqlc_error (so->so_sc->sc_cc, "S0022", "No column %s.%s.", col_ref->_.col_ref.prefix, col_ref->_.col_ref.name);
+  else
+    sqlc_error (so->so_sc->sc_cc, "S0022", "No column %s.", col_ref->_.col_ref.name);
   return 0; /* dummy */
 }
 
@@ -813,7 +811,7 @@ done:
 	      snprintf (temp, sizeof (temp), "row level security: %.900s", ((char **) err)[2]);
 	      strncpy (state, ((char **) err)[1], sizeof (state));
 	      dk_free_tree (err);
-	      sqlc_new_error (so->so_sc->sc_cc, state, "SQ191", temp);
+	      sqlc_new_error (so->so_sc->sc_cc, state, "SQ191", "%s", temp);
 	    }
 	  else
 	    sqlc_resignal_1 (so->so_sc->sc_cc, err);
@@ -916,7 +914,7 @@ sqlo_add_table_ref (sqlo_t * so, ST ** tree_ret, dk_set_t *res)
 		ST_P (view, INTERSECT_ALL_ST))
 	      {
 		view = sqlp_view_def (NULL, view, 1);
-		view = sqlc_union_dt_wrap (view);
+		view = sqlc_union_dt_wrap (so->so_sc, view);
 	      }
 	    sqlo_scope (so, &view);
 	    if (ST_P (view, SELECT_STMT))
@@ -1003,7 +1001,7 @@ if (J_INNER == tree->_.join.type && right_ot->ot_is_outer)
 	sco_merge (old_sco, sco);
 	so->so_scope = old_sco;
 	if (tree->_.join.type == OJ_LEFT || tree->_.join.type == OJ_FULL)
-	  right_ot->ot_is_outer = 1;
+	  left_ot->ot_is_left = right_ot->ot_is_outer = 1;
 	else if (!sco->sco_has_jt && tree->_.join.type == J_INNER &&
 	    !ST_P (left_ot->ot_dt, PROC_TABLE) &&
 	    !ST_P (right_ot->ot_dt, PROC_TABLE))
@@ -1625,9 +1623,11 @@ sqlo_check_ft_offband (sqlo_t * so, op_table_t * ot, ST ** args, char type)
 	  continue;
 	}
       if (inx >= surely_option_idx)
-	sqlc_error (sc->sc_cc, "37000",
-          "Argument %d of %s is '%.300s', not a keyword from list OFFBAND, DESCENDING, RANGES, MAIN_RANGES, ATTR_RANGES, SCORE, SCORE_LIMIT, EXT_FTI, GEO, GEO_RDF, PRECISION",
-	  inx + 1, sqlo_spec_predicate_name(type), arg );
+	{
+	  sqlc_error (sc->sc_cc, "37000",
+	      "Argument %d of %s is '%.300s', not a keyword from list OFFBAND, DESCENDING, RANGES, MAIN_RANGES, ATTR_RANGES, SCORE, SCORE_LIMIT, EXT_FTI, GEO, GEO_RDF, PRECISION",
+	      inx + 1, sqlo_spec_predicate_name (type), arg);
+	}
     }
   if (off)
     ot->ot_text_offband = (op_virt_col_t **) list_to_array (off);
@@ -1705,7 +1705,8 @@ sqlo_implied_columns_of_contains (sqlo_t *so, ST *tree, int add_score)
       if (BOX_ELEMENTS(args) < 1 || !ST_COLUMN (args[0], COL_DOTTED))
 	sqlc_error (so->so_sc->sc_cc, "37000",
 	    "The first argument of %s must be a column", sqlo_spec_predicate_name (ctype));
-
+      if (args[0]->_.col_ref.name == STAR)
+        sqlc_new_error (so->so_sc->sc_cc, "42000", "SQ064", "Illegal use of '*'.");
       ot = sco_is_defd (so->so_scope, args[0],
 	  args[0]->_.col_ref.prefix ? SCO_THIS_QUAL : SCO_UNQUALIFIED, 1);
       if (!ot || !ot->ot_table)
@@ -2024,7 +2025,7 @@ sqlo_expand_jts (sqlo_t *so, ST **ptree, ST *select_stmt, int was_top)
 
 	  *ptree = t_listst (5, UNION_ST, left_oj_tree->_.table_ref.table,
 	      right_oj_tree->_.table_ref.table, NULL, 0);
-	  *ptree = sqlc_union_dt_wrap (*ptree);
+	  *ptree = sqlc_union_dt_wrap (so->so_sc, *ptree);
 	  res ++;
 	}
     }
@@ -2445,14 +2446,14 @@ sqlo_check_group_by_cols (sqlo_t *so, ST *tree, ST *** group, op_table_t *dt_ot,
 
 
 static void
-sqlo_oby_remove_scalar_exps (sqlo_t *so, ST *** oby)
+sqlo_oby_remove_scalar_exps (sqlo_t *so, ST *** oby, int gb_ua)
 { /* remove all scalar order by's (as they do not contribute nothing) */
   dk_set_t set = NULL;
   int have_const_obys = 0, inx;
 
   DO_BOX (ST *, spec, inx, (*oby))
     {
-      if (DV_ARRAY_OF_POINTER == DV_TYPE_OF (spec->_.o_spec.col))
+      if (DV_ARRAY_OF_POINTER == DV_TYPE_OF (spec->_.o_spec.col) || (gb_ua && DV_TYPE_OF (spec->_.o_spec.col) != DV_DB_NULL))
 	{
 	  t_set_push (&set, spec);
 	}
@@ -2728,6 +2729,7 @@ sqlo_select_scope (sqlo_t * so, ST ** ptree)
 
   if (texp)
     {
+      int is_not_one_gb;
       ot->ot_opts = ST_OPT (texp, caddr_t *, _.table_exp.opts);
       ot->ot_fixed_order = (int)(ptrlong) sqlo_opt_value (ot->ot_opts, OPT_ORDER);
       _DO_BOX (inx, texp->_.table_exp.from)
@@ -2786,15 +2788,32 @@ sqlo_select_scope (sqlo_t * so, ST ** ptree)
       else
       sqlo_scope_array (so, texp->_.table_exp.group_by);
 
+      is_not_one_gb = texp->_.table_exp.group_by_full && BOX_ELEMENTS(texp->_.table_exp.group_by_full) > 1;
       if (texp->_.table_exp.order_by)
-	sqlo_oby_exp_cols (so, tree, texp->_.table_exp.order_by);
+        {
+          sqlo_oby_exp_cols (so, tree, texp->_.table_exp.order_by);
+          sqlo_oby_remove_scalar_exps (so, &texp->_.table_exp.order_by, 0);
+        }
       if (texp->_.table_exp.group_by)
 	{
+          int has_ua = 0;
 	  sqlo_oby_exp_cols (so, tree, texp->_.table_exp.group_by);
+          if (!is_not_one_gb) /* for cube/rollup should be done later */
+            {
+              DO_SET (ST *, fref, &so->so_this_dt->ot_fun_refs)
+                {
+                  if (AMMSC_USER == fref->_.fn_ref.fn_code)
+                    {
+                      has_ua = 1;
+                      break;
+                    }
+                }
+              END_DO_SET();
+              sqlo_oby_remove_scalar_exps (so, &texp->_.table_exp.group_by, has_ua);
+            }
 	}
       if (so->so_this_dt->ot_fun_refs || texp->_.table_exp.group_by)
 	{
-	  int is_not_one_gb = texp->_.table_exp.group_by_full && BOX_ELEMENTS(texp->_.table_exp.group_by_full) > 1;
 	  sqlo_check_group_by_cols (so, (ST *) tree->_.select_stmt.selection, &(texp->_.table_exp.group_by), ot, is_not_one_gb);
 	  sqlo_replace_as_exps ((ST **) &(texp->_.table_exp.group_by), so->so_scope);
 	  if (texp->_.table_exp.group_by)
@@ -2807,7 +2826,7 @@ sqlo_select_scope (sqlo_t * so, ST ** ptree)
 	  sqlo_check_group_by_cols (so, (ST *) texp->_.table_exp.order_by,
 	      &(texp->_.table_exp.group_by), ot, is_not_one_gb);
 	}
-      sqlo_oby_remove_scalar_exps (so, &texp->_.table_exp.order_by);
+      sqlo_oby_remove_scalar_exps (so, &texp->_.table_exp.order_by, 0);
     }
   else
     sqlo_scope_array (so, (ST**) tree->_.select_stmt.selection);
@@ -3317,9 +3336,9 @@ sqlo_scope (sqlo_t * so, ST ** ptree)
 	{
 	  ST *left;
 	  if (IS_UNION_ST (tree->_.set_exp.left))
-	    tree->_.set_exp.left = sqlc_union_dt_wrap (tree->_.set_exp.left);
+	    tree->_.set_exp.left = sqlc_union_dt_wrap (so->so_sc, tree->_.set_exp.left);
 	  if (IS_UNION_ST (tree->_.set_exp.right))
-	    tree->_.set_exp.right = sqlc_union_dt_wrap (tree->_.set_exp.right);
+	    tree->_.set_exp.right = sqlc_union_dt_wrap (so->so_sc, tree->_.set_exp.right);
 	  left = sqlp_union_tree_select (tree);
 	  sqlo_union_scope (so, ptree, left);
 	  break;
