@@ -1,10 +1,8 @@
 --
---  $Id$
---
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2018 OpenLink Software
+--  Copyright (C) 1998-2024 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -19,7 +17,6 @@
 --  with this program; if not, write to the Free Software Foundation, Inc.,
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 --
---
 
 create table DB.DBA.URL_REWRITE_RULE_LIST (
   URRL_LIST     varchar not null,
@@ -44,23 +41,6 @@ create table DB.DBA.URL_REWRITE_RULE (
   primary key (URR_RULE) )
 ;
 
---#IF VER=5
---!AFTER
-alter table DB.DBA.URL_REWRITE_RULE add URR_ACCEPT_PATTERN varchar
-;
-
---!AFTER
-alter table DB.DBA.URL_REWRITE_RULE add URR_NO_CONTINUATION int
-;
-
---!AFTER
-alter table DB.DBA.URL_REWRITE_RULE add URR_HTTP_REDIRECT int
-;
-
---!AFTER
-alter table DB.DBA.URL_REWRITE_RULE add URR_HTTP_HEADERS varchar
-;
---#ENDIF
 
 create table DB.DBA.HTTP_VARIANT_MAP (
     VM_ID		integer identity,
@@ -78,11 +58,6 @@ create table DB.DBA.HTTP_VARIANT_MAP (
 create unique index HTTP_VARIANT_MAP_ID on DB.DBA.HTTP_VARIANT_MAP (VM_ID)
 ;
 
---#IF VER=5
---!AFTER
-alter table DB.DBA.HTTP_VARIANT_MAP add VM_CONTENT_LOCATION_HOOK  varchar
-;
---#ENDIF
 
 create procedure DB.DBA.URLREWRITE_CREATE_RULE (
   in rule_type int,
@@ -931,9 +906,13 @@ create procedure DB.DBA.URLREWRITE_CALC_QS (in accept varchar, in s_accept varch
 	    q := 1.0;
 	  else
 	    {
-	      tmp := split_and_decode (q, 0, '\0\0=');
-	      if (length (tmp) = 2)
-		q := atof (tmp[1]);
+              declare qtok varchar;
+             tmp := split_and_decode (trim(q), 0, '\0\0=');
+              qtok := null;
+              if (isvector (tmp) and 0 = mod (length(tmp), 2))
+                qtok := get_keyword ('q', tmp);
+	      if (qtok is not null)
+		q := atof (qtok);
 	      else
 		q := 1.0;
 	    }
@@ -1436,16 +1415,10 @@ create procedure DB.DBA.URLREWRITE_DUMP_RULELIST_SQL (in rulelist_iri varchar)
 }
 ;
 
---#IF VER=5
---!AFTER
---#ENDIF
-virt_proxy_init ()
+DB.DBA.virt_proxy_init ()
 ;
 
---#IF VER=5
---!AFTER
---#ENDIF
-grant execute on ext_http_proxy to PROXY
+grant execute on DB.DBA.ext_http_proxy to PROXY
 ;
 
 -- /* Example for 'denote' and 'entity' IRI patterns */
@@ -1454,8 +1427,7 @@ grant execute on ext_http_proxy to PROXY
 
 
 -- supported mime types
-create procedure
-url_rewrite_mime_types ()
+create procedure DB.DBA.url_rewrite_mime_types ()
 {
   return vector (
       vector ('html',  	'text/html', 		1.0),
@@ -1475,11 +1447,10 @@ url_rewrite_mime_types ()
 }
 ;
 
-create procedure
-url_rewrite_mime_pattern ()
+create procedure DB.DBA.url_rewrite_mime_pattern ()
 {
   declare x, res any;
-  x := url_rewrite_mime_types ();
+  x := DB.DBA.url_rewrite_mime_types ();
   res := '';
   foreach (varchar p in x) do
     {
@@ -1489,8 +1460,9 @@ url_rewrite_mime_pattern ()
 }
 ;
 
-create procedure
-url_rewrite_gen_describe (in graph varchar, in iri_spf varchar)
+create procedure DB.DBA.url_rewrite_gen_describe (
+  in graph varchar,
+  in iri_spf varchar)
 {
   declare ret, qr any;
   qr := sprintf ('DESCRIBE <%s>', iri_spf);
@@ -1501,17 +1473,22 @@ url_rewrite_gen_describe (in graph varchar, in iri_spf varchar)
 }
 ;
 
-create procedure
-url_rewrite_gen_vsp (in graph varchar, in iri_spf varchar)
+create procedure DB.DBA.url_rewrite_gen_vsp (
+  in graph varchar,
+  in iri_spf varchar)
 {
-  declare ret, qr any;
+  declare ret any;
   ret := sprintf ('/describe/?url=%s', iri_spf);
   return ret;
 }
 ;
 
-create procedure
-url_rewrite_from_template (in prefix varchar, in graph varchar, in iri_pattern varchar, in url_pattern varchar, in flags int := 0)
+create procedure DB.DBA.url_rewrite_from_template (
+  in prefix varchar,
+  in graph varchar,
+  in iri_pattern varchar,
+  in url_pattern varchar,
+  in flags int := 0)
 {
   declare arr, h, iri_path, iri_regex, iri_spf, iri_tcn, url_spf, url_regex, url_tcn, iri_param, url_param, iri_vd, url_vd any;
   declare pos, nth, fct int;
@@ -1609,19 +1586,19 @@ url_rewrite_from_template (in prefix varchar, in graph varchar, in iri_pattern v
 		sys_sql_val_print (iri_param),
 		rtrim (replace (replace (url_spf, '{Extension}', ''), '//', '/'), '.'),
 		sys_sql_val_print (url_param),
-		url_rewrite_mime_pattern ()
+		DB.DBA.url_rewrite_mime_pattern ()
 		), ses);
 
   http ('\n', ses);
   http (sprintf ('delete from DB.DBA.HTTP_VARIANT_MAP where VM_RULELIST = \'%s_iri_rule_list\';\n', prefix), ses);
   if (flags and exists (select 1 from VAD.DBA.VAD_REGISTRY where R_KEY like '/VAD/fct/%/resources/dav/%'))
     fct := 1;
-  mime_types := url_rewrite_mime_types ();
+  mime_types := DB.DBA.url_rewrite_mime_types ();
   foreach (any x in mime_types) do
     {
       declare redir varchar;
       if (fct and x[0] = 'html')
-	redir := url_rewrite_gen_vsp (graph, iri_tcn);
+	redir := DB.DBA.url_rewrite_gen_vsp (graph, iri_tcn);
       else
 	redir := replace (url_tcn, '{Extension}', x[0]);
       http (sprintf ('DB.DBA.HTTP_VARIANT_ADD (\'%s_iri_rule_list\', \'%s\', \'%s\', \'%s\', %.2f);\n',
@@ -1648,7 +1625,7 @@ url_rewrite_from_template (in prefix varchar, in graph varchar, in iri_pattern v
   foreach (any x in mime_types) do
     {
       declare redir any;
-      redir := url_rewrite_gen_describe (graph, iri_spf) || replace (sprintf ('&format=%U', x[1]), '%', '%%');
+      redir := DB.DBA.url_rewrite_gen_describe (graph, iri_spf) || replace (sprintf ('&format=%U', x[1]), '%', '%%');
       http (sprintf ('DB.DBA.URLREWRITE_CREATE_REGEX_RULE ( \'%s_url_rule_%d\', 1, \'%s\', %s, 1, \'%s\', %s, null, null, 2, null, \'Content-Type: %s\'); \n',
 		prefix, nth,
 		replace (url_regex, '{Extension}', x[0]),
