@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -309,7 +309,7 @@ dc_set_rb (data_col_t * dc, int inx, uint32 dt_lang, int flags, caddr_t val, cad
   rb->rb_lang = dt_lang & 0xffff;
   rb->rb_type = dt_lang >> 16;
   rb->rb_box = DV_DB_NULL == DV_TYPE_OF (lng) ? val : lng;
-  if (RDF_BOX_GEO_TYPE == rb->rb_type)
+  if (RDF_BOX_GEO_TYPE == rb->rb_type || RB_IS_DURATION (rb->rb_type))
     rb->rb_box = box_deserialize_string (rb->rb_box, INT32_MAX, 0);
   DC_CHECK_LEN (dc, inx);
   DC_FILL_TO (dc, int64, inx);
@@ -317,7 +317,7 @@ dc_set_rb (data_col_t * dc, int inx, uint32 dt_lang, int flags, caddr_t val, cad
   dc->dc_n_values = inx;
   dc_append_rb (dc, (caddr_t) rb);
   dc->dc_n_values = save;
-  if (RDF_BOX_GEO_TYPE == rb->rb_type)
+  if (RDF_BOX_GEO_TYPE == rb->rb_type || RDF_BOX_INTERVAL == rb->rb_type)
     dk_free_box (rb->rb_box);
 }
 
@@ -598,7 +598,6 @@ bif_ro2sq_vec_1 (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args, state_s
 		{
 		  dc->dc_any_null = 1;
 		  dc_no_empty_marks (dc, empty_mark);
-		  bing ();
 		}
 	}
 	}
@@ -814,6 +813,8 @@ rbs_string_range (dtp_t ** buf, int * len, int * is_string)
     *is_string = 0;
 }
 
+#define DT_MODE_MASK(v) ~((DT_TYPE_DATE == DT_DT_TYPE((v))) ? DT_PRINT_MODE_HMS : ((DT_TYPE_TIME == DT_DT_TYPE((v))) ? DT_PRINT_MODE_YMD : 0))
+
 extern int rb_type__xsd_boolean;
 
 void
@@ -865,6 +866,7 @@ bif_str_vec (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args, state_slot_
                     char temp[100];
                     int mode = DT_PRINT_MODE_XML | dt_print_flags_of_rb_type (rb->rb_type);
                     int save = dc->dc_n_values;
+                    mode &= DT_MODE_MASK(rb->rb_box);
                     dc->dc_n_values = set;
                     dt_to_iso8601_string_ext (rb->rb_box, temp, sizeof (temp), mode);
                     dc_append_box (dc, box_dv_short_string (temp));
@@ -938,7 +940,7 @@ bif_iri_to_id_vec (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args, state
   cl_req_group_t * clrg;
   cucurbit_t * cu;
   int is_cl = CL_RUN_CLUSTER == cl_run_local_only;
-  if (1 != n_args)
+  if (1 != n_args || !rdf_rpid64_mode)
     {
       *err_ret = BIF_NOT_VECTORED;
       return;
