@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -291,6 +291,8 @@ User-level object is of type DV_XML_ENTITY. */
 #define DV_WIDESTRINGP(q) \
   (IS_BOX_POINTER (q) && (DV_WIDE == box_tag (q) || DV_LONG_WIDE == box_tag (q)))
 
+#define DV_STRING_TYPE(q) \
+  (IS_BOX_POINTER (q) && ((DV_STRING == box_tag (q)) || (DV_UNAME == box_tag (q)) || (DV_SYMBOL == box_tag (q))))
 
 
 #define DV_LONG_INT_PREC 10
