@@ -6,7 +6,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2018 OpenLink Software
+ *  Copyright (C) 1998-2024 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -26,21 +26,34 @@
 #ifndef _EXE_EXPORT_H
 #define _EXE_EXPORT_H
 
+#ifdef __cplusplus
+#define EXE_EXPORT_LINKAGE "C"
+#else
+#define EXE_EXPORT_LINKAGE
+#endif
+
 #define EXE_EXPORT_TYPED(rettype,name) \
-extern rettype name
+extern EXE_EXPORT_LINKAGE rettype name
 
 #ifdef _USRDLL
 
 #define EXE_EXPORT(rettype,name,arglist) \
 typedef rettype typeof__##name arglist; \
-extern rettype name arglist
+extern EXE_EXPORT_LINKAGE rettype name arglist
 
 #else
 
 #define EXE_EXPORT(rettype,name,arglist) \
 typedef rettype typeof__##name arglist; \
-extern rettype name arglist
+extern EXE_EXPORT_LINKAGE rettype name arglist
 
 #endif
 
+struct _gate_export_item_s { void *_ptr; const char *_name; };
+typedef struct _gate_export_item_s _gate_export_item_t;
+
+extern _gate_export_item_t _gate_export_data[];
+
+extern EXE_EXPORT_LINKAGE int _gate_export (_gate_export_item_t *tgt);
+
 #endif
